@@ -1,48 +1,51 @@
 /****************************************************************************
 **
-** Copyright (C) 2008 Nokia Corporation and/or its subsidiary(-ies).
-** Contact: Qt Software Information (qt-info@nokia.com)
+** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the documentation of the Qt Toolkit.
 **
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial Usage
 ** Licensees holding valid Qt Commercial licenses may use this file in
 ** accordance with the Qt Commercial License Agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and Nokia.
 **
+** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+**
+** In addition, as a special exception, Nokia gives you certain
+** additional rights. These rights are described in the Nokia Qt LGPL
+** Exception version 1.0, included in the file LGPL_EXCEPTION.txt in this
+** package.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License versions 2.0 or 3.0 as published by the Free
-** Software Foundation and appearing in the file LICENSE.GPL included in
-** the packaging of this file.  Please review the following information
-** to ensure GNU General Public Licensing requirements will be met:
-** http://www.fsf.org/licensing/licenses/info/GPLv2.html and
-** http://www.gnu.org/copyleft/gpl.html.  In addition, as a special
-** exception, Nokia gives you certain additional rights. These rights
-** are described in the Nokia Qt GPL Exception version 1.3, included in
-** the file GPL_EXCEPTION.txt in this package.
-**
-** Qt for Windows(R) Licensees
-** As a special exception, Nokia, as the sole copyright holder for Qt
-** Designer, grants users of the Qt/Eclipse Integration plug-in the
-** right for the Qt/Eclipse Integration to link to functionality
-** provided by Qt Designer and its related libraries.
+** General Public License version 3.0 as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU General Public License version 3.0 requirements will be
+** met: http://www.gnu.org/copyleft/gpl.html.
 **
 ** If you are unsure which license is appropriate for your use, please
-** contact the sales department at qt-sales@nokia.com.
+** contact the sales department at http://www.qtsoftware.com/contact.
+** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
 /*
-  
+
   This file exists only to make internal all the classes you see
   below. they are all in the QPatternist namespace, but they don't
-  have qdoc documentation because they are all declared in xxx_p.h
-  files. Without these \internal declarations, the class names
+  have qdoc documentation because they are all declared in xxx_p.h files. Without these \internal declarations, the class names
   appear on the Inheritance Hierarchy page, which is bad because
-  clicking on them brings up the "File not found" page. 
+  clicking on them brings up the "File not found" page.
  */
 
 #include "qitem_p.h"
@@ -242,8 +245,6 @@ using namespace QPatternist;
   \internal */
 /*! \class XPath20CoreFunctions
   \internal */
-/*! \class XSLT10CoreFunctions
-  \internal */
 /*! \class AdjustDateTimeToTimezoneFN
   \internal */
 /*! \class AdjustDateToTimezoneFN
@@ -580,10 +581,6 @@ using namespace QPatternist;
   \internal */
 /*! \class AccelTree
   \internal */
-/*! \class QObjectNodeModel
-  \internal */
-/*! \class QAbstractXmlReceiver
-  \internal */
 /*! \class QNameComparator
   \internal */
 /*! \class QNameComparatorLocator
@@ -594,8 +591,6 @@ using namespace QPatternist;
   \internal */
 /*! \class QNameValue
   \internal */
-/*! \class QObjectPropertyToAttributeIterator
-  \internal */
 /*! \class RemoveFN
   \internal */
 /*! \class ResolveQNameFN
