@@ -1,43 +1,41 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
+** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
-** This file may be used under the terms of the GNU General Public
-** License versions 2.0 or 3.0 as published by the Free Software
-** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file.  Alternatively you may (at
-** your option) use any later version of the GNU General Public
-** License if such license has been publicly approved by Trolltech ASA
-** (or its successors, if any) and the KDE Free Qt Foundation. In
-** addition, as a special exception, Trolltech gives you certain
-** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.2, which can be found at
-** http://www.trolltech.com/products/qt/gplexception/ and in the file
-** GPL_EXCEPTION.txt in this package.
+** $QT_BEGIN_LICENSE:LGPL$
+** Commercial Usage
+** Licensees holding valid Qt Commercial licenses may use this file in
+** accordance with the Qt Commercial License Agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Nokia.
 **
-** Please review the following information to ensure GNU General
-** Public Licensing requirements will be met:
-** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
-** you are unsure which license is appropriate for your use, please
-** review the following information:
-** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
-** or contact the sales department at sales@trolltech.com.
+** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Trolltech, as the sole
-** copyright holder for Qt Designer, grants users of the Qt/Eclipse
-** Integration plug-in the right for the Qt/Eclipse Integration to
-** link to functionality provided by Qt Designer and its related
-** libraries.
+** In addition, as a special exception, Nokia gives you certain
+** additional rights. These rights are described in the Nokia Qt LGPL
+** Exception version 1.0, included in the file LGPL_EXCEPTION.txt in this
+** package.
 **
-** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
-** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
-** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
-** granted herein.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3.0 as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU General Public License version 3.0 requirements will be
+** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
-** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+** If you are unsure which license is appropriate for your use, please
+** contact the sales department at http://www.qtsoftware.com/contact.
+** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
@@ -157,6 +155,31 @@ QT_BEGIN_NAMESPACE
     data, followed by the data. Note that any encoding/decoding of
     the data (apart from the length quint32) must be done by you.
 
+    \target Serializing Qt Classes
+    \section1 Reading and writing other Qt classes.
+
+    In addition to the overloaded stream operators documented here,
+    any Qt classes that you might want to serialize to a QDataStream
+    will have appropriate stream operators declared as non-member of
+    the class:
+
+    \code
+    QDataStream &operator<<(QDataStream &, const QXxx &);
+    QDataStream &operator>>(QDataStream &, QXxx &);
+    \endcode
+
+    For example, here are the stream operators declared as non-members
+    of the QImage class:
+
+    \code
+    QDataStream & operator<< (QDataStream& stream, const QImage& image);
+    QDataStream & operator>> (QDataStream& stream, QImage& image);
+    \endcode
+
+    To see if your favorite Qt class has similar stream operators
+    defined, check the \bold {Related Non-Members} section of the
+    class's documentation page.
+
     \sa QTextStream QVariant
 */
 
@@ -199,10 +222,10 @@ QT_BEGIN_NAMESPACE
 #endif
 
 enum {
-    DefaultStreamVersion = QDataStream::Qt_4_4
+    DefaultStreamVersion = QDataStream::Qt_4_5
 };
 
-// ### 4.0: when streaming invalid QVariants, just the type should
+// ### 5.0: when streaming invalid QVariants, just the type should
 // be written, no "data" after it
 
 /*!
@@ -255,6 +278,9 @@ QDataStream::QDataStream(QIODevice *d)
 QDataStream::QDataStream(QByteArray *a, int mode)
 {
     QBuffer *buf = new QBuffer(a);
+#ifndef QT_NO_QOBJECT
+    buf->blockSignals(true);
+#endif
     buf->open(QIODevice::OpenMode(mode));
     dev = buf;
     owndev = true;
@@ -281,6 +307,9 @@ QDataStream::QDataStream(QByteArray *a, int mode)
 QDataStream::QDataStream(QByteArray *a, QIODevice::OpenMode flags)
 {
     QBuffer *buf = new QBuffer(a);
+#ifndef QT_NO_QOBJECT
+    buf->blockSignals(true);
+#endif
     buf->open(flags);
     dev = buf;
     owndev = true;
@@ -301,6 +330,9 @@ QDataStream::QDataStream(QByteArray *a, QIODevice::OpenMode flags)
 QDataStream::QDataStream(const QByteArray &a)
 {
     QBuffer *buf = new QBuffer;
+#ifndef QT_NO_QOBJECT
+    buf->blockSignals(true);
+#endif
     buf->setData(a);
     buf->open(QIODevice::ReadOnly);
     dev = buf;
@@ -482,6 +514,8 @@ void QDataStream::setByteOrder(ByteOrder bo)
     \value Qt_4_2 Version 8 (Qt 4.2)
     \value Qt_4_3 Version 9 (Qt 4.3)
     \value Qt_4_4 Version 10 (Qt 4.4)
+    \value Qt_4_5 Version 11 (Qt 4.5)
+    \omitvalue Qt_4_6
 
     \sa setVersion(), version()
 */
@@ -588,11 +622,16 @@ QDataStream &QDataStream::operator>>(qint16 &i)
             setStatus(ReadPastEnd);
         }
     } else {
-        register uchar *p = (uchar *)(&i);
+        union {
+            qint16 val1;
+            char val2[2];
+        } x;
+        char *p = x.val2;
         char b[2];
         if (dev->read(b, 2) == 2) {
             *p++ = b[1];
             *p = b[0];
+            i = x.val1;
         } else {
             setStatus(ReadPastEnd);
         }
@@ -626,13 +665,18 @@ QDataStream &QDataStream::operator>>(qint32 &i)
             setStatus(ReadPastEnd);
         }
     } else {                                        // swap bytes
-        uchar *p = (uchar *)(&i);
+        union {
+            qint32 val1;
+            char val2[4];
+        } x;
+        char *p = x.val2;
         char b[4];
         if (dev->read(b, 4) == 4) {
             *p++ = b[3];
             *p++ = b[2];
             *p++ = b[1];
             *p   = b[0];
+            i = x.val1;
         } else {
             setStatus(ReadPastEnd);
         }
@@ -669,7 +713,12 @@ QDataStream &QDataStream::operator>>(qint64 &i)
             setStatus(ReadPastEnd);
         }
     } else {                                        // swap bytes
-        uchar *p = (uchar *)(&i);
+        union {
+            qint64 val1;
+            char val2[8];
+        } x;
+
+        char *p = x.val2;
         char b[8];
         if (dev->read(b, 8) == 8) {
             *p++ = b[7];
@@ -680,6 +729,7 @@ QDataStream &QDataStream::operator>>(qint64 &i)
             *p++ = b[2];
             *p++ = b[1];
             *p   = b[0];
+            i = x.val1;
         } else {
             setStatus(ReadPastEnd);
         }
@@ -717,13 +767,19 @@ QDataStream &QDataStream::operator>>(float &f)
             setStatus(ReadPastEnd);
         }
     } else {                                        // swap bytes
-        uchar *p = (uchar *)(&f);
+        union {
+            float val1;
+            char val2[4];
+        } x;
+
+        char *p = x.val2;
         char b[4];
         if (dev->read(b, 4) == 4) {
             *p++ = b[3];
             *p++ = b[2];
             *p++ = b[1];
             *p = b[0];
+            f = x.val1;
         } else {
             setStatus(ReadPastEnd);
         }
@@ -754,7 +810,11 @@ QDataStream &QDataStream::operator>>(double &f)
             setStatus(ReadPastEnd);
         }
     } else {                                        // swap bytes
-        register uchar *p = (uchar *)(&f);
+        union {
+            double val1;
+            char val2[8];
+        } x;
+        char *p = x.val2;
         char b[8];
         if (dev->read(b, 8) == 8) {
             *p++ = b[7];
@@ -765,13 +825,18 @@ QDataStream &QDataStream::operator>>(double &f)
             *p++ = b[2];
             *p++ = b[1];
             *p   = b[0];
+            f = x.val1;
         } else {
             setStatus(ReadPastEnd);
         }
     }
 #else
     //non-standard floating point format
-    register uchar *p = (uchar *)(&f);
+    union {
+        double val1;
+        char val2[8];
+    } x;
+    char *p = x.val2;
     char b[8];
     if (dev->read(b, 8) == 8) {
         if (noswap) {
@@ -793,6 +858,7 @@ QDataStream &QDataStream::operator>>(double &f)
             *p++ = b[Q_DF(1)];
             *p = b[Q_DF(0)];
         }
+        f = x.val1;
     } else {
         setStatus(ReadPastEnd);
     }
@@ -936,7 +1002,12 @@ QDataStream &QDataStream::operator<<(qint16 i)
     if (noswap) {
         dev->write((char *)&i, sizeof(qint16));
     } else {                                        // swap bytes
-        register uchar *p = (uchar *)(&i);
+        union {
+            qint16 val1;
+            char val2[2];
+        } x;
+        x.val1 = i;
+        char *p = x.val2;
         char b[2];
         b[1] = *p++;
         b[0] = *p;
@@ -958,7 +1029,12 @@ QDataStream &QDataStream::operator<<(qint32 i)
     if (noswap) {
         dev->write((char *)&i, sizeof(qint32));
     } else {                                        // swap bytes
-        register uchar *p = (uchar *)(&i);
+        union {
+            qint32 val1;
+            char val2[4];
+        } x;
+        x.val1 = i;
+        char *p = x.val2;
         char b[4];
         b[3] = *p++;
         b[2] = *p++;
@@ -988,13 +1064,18 @@ QDataStream &QDataStream::operator<<(qint64 i)
 {
     CHECK_STREAM_PRECOND(*this)
     if (version() < 6) {
-	quint32 i1 = i & 0xffffffff;
-	quint32 i2 = i >> 32;
-	*this << i2 << i1;
+        quint32 i1 = i & 0xffffffff;
+        quint32 i2 = i >> 32;
+        *this << i2 << i1;
     } else if (noswap) {                        // no conversion needed
         dev->write((char *)&i, sizeof(qint64));
     } else {                                        // swap bytes
-        register uchar *p = (uchar *)(&i);
+        union {
+            qint64 val1;
+            char val2[8];
+        } x;
+        x.val1 = i;
+        char *p = x.val2;
         char b[8];
         b[7] = *p++;
         b[6] = *p++;
@@ -1043,7 +1124,12 @@ QDataStream &QDataStream::operator<<(float f)
     if (noswap) {                                // no conversion needed
         dev->write((char *)&g, sizeof(float));
     } else {                                // swap bytes
-        register uchar *p = (uchar *)(&g);
+        union {
+            float val1;
+            char val2[4];
+        } x;
+        x.val1 = f;
+        char *p = x.val2;
         char b[4];
         b[3] = *p++;
         b[2] = *p++;
@@ -1069,7 +1155,12 @@ QDataStream &QDataStream::operator<<(double f)
     if (noswap) {
         dev->write((char *)&f, sizeof(double));
     } else {
-        register uchar *p = (uchar *)(&f);
+        union {
+            double val1;
+            char val2[8];
+        } x;
+        x.val1 = f;
+        char *p = x.val2;
         char b[8];
         b[7] = *p++;
         b[6] = *p++;
@@ -1082,7 +1173,12 @@ QDataStream &QDataStream::operator<<(double f)
         dev->write(b, 8);
     }
 #else
-    register uchar *p = (uchar *)(&f);
+    union {
+        double val1;
+        char val2[8];
+    } x;
+    x.val1 = f;
+    char *p = x.val2;
     char b[8];
     if (noswap) {
         b[Q_DF(0)] = *p++;
@@ -1197,8 +1293,10 @@ int QDataStream::skipRawData(int len)
         }
         return sumRead;
     } else {
-        quint64 pos = dev->pos();
-        len = qMin(int(dev->size() - pos), len);
+        qint64 pos = dev->pos();
+        qint64 size = dev->size();
+        if (pos + len > size)
+            len = size - pos;
         if (!dev->seek(pos + len))
             return -1;
         return len;
