@@ -1,43 +1,41 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
+** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
-** This file may be used under the terms of the GNU General Public
-** License versions 2.0 or 3.0 as published by the Free Software
-** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file.  Alternatively you may (at
-** your option) use any later version of the GNU General Public
-** License if such license has been publicly approved by Trolltech ASA
-** (or its successors, if any) and the KDE Free Qt Foundation. In
-** addition, as a special exception, Trolltech gives you certain
-** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.2, which can be found at
-** http://www.trolltech.com/products/qt/gplexception/ and in the file
-** GPL_EXCEPTION.txt in this package.
+** $QT_BEGIN_LICENSE:LGPL$
+** Commercial Usage
+** Licensees holding valid Qt Commercial licenses may use this file in
+** accordance with the Qt Commercial License Agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Nokia.
 **
-** Please review the following information to ensure GNU General
-** Public Licensing requirements will be met:
-** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
-** you are unsure which license is appropriate for your use, please
-** review the following information:
-** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
-** or contact the sales department at sales@trolltech.com.
+** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Trolltech, as the sole
-** copyright holder for Qt Designer, grants users of the Qt/Eclipse
-** Integration plug-in the right for the Qt/Eclipse Integration to
-** link to functionality provided by Qt Designer and its related
-** libraries.
+** In addition, as a special exception, Nokia gives you certain
+** additional rights. These rights are described in the Nokia Qt LGPL
+** Exception version 1.0, included in the file LGPL_EXCEPTION.txt in this
+** package.
 **
-** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
-** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
-** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
-** granted herein.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3.0 as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU General Public License version 3.0 requirements will be
+** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
-** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+** If you are unsure which license is appropriate for your use, please
+** contact the sales department at http://www.qtsoftware.com/contact.
+** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
@@ -58,6 +56,10 @@
 #include <errno.h>
 #endif
 
+#ifdef QT_NO_QOBJECT
+#define tr(X) QString::fromLatin1(X)
+#endif
+
 QT_BEGIN_NAMESPACE
 
 static const int QFILE_WRITEBUFFER_SIZE = 16384;
@@ -323,6 +325,18 @@ QFilePrivate::setError(QFile::FileError err, int errNum)
     platform-dependent: On Unix, the rights of the owner of the file
     are returned and on Windows the rights of the current user are
     returned. This behavior might change in a future Qt version.
+
+    Note that Qt does not by default check for permissions on NTFS
+    file systems, as this may decrease the performance of file
+    handling considerably. It is possible to force permission checking
+    on NTFS by including the following code in your source:
+
+    \snippet doc/src/snippets/ntfsp.cpp 0
+
+    Permission checking is then turned on and off by incrementing and
+    decrementing \c qt_ntfs_permission_lookup by 1.
+
+    \snippet doc/src/snippets/ntfsp.cpp 1
 */
 
 #ifdef QT3_SUPPORT
@@ -642,7 +656,11 @@ QFile::remove()
             unsetError();
             return true;
         }
+#if defined(Q_OS_WIN)
+        d->setError(QFile::RemoveError, GetLastError());
+#else
         d->setError(QFile::RemoveError, errno);
+#endif
     }
     return false;
 }
@@ -687,41 +705,60 @@ QFile::rename(const QString &newName)
         // ### Race condition. If a file is moved in after this, it /will/ be
         // overwritten. On Unix, the proper solution is to use hardlinks:
         // return ::link(old, new) && ::remove(old);
-        d->setError(QFile::RenameError, QLatin1String("Destination file exists"));
+        d->setError(QFile::RenameError, tr("Destination file exists"));
         return false;
     }
     close();
     if(error() == QFile::NoError) {
         if (fileEngine()->rename(newName)) {
             unsetError();
+            // engine was able to handle the new name so we just reset it
+            fileEngine()->setFileName(newName);
+            d->fileName = newName;
             return true;
         }
 
-        QFile in(fileName());
+        if (isSequential()) {
+            d->setError(QFile::RenameError, tr("Will not rename sequential file using block copy"));
+            return false;
+        }
+
         QFile out(newName);
-        if (in.open(QIODevice::ReadOnly)) {
+        if (open(QIODevice::ReadOnly)) {
             if (out.open(QIODevice::WriteOnly | QIODevice::Truncate)) {
                 bool error = false;
                 char block[4096];
-                while (!in.atEnd()) {
-                    qint64 read = in.read(block, sizeof(block));
-                    if (read == -1) {
-                        d->setError(QFile::RenameError, in.errorString());
-                        error = true;
-                        break;
-                    }
-                    if (read != out.write(block, read)) {
+                qint64 bytes;
+                while ((bytes = read(block, sizeof(block))) > 0) {
+                    if (bytes != out.write(block, bytes)) {
                         d->setError(QFile::RenameError, out.errorString());
                         error = true;
                         break;
                     }
                 }
-                if(!error)
-                    in.remove();
+                if (bytes == -1) {
+                    d->setError(QFile::RenameError, errorString());
+                    error = true;
+                }
+                if(!error) {
+                    if (!remove()) {
+                        d->setError(QFile::RenameError, tr("Cannot remove source file"));
+                        error = true;
+                    }
+                }
+                if (error)
+                    out.remove();
+                else {
+                    setPermissions(permissions());
+                    unsetError();
+                    setFileName(newName);
+                }
+                close();
                 return !error;
             }
+            close();
         }
-        d->setError(QFile::RenameError, out.isOpen() ? in.errorString() : out.errorString());
+        d->setError(QFile::RenameError, out.isOpen() ? errorString() : out.errorString());
     }
     return false;
 }
@@ -818,7 +855,7 @@ QFile::copy(const QString &newName)
         // ### Race condition. If a file is moved in after this, it /will/ be
         // overwritten. On Unix, the proper solution is to use hardlinks:
         // return ::link(old, new) && ::remove(old); See also rename().
-        d->setError(QFile::CopyError, QLatin1String("Destination file exists"));
+        d->setError(QFile::CopyError, tr("Destination file exists"));
         return false;
     }
     close();
@@ -830,8 +867,7 @@ QFile::copy(const QString &newName)
             bool error = false;
             if(!open(QFile::ReadOnly)) {
                 error = true;
-                QString errorMessage = QLatin1String("Cannot open %1 for input");
-                d->setError(QFile::CopyError, errorMessage.arg(d->fileName));
+                d->setError(QFile::CopyError, tr("Cannot open %1 for input").arg(d->fileName));
             } else {
                 QString fileTemplate = QLatin1String("%1/qt_temp.XXXXXX");
 #ifdef QT_NO_TEMPORARYFILE
@@ -848,7 +884,7 @@ QFile::copy(const QString &newName)
 #endif
                 if (error) {
                     out.close();
-                    d->setError(QFile::CopyError, QLatin1String("Cannot open for output"));
+                    d->setError(QFile::CopyError, tr("Cannot open for output"));
                 } else {
                     char block[4096];
                     qint64 totalRead = 0;
@@ -858,7 +894,7 @@ QFile::copy(const QString &newName)
                             break;
                         totalRead += in;
                         if(in != out.write(block, in)) {
-                            d->setError(QFile::CopyError, QLatin1String("Failure to write block"));
+                            d->setError(QFile::CopyError, tr("Failure to write block"));
                             error = true;
                             break;
                         }
@@ -871,14 +907,17 @@ QFile::copy(const QString &newName)
                     }
                     if (!error && !out.rename(newName)) {
                         error = true;
-                        QString errorMessage = QLatin1String("Cannot create %1 for output");
-                        d->setError(QFile::CopyError, errorMessage.arg(newName));
+                        d->setError(QFile::CopyError, tr("Cannot create %1 for output").arg(newName));
                     }
-#ifndef QT_NO_TEMPORARYFILE
+#ifdef QT_NO_TEMPORARYFILE
+                    if (error)
+                        out.remove();
+#else
                     if (!error)
                         out.setAutoRemove(false);
 #endif
                 }
+                close();
             }
             if(!error) {
                 QFile::setPermissions(newName, permissions());
@@ -987,7 +1026,7 @@ bool QFile::open(OpenMode mode)
     \bold{Warning:}
     \list 1
         \o If \a fh is \c stdin, \c stdout, or \c stderr, you may not be able
-           to seek(). See QIODevice::isSequentialAccess() for more information.
+           to seek(). See QIODevice::isSequential() for more information.
         \o Since this function opens the file without specifying the file name,
            you cannot use this QFile with a QFileInfo.
     \endlist
@@ -1062,7 +1101,7 @@ bool QFile::open(FILE *fh, OpenMode mode)
     stderr), you may not be able to seek(). size() is set to \c
     LLONG_MAX (in \c <climits>).
 
-    \warning For Windows CE you may not be able to call seek(), setSize(), 
+    \warning For Windows CE you may not be able to call seek(), setSize(),
     fileTime(). size() is set to \c 0.
 
     \warning Since this function opens the file without specifying the file name,
@@ -1336,9 +1375,10 @@ QFile::flush()
 }
 
 /*!
-  \reimp
-*/
+  Flushes the file and then closes it.
 
+  \sa QIODevice::close()
+*/
 void
 QFile::close()
 {
