@@ -1,43 +1,41 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
+** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
-** This file may be used under the terms of the GNU General Public
-** License versions 2.0 or 3.0 as published by the Free Software
-** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file.  Alternatively you may (at
-** your option) use any later version of the GNU General Public
-** License if such license has been publicly approved by Trolltech ASA
-** (or its successors, if any) and the KDE Free Qt Foundation. In
-** addition, as a special exception, Trolltech gives you certain
-** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.2, which can be found at
-** http://www.trolltech.com/products/qt/gplexception/ and in the file
-** GPL_EXCEPTION.txt in this package.
+** $QT_BEGIN_LICENSE:LGPL$
+** Commercial Usage
+** Licensees holding valid Qt Commercial licenses may use this file in
+** accordance with the Qt Commercial License Agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Nokia.
 **
-** Please review the following information to ensure GNU General
-** Public Licensing requirements will be met:
-** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
-** you are unsure which license is appropriate for your use, please
-** review the following information:
-** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
-** or contact the sales department at sales@trolltech.com.
+** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Trolltech, as the sole
-** copyright holder for Qt Designer, grants users of the Qt/Eclipse
-** Integration plug-in the right for the Qt/Eclipse Integration to
-** link to functionality provided by Qt Designer and its related
-** libraries.
+** In addition, as a special exception, Nokia gives you certain
+** additional rights. These rights are described in the Nokia Qt LGPL
+** Exception version 1.0, included in the file LGPL_EXCEPTION.txt in this
+** package.
 **
-** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
-** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
-** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
-** granted herein.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3.0 as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU General Public License version 3.0 requirements will be
+** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
-** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+** If you are unsure which license is appropriate for your use, please
+** contact the sales department at http://www.qtsoftware.com/contact.
+** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
@@ -135,8 +133,7 @@ QFileInfoPrivate::~QFileInfoPrivate()
     data = 0;
 }
 
-void
-QFileInfoPrivate::initFileEngine(const QString &file)
+void QFileInfoPrivate::initFileEngine(const QString &file)
 {
     detach();
     delete data->fileEngine;
@@ -148,7 +145,7 @@ QFileInfoPrivate::initFileEngine(const QString &file)
 
 bool QFileInfoPrivate::hasAccess(Access access) const
 {
-    if (!(data->fileEngine->fileFlags() & QAbstractFileEngine::LocalDiskFlag)) {
+    if (!(getFileFlags(QAbstractFileEngine::FileInfoAll) & QAbstractFileEngine::LocalDiskFlag)) {
         switch (access) {
         case ReadAccess:
             return getFileFlags(QAbstractFileEngine::ReadUserPerm);
@@ -194,10 +191,11 @@ bool QFileInfoPrivate::hasAccess(Access access) const
 }
 
 void QFileInfoPrivate::detach()
-{ qAtomicDetach(data); }
+{
+    qAtomicDetach(data);
+}
 
-QString
-QFileInfoPrivate::getFileName(QAbstractFileEngine::FileName name) const
+QString QFileInfoPrivate::getFileName(QAbstractFileEngine::FileName name) const
 {
     if(data->cache_enabled && data->fileNames.contains((int)name))
         return data->fileNames.value(name);
@@ -207,8 +205,7 @@ QFileInfoPrivate::getFileName(QAbstractFileEngine::FileName name) const
     return ret;
 }
 
-uint
-QFileInfoPrivate::getFileFlags(QAbstractFileEngine::FileFlags request) const
+uint QFileInfoPrivate::getFileFlags(QAbstractFileEngine::FileFlags request) const
 {
     // We split the testing into tests for for LinkType, BundleType and the rest.
     // In order to determine if a file is a symlink or not, we have to lstat().
@@ -257,8 +254,7 @@ QFileInfoPrivate::getFileFlags(QAbstractFileEngine::FileFlags request) const
     return flags & request;
 }
 
-QDateTime
-&QFileInfoPrivate::getFileTime(QAbstractFileEngine::FileTime request) const
+QDateTime &QFileInfoPrivate::getFileTime(QAbstractFileEngine::FileTime request) const
 {
     if (!data->cache_enabled)
         data->clearFlags();
@@ -416,6 +412,9 @@ QFileInfo::QFileInfo(const QFile &file) : d_ptr(new QFileInfoPrivate())
     If \a dir has a relative path, the QFileInfo will also have a
     relative path.
 
+    If \a file is an absolute path, then the directory specified
+    by \a dir will be disregarded.
+
     \sa isRelative()
 */
 
@@ -461,9 +460,7 @@ QFileInfo::~QFileInfo()
 /*!
     \overload
 */
-
-bool
-QFileInfo::operator==(const QFileInfo &fileinfo) const
+bool QFileInfo::operator==(const QFileInfo &fileinfo) const
 {
     Q_D(const QFileInfo);
     // ### Qt 5: understand long and short file names on Windows
@@ -475,8 +472,8 @@ QFileInfo::operator==(const QFileInfo &fileinfo) const
     if(d->data->fileEngine->caseSensitive() != fileinfo.d_func()->data->fileEngine->caseSensitive())
         return false;
     if(fileinfo.size() == size()) { //if the size isn't the same...
-        QString file1 = absoluteFilePath(),
-                file2 = fileinfo.absoluteFilePath();
+        QString file1 = canonicalFilePath(),
+                file2 = fileinfo.canonicalFilePath();
         if(file1.length() == file2.length()) {
             if(!fileinfo.d_func()->data->fileEngine->caseSensitive()) {
                 for(int i = 0; i < file1.length(); i++) {
@@ -538,8 +535,7 @@ QFileInfo &QFileInfo::operator=(const QFileInfo &fileinfo)
     \sa isRelative(), QDir::setCurrent(), QDir::isRelativePath()
 */
 
-void
-QFileInfo::setFile(const QString &file)
+void QFileInfo::setFile(const QString &file)
 {
     Q_D(QFileInfo);
     d->initFileEngine(file);
@@ -557,8 +553,7 @@ QFileInfo::setFile(const QString &file)
     \sa isRelative()
 */
 
-void
-QFileInfo::setFile(const QFile &file)
+void QFileInfo::setFile(const QFile &file)
 {
     Q_D(QFileInfo);
     d->initFileEngine(file.fileName());
@@ -576,35 +571,34 @@ QFileInfo::setFile(const QFile &file)
     \sa isRelative()
 */
 
-void
-QFileInfo::setFile(const QDir &dir, const QString &file)
+void QFileInfo::setFile(const QDir &dir, const QString &file)
 {
     Q_D(QFileInfo);
     d->initFileEngine(dir.filePath(file));
 }
 
 /*!
-    Returns the absolute path including the file name.
+    Returns an absolute path including the file name.
 
     The absolute path name consists of the full path and the file
     name. On Unix this will always begin with the root, '/',
     directory. On Windows this will always begin 'D:/' where D is a
     drive letter, except for network shares that are not mapped to a
     drive letter, in which case the path will begin '//sharename/'.
+    QFileInfo will uppercase drive letters. Note that QDir does not do
+    this. The code snippet below shows this.
+
+    \snippet doc/src/snippets/code/src_corelib_io_qfileinfo.cpp newstuff
 
     This function returns the same as filePath(), unless isRelative()
-    is true.
+    is true. In contrast to canonicalFilePath(), symbolic links or
+    redundant "." or ".." elements are not necessarily removed.
 
     If the QFileInfo is empty it returns QDir::currentPath().
 
-    This function can be time consuming under Unix (in the order of
-    milliseconds).
-
-    \sa isRelative(), filePath()
+    \sa filePath(), canonicalFilePath(), isRelative()
 */
-
-QString
-QFileInfo::absoluteFilePath() const
+QString QFileInfo::absoluteFilePath() const
 {
     Q_D(const QFileInfo);
     if(!d->data->fileEngine)
@@ -613,19 +607,16 @@ QFileInfo::absoluteFilePath() const
 }
 
 /*!
-    Returns the canonical path, i.e. a path without symbolic links or
-    redundant "." or ".." elements.
+  Returns the canonical path including the file name, i.e. an absolute
+  path without symbolic links or redundant "." or ".." elements.
 
-    On systems that do not have symbolic links this function will
-    always return the same string that absoluteFilePath() returns. If
-    the canonical path does not exist (normally due to dangling
-    symbolic links) canonicalFilePath() returns an empty string.
+  If the file does not exist, canonicalFilePath() returns an empty
+  string.
 
-    \sa filePath(), absoluteFilePath()
+  \sa filePath(), absoluteFilePath(), dir()
 */
 
-QString
-QFileInfo::canonicalFilePath() const
+QString QFileInfo::canonicalFilePath() const
 {
     Q_D(const QFileInfo);
     if(!d->data->fileEngine)
@@ -635,17 +626,24 @@ QFileInfo::canonicalFilePath() const
 
 
 /*!
-    Returns the file's path absolute path. This doesn't include the
+    Returns a file's path absolute path. This doesn't include the
     file name.
 
+    On Unix the absolute path will always begin with the root, '/',
+    directory. On Windows this will always begin 'D:/' where D is a
+    drive letter, except for network shares that are not mapped to a
+    drive letter, in which case the path will begin '//sharename/'.
+
+    In contrast to canonicalPath() symbolic links or redundant "." or
+    ".." elements are not necessarily removed.
+
     \warning If the QFileInfo object was created with an empty QString,
               the behavior of this function is undefined.
 
-    \sa dir(), filePath(), fileName(), isRelative(), path()
+    \sa absoluteFilePath(), path(), canonicalPath(), fileName(), isRelative()
 */
 
-QString
-QFileInfo::absolutePath() const
+QString QFileInfo::absolutePath() const
 {
     Q_D(const QFileInfo);
     if(!d->data->fileEngine)
@@ -654,19 +652,15 @@ QFileInfo::absolutePath() const
 }
 
 /*!
-    Returns the canonical path, i.e. a path without symbolic links or
-    redundant "." or ".." elements.
+    Returns the file's path canonical path (excluding the file name),
+    i.e. an absolute path without symbolic links or redundant "." or ".." elements.
 
-    On systems that do not have symbolic links this function will
-    always return the same string that absolutePath() returns. If the
-    canonical path does not exist (normally due to dangling symbolic
-    links) canonicalPath() returns an empty string.
+    If the file does not exist, canonicalPath() returns an empty string.
 
-    \sa absolutePath()
+    \sa path(), absolutePath()
 */
 
-QString
-QFileInfo::canonicalPath() const
+QString QFileInfo::canonicalPath() const
 {
     Q_D(const QFileInfo);
     if(!d->data->fileEngine)
@@ -678,11 +672,14 @@ QFileInfo::canonicalPath() const
 /*!
     Returns the file's path. This doesn't include the file name.
 
-    \sa dir(), filePath(), fileName(), isRelative(), absolutePath()
+    Note that, if this QFileInfo object is given a path ending in a
+    slash, the name of the file is considered empty and this function
+    will return the entire path.
+
+    \sa filePath(), absolutePath(), canonicalPath(), dir(), fileName(), isRelative()
 */
 
-QString
-QFileInfo::path() const
+QString QFileInfo::path() const
 {
     Q_D(const QFileInfo);
     if(!d->data->fileEngine)
@@ -707,8 +704,7 @@ QFileInfo::path() const
     \sa isAbsolute()
 */
 
-bool
-QFileInfo::isRelative() const
+bool QFileInfo::isRelative() const
 {
     Q_D(const QFileInfo);
     if(!d->data->fileEngine)
@@ -725,8 +721,7 @@ QFileInfo::isRelative() const
     \sa filePath(), isRelative()
 */
 
-bool
-QFileInfo::makeAbsolute()
+bool QFileInfo::makeAbsolute()
 {
     Q_D(QFileInfo);
     if(!d->data->fileEngine || !d->data->fileEngine->isRelativePath())
@@ -738,10 +733,12 @@ QFileInfo::makeAbsolute()
 
 /*!
     Returns true if the file exists; otherwise returns false.
+
+    \note If the file is a symlink that points to a non existing
+     file, false is returned.
 */
 
-bool
-QFileInfo::exists() const
+bool QFileInfo::exists() const
 {
     Q_D(const QFileInfo);
     if(!d->data->fileEngine)
@@ -757,8 +754,7 @@ QFileInfo::exists() const
     to detect changes on the file.
 */
 
-void
-QFileInfo::refresh()
+void QFileInfo::refresh()
 {
     Q_D(QFileInfo);
     d->reset();
@@ -768,11 +764,10 @@ QFileInfo::refresh()
     Returns the file name, including the path (which may be absolute
     or relative).
 
-    \sa isRelative(), absoluteFilePath()
+    \sa absoluteFilePath(), canonicalFilePath(), isRelative()
 */
 
-QString
-QFileInfo::filePath() const
+QString QFileInfo::filePath() const
 {
     Q_D(const QFileInfo);
     if(!d->data->fileEngine)
@@ -786,11 +781,13 @@ QFileInfo::filePath() const
     Example:
     \snippet doc/src/snippets/code/src_corelib_io_qfileinfo.cpp 3
 
+    Note that, if this QFileInfo object is given a path ending in a
+    slash, the name of the file is considered empty.
+
     \sa isRelative(), filePath(), baseName(), extension()
 */
 
-QString
-QFileInfo::fileName() const
+QString QFileInfo::fileName() const
 {
     Q_D(const QFileInfo);
     if(!d->data->fileEngine)
@@ -811,8 +808,7 @@ QFileInfo::fileName() const
     \sa isBundle(), filePath(), baseName(), extension()
 */
 
-QString
-QFileInfo::bundleName() const
+QString QFileInfo::bundleName() const
 {
     Q_D(const QFileInfo);
     if(!d->data->fileEngine)
@@ -837,8 +833,7 @@ QFileInfo::bundleName() const
     \sa fileName(), suffix(), completeSuffix(), completeBaseName()
 */
 
-QString
-QFileInfo::baseName() const
+QString QFileInfo::baseName() const
 {
     Q_D(const QFileInfo);
     if(!d->data->fileEngine)
@@ -858,8 +853,7 @@ QFileInfo::baseName() const
     \sa fileName(), suffix(), completeSuffix(), baseName()
 */
 
-QString
-QFileInfo::completeBaseName() const
+QString QFileInfo::completeBaseName() const
 {
     Q_D(const QFileInfo);
     if(!d->data->fileEngine)
@@ -881,8 +875,7 @@ QFileInfo::completeBaseName() const
     \sa fileName(), suffix(), baseName(), completeBaseName()
 */
 
-QString
-QFileInfo::completeSuffix() const
+QString QFileInfo::completeSuffix() const
 {
     Q_D(const QFileInfo);
     if(!d->data->fileEngine)
@@ -910,8 +903,7 @@ QFileInfo::completeSuffix() const
     \sa fileName(), completeSuffix(), baseName(), completeBaseName()
 */
 
-QString
-QFileInfo::suffix() const
+QString QFileInfo::suffix() const
 {
     Q_D(const QFileInfo);
     if(!d->data->fileEngine)
@@ -940,11 +932,10 @@ QFileInfo::suffix() const
 
   \snippet doc/src/snippets/fileinfo/main.cpp 1
 
-  \sa dirPath(), filePath(), fileName(), isRelative(), absoluteDir()
+  \sa absolutePath(), filePath(), fileName(), isRelative(), absoluteDir()
 */
 
-QDir
-QFileInfo::dir() const
+QDir QFileInfo::dir() const
 {
     // ### Qt5: Maybe rename this to parentDirectory(), considering what it actually do?
     return QDir(path());
@@ -953,11 +944,10 @@ QFileInfo::dir() const
 /*!
     Returns the file's absolute path as a QDir object.
 
-    \sa filePath(), fileName(), isRelative(), dir()
+    \sa dir(), filePath(), fileName(), isRelative()
 */
 
-QDir
-QFileInfo::absoluteDir() const
+QDir QFileInfo::absoluteDir() const
 {
     return QDir(absolutePath());
 }
@@ -981,8 +971,7 @@ QDir QFileInfo::dir(bool absPath) const
     \sa isWritable(), isExecutable(), permission()
 */
 
-bool
-QFileInfo::isReadable() const
+bool QFileInfo::isReadable() const
 {
     Q_D(const QFileInfo);
     if(!d->data->fileEngine)
@@ -996,9 +985,7 @@ QFileInfo::isReadable() const
     \sa isReadable(), isExecutable(), permission()
 */
 
-
-bool
-QFileInfo::isWritable() const
+bool QFileInfo::isWritable() const
 {
     Q_D(const QFileInfo);
     if(!d->data->fileEngine)
@@ -1012,8 +999,7 @@ QFileInfo::isWritable() const
     \sa isReadable(), isWritable(), permission()
 */
 
-bool
-QFileInfo::isExecutable() const
+bool QFileInfo::isExecutable() const
 {
     Q_D(const QFileInfo);
     if(!d->data->fileEngine)
@@ -1027,8 +1013,7 @@ QFileInfo::isExecutable() const
     \bold{Note:} This function returns true for the special entries
     "." and ".." on Unix, even though QDir::entryList threats them as shown.
 */
-bool
-QFileInfo::isHidden() const
+bool QFileInfo::isHidden() const
 {
     Q_D(const QFileInfo);
     if(!d->data->fileEngine)
@@ -1044,8 +1029,7 @@ QFileInfo::isHidden() const
     \sa isDir(), isSymLink(), isBundle()
 */
 
-bool
-QFileInfo::isFile() const
+bool QFileInfo::isFile() const
 {
     Q_D(const QFileInfo);
     if(!d->data->fileEngine)
@@ -1060,8 +1044,7 @@ QFileInfo::isFile() const
     \sa isFile(), isSymLink(), isBundle()
 */
 
-bool
-QFileInfo::isDir() const
+bool QFileInfo::isDir() const
 {
     Q_D(const QFileInfo);
     if(!d->data->fileEngine)
@@ -1078,8 +1061,7 @@ QFileInfo::isDir() const
     \sa isDir(), isSymLink(), isFile()
 */
 
-bool
-QFileInfo::isBundle() const
+bool QFileInfo::isBundle() const
 {
     Q_D(const QFileInfo);
     if(!d->data->fileEngine)
@@ -1099,11 +1081,13 @@ QFileInfo::isBundle() const
 
     \snippet doc/src/snippets/code/src_corelib_io_qfileinfo.cpp 9
 
+    \note If the symlink points to a non existing file, exists() returns
+     false.
+
     \sa isFile(), isDir(), symLinkTarget()
 */
 
-bool
-QFileInfo::isSymLink() const
+bool QFileInfo::isSymLink() const
 {
     Q_D(const QFileInfo);
     if(!d->data->fileEngine)
@@ -1117,8 +1101,7 @@ QFileInfo::isSymLink() const
   returns false.
 */
 
-bool
-QFileInfo::isRoot() const
+bool QFileInfo::isRoot() const
 {
     Q_D(const QFileInfo);
     if (!d->data->fileEngine)
@@ -1146,8 +1129,7 @@ QFileInfo::isRoot() const
 
     Use symLinkTarget() instead.
 */
-QString
-QFileInfo::readLink() const
+QString QFileInfo::readLink() const
 {
     Q_D(const QFileInfo);
     if(!d->data->fileEngine)
@@ -1166,8 +1148,7 @@ QFileInfo::readLink() const
     \sa ownerId(), group(), groupId()
 */
 
-QString
-QFileInfo::owner() const
+QString QFileInfo::owner() const
 {
     Q_D(const QFileInfo);
     if(!d->data->fileEngine)
@@ -1184,8 +1165,7 @@ QFileInfo::owner() const
     \sa owner(), group(), groupId()
 */
 
-uint
-QFileInfo::ownerId() const
+uint QFileInfo::ownerId() const
 {
     Q_D(const QFileInfo);
     if(!d->data->fileEngine)
@@ -1204,8 +1184,7 @@ QFileInfo::ownerId() const
     \sa groupId(), owner(), ownerId()
 */
 
-QString
-QFileInfo::group() const
+QString QFileInfo::group() const
 {
     Q_D(const QFileInfo);
     if(!d->data->fileEngine)
@@ -1222,8 +1201,7 @@ QFileInfo::group() const
     \sa group(), owner(), ownerId()
 */
 
-uint
-QFileInfo::groupId() const
+uint QFileInfo::groupId() const
 {
     Q_D(const QFileInfo);
     if(!d->data->fileEngine)
@@ -1245,8 +1223,7 @@ QFileInfo::groupId() const
     \sa isReadable(), isWritable(), isExecutable()
 */
 
-bool
-QFileInfo::permission(QFile::Permissions permissions) const
+bool QFileInfo::permission(QFile::Permissions permissions) const
 {
     Q_D(const QFileInfo);
     if(!d->data->fileEngine)
@@ -1259,8 +1236,7 @@ QFileInfo::permission(QFile::Permissions permissions) const
     QFile::Permissions for the file.
 */
 
-QFile::Permissions
-QFileInfo::permissions() const
+QFile::Permissions QFileInfo::permissions() const
 {
     Q_D(const QFileInfo);
     if(!d->data->fileEngine)
@@ -1276,8 +1252,7 @@ QFileInfo::permissions() const
     \sa exists()
 */
 
-qint64
-QFileInfo::size() const
+qint64 QFileInfo::size() const
 {
     Q_D(const QFileInfo);
     if(!d->data->fileEngine)
@@ -1303,8 +1278,7 @@ QFileInfo::size() const
     \sa lastModified() lastRead()
 */
 
-QDateTime
-QFileInfo::created() const
+QDateTime QFileInfo::created() const
 {
     Q_D(const QFileInfo);
     if(!d->data->fileEngine)
@@ -1318,8 +1292,7 @@ QFileInfo::created() const
     \sa created() lastRead()
 */
 
-QDateTime
-QFileInfo::lastModified() const
+QDateTime QFileInfo::lastModified() const
 {
     Q_D(const QFileInfo);
     if(!d->data->fileEngine)
@@ -1336,8 +1309,7 @@ QFileInfo::lastModified() const
     \sa created() lastModified()
 */
 
-QDateTime
-QFileInfo::lastRead() const
+QDateTime QFileInfo::lastRead() const
 {
     Q_D(const QFileInfo);
     if(!d->data->fileEngine)
@@ -1349,8 +1321,7 @@ QFileInfo::lastRead() const
     Detaches all internal data.
 */
 
-void
-QFileInfo::detach()
+void QFileInfo::detach()
 {
     Q_D(QFileInfo);
     d->detach();
@@ -1381,8 +1352,7 @@ bool QFileInfo::caching() const
     \sa refresh(), caching()
 */
 
-void
-QFileInfo::setCaching(bool enable)
+void QFileInfo::setCaching(bool enable)
 {
     Q_D(QFileInfo);
     detach();
