@@ -1,43 +1,41 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
+** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
-** This file may be used under the terms of the GNU General Public
-** License versions 2.0 or 3.0 as published by the Free Software
-** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file.  Alternatively you may (at
-** your option) use any later version of the GNU General Public
-** License if such license has been publicly approved by Trolltech ASA
-** (or its successors, if any) and the KDE Free Qt Foundation. In
-** addition, as a special exception, Trolltech gives you certain
-** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.2, which can be found at
-** http://www.trolltech.com/products/qt/gplexception/ and in the file
-** GPL_EXCEPTION.txt in this package.
+** $QT_BEGIN_LICENSE:LGPL$
+** Commercial Usage
+** Licensees holding valid Qt Commercial licenses may use this file in
+** accordance with the Qt Commercial License Agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Nokia.
 **
-** Please review the following information to ensure GNU General
-** Public Licensing requirements will be met:
-** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
-** you are unsure which license is appropriate for your use, please
-** review the following information:
-** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
-** or contact the sales department at sales@trolltech.com.
+** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Trolltech, as the sole
-** copyright holder for Qt Designer, grants users of the Qt/Eclipse
-** Integration plug-in the right for the Qt/Eclipse Integration to
-** link to functionality provided by Qt Designer and its related
-** libraries.
+** In addition, as a special exception, Nokia gives you certain
+** additional rights. These rights are described in the Nokia Qt LGPL
+** Exception version 1.0, included in the file LGPL_EXCEPTION.txt in this
+** package.
 **
-** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
-** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
-** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
-** granted herein.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3.0 as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU General Public License version 3.0 requirements will be
+** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
-** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+** If you are unsure which license is appropriate for your use, please
+** contact the sales department at http://www.qtsoftware.com/contact.
+** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
@@ -175,7 +173,7 @@ QIODevicePrivate::~QIODevicePrivate()
     reading; for example, if new data has arrived on the network or if
     additional data is appended to a file that you are reading
     from. You can call bytesAvailable() to determine the number of
-    bytes that currently available for reading. It's common to use
+    bytes that are currently available for reading. It's common to use
     bytesAvailable() together with the readyRead() signal when
     programming with asynchronous devices such as QTcpSocket, where
     fragments of data can arrive at arbitrary points in
@@ -920,7 +918,7 @@ QByteArray QIODevice::read(qint64 maxSize)
         qint64 readBytes = read(buffer, bytesToRead);
         if (readBytes <= 0)
             break;
-        tmp += QByteArray(buffer, (int) readBytes);
+        tmp.append(buffer, (int) readBytes);
         readSoFar += readBytes;
     } while (readSoFar < maxSize && bytesAvailable() > 0);
 
@@ -947,9 +945,9 @@ QByteArray QIODevice::readAll()
 
     QByteArray tmp;
     if (d->isSequential() || size() == 0) {
-        // Read it in chunks, bytesAvailable() is unreliable for sequential
-        // devices.
-        const int chunkSize = 4096;
+        // Read it in chunks. Use bytesAvailable() as an unreliable hint for
+        // sequential devices, but try to read 4K as a minimum.
+        int chunkSize = qMax(qint64(4096), bytesAvailable());
         qint64 totalRead = 0;
         forever {
             tmp.resize(tmp.size() + chunkSize);
@@ -958,6 +956,7 @@ QByteArray QIODevice::readAll()
             if (readBytes <= 0)
                 return tmp;
             totalRead += readBytes;
+            chunkSize = qMax(qint64(4096), bytesAvailable());
         }
     } else {
         // Read it all in one go.
@@ -1185,7 +1184,7 @@ qint64 QIODevice::readLineData(char *data, qint64 maxSize)
     signal.
 
     Subclasses that reimplement this function must call the base
-    implementation in order to include the size of the QIODevice's buffer. Example:
+    implementation in order to include the contents of the QIODevice's buffer. Example:
 
     \snippet doc/src/snippets/code/src_corelib_io_qiodevice.cpp 3
 
@@ -1277,6 +1276,27 @@ qint64 QIODevice::write(const char *data, qint64 maxSize)
     return written;
 }
 
+/*!
+    \since 4.5
+
+    \overload
+ 
+    Writes data from a zero-terminated string of 8-bit characters to the
+    device. Returns the number of bytes that were actually written, or
+    -1 if an error occurred. This is equivalent to
+    \code
+    ...
+    QIODevice::write(data, qstrlen(data));
+    ...
+    \endcode
+
+    \sa read() writeData()
+*/
+qint64 QIODevice::write(const char *data)
+{
+    return write(data, qstrlen(data));
+}
+
 /*! \fn qint64 QIODevice::write(const QByteArray &byteArray)
 
     \overload
