@@ -1,37 +1,41 @@
 /****************************************************************************
 **
-** Copyright (C) 2008 Nokia Corporation and/or its subsidiary(-ies).
-** Contact: Qt Software Information (qt-info@nokia.com)
+** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial Usage
 ** Licensees holding valid Qt Commercial licenses may use this file in
 ** accordance with the Qt Commercial License Agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and Nokia.
 **
+** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+**
+** In addition, as a special exception, Nokia gives you certain
+** additional rights. These rights are described in the Nokia Qt LGPL
+** Exception version 1.0, included in the file LGPL_EXCEPTION.txt in this
+** package.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License versions 2.0 or 3.0 as published by the Free
-** Software Foundation and appearing in the file LICENSE.GPL included in
-** the packaging of this file.  Please review the following information
-** to ensure GNU General Public Licensing requirements will be met:
-** http://www.fsf.org/licensing/licenses/info/GPLv2.html and
-** http://www.gnu.org/copyleft/gpl.html.  In addition, as a special
-** exception, Nokia gives you certain additional rights. These rights
-** are described in the Nokia Qt GPL Exception version 1.3, included in
-** the file GPL_EXCEPTION.txt in this package.
-**
-** Qt for Windows(R) Licensees
-** As a special exception, Nokia, as the sole copyright holder for Qt
-** Designer, grants users of the Qt/Eclipse Integration plug-in the
-** right for the Qt/Eclipse Integration to link to functionality
-** provided by Qt Designer and its related libraries.
+** General Public License version 3.0 as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU General Public License version 3.0 requirements will be
+** met: http://www.gnu.org/copyleft/gpl.html.
 **
 ** If you are unsure which license is appropriate for your use, please
-** contact the sales department at qt-sales@nokia.com.
+** contact the sales department at http://www.qtsoftware.com/contact.
+** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
@@ -46,6 +50,7 @@
 #include <qhash.h>
 
 #include "qthread_p.h"
+#include "private/qcoreapplication_p.h"
 
 /*
 #ifdef Q_OS_WIN32
@@ -82,6 +87,16 @@ QThreadData::~QThreadData()
 {
     Q_ASSERT(_ref == 0);
 
+    // In the odd case that Qt is running on a secondary thread, the main
+    // thread instance will have been dereffed asunder because of the deref in
+    // QThreadData::current() and the deref in the pthread_destroy. To avoid
+    // crashing during QCoreApplicationData's global static cleanup we need to
+    // safeguard the main thread here.. This fix is a bit crude, but it solves
+    // the problem...
+    if (this->thread == QCoreApplicationPrivate::theMainThread) {
+       QCoreApplicationPrivate::theMainThread = 0;
+    }
+
     QThread *t = thread;
     thread = 0;
     delete t;
@@ -265,6 +280,14 @@ void QAdoptedThread::run()
     if the number of processor cores could not be detected.
 */
 
+/*!
+    \fn void QThread::yieldCurrentThread()
+
+    Yields execution of the current thread to another runnable thread,
+    if any. Note that the operating system decides to which thread to
+    switch.
+*/
+
 /*!
     \fn void QThread::start(Priority priority)
 
@@ -274,6 +297,12 @@ void QAdoptedThread::run()
     priority parameter. If the thread is already running, this
     function does nothing.
 
+    The effect of the \a priority parameter is dependent on the
+    operating system's scheduling policy. In particular, the \a priority
+    will be ignored on systems that do not support thread priorities
+    (such as on Linux, see http://linux.die.net/man/2/sched_setscheduler
+    for more details).
+
     \sa run(), terminate()
 */
 
@@ -567,6 +596,12 @@ void QThread::cleanup()
     The \a priority argument can be any value in the \c
     QThread::Priority enum except for \c InheritPriorty.
 
+    The effect of the \a priority parameter is dependent on the
+    operating system's scheduling policy. In particular, the \a priority
+    will be ignored on systems that do not support thread priorities
+    (such as on Linux, see http://linux.die.net/man/2/sched_setscheduler
+    for more details).
+
     \sa Priority priority() start()
 */
 
@@ -685,6 +720,7 @@ QT_BEGIN_INCLUDE_NAMESPACE
 #include <private/qcoreapplication_p.h>
 QT_END_INCLUDE_NAMESPACE
 
+Q_GLOBAL_STATIC_WITH_ARGS(QThreadData, staticThreadData, (0));
 QThread* QThread::instance = 0;
 
 QThread::QThread() : QObject(*new QThreadPrivate, (QObject*)0)
@@ -698,7 +734,7 @@ QThreadData* QThreadData::current()
 {
     if (QThread::instance)
         return QThread::instance->d_func()->data;
-    return 0;
+    return staticThreadData();
 }
 
 #endif // QT_NO_THREAD
