@@ -1,43 +1,41 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
+** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
-** This file may be used under the terms of the GNU General Public
-** License versions 2.0 or 3.0 as published by the Free Software
-** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file.  Alternatively you may (at
-** your option) use any later version of the GNU General Public
-** License if such license has been publicly approved by Trolltech ASA
-** (or its successors, if any) and the KDE Free Qt Foundation. In
-** addition, as a special exception, Trolltech gives you certain
-** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.2, which can be found at
-** http://www.trolltech.com/products/qt/gplexception/ and in the file
-** GPL_EXCEPTION.txt in this package.
+** $QT_BEGIN_LICENSE:LGPL$
+** Commercial Usage
+** Licensees holding valid Qt Commercial licenses may use this file in
+** accordance with the Qt Commercial License Agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Nokia.
 **
-** Please review the following information to ensure GNU General
-** Public Licensing requirements will be met:
-** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
-** you are unsure which license is appropriate for your use, please
-** review the following information:
-** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
-** or contact the sales department at sales@trolltech.com.
+** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Trolltech, as the sole
-** copyright holder for Qt Designer, grants users of the Qt/Eclipse
-** Integration plug-in the right for the Qt/Eclipse Integration to
-** link to functionality provided by Qt Designer and its related
-** libraries.
+** In addition, as a special exception, Nokia gives you certain
+** additional rights. These rights are described in the Nokia Qt LGPL
+** Exception version 1.0, included in the file LGPL_EXCEPTION.txt in this
+** package.
 **
-** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
-** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
-** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
-** granted herein.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3.0 as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU General Public License version 3.0 requirements will be
+** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
-** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+** If you are unsure which license is appropriate for your use, please
+** contact the sales department at http://www.qtsoftware.com/contact.
+** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
@@ -102,13 +100,13 @@ QLinkedListData QLinkedListData::shared_null = {
     from a list one at a time and calls \c delete on them:
     \snippet doc/src/snippets/code/src_corelib_tools_qlinkedlist.cpp 2
 
-    QLinkedList's value type must be an \l{assignable data type}.
-    This covers most data types that are commonly used, but the
-    compiler won't let you, for example, store a QWidget as a value;
-    instead, store a QWidget *. A few functions have additional
-    requirements; for example, contains() and removeAll() expect the
-    value type to support \c operator==(). These requirements are
-    documented on a per-function basis.
+    QLinkedList's value type must be an \l {assignable data type}. This
+    covers most data types that are commonly used, but the compiler
+    won't let you, for example, store a QWidget as a value; instead,
+    store a QWidget *. A few functions have additional requirements;
+    for example, contains() and removeAll() expect the value type to
+    support \c operator==().  These requirements are documented on a
+    per-function basis.
 
     If you want to insert, modify, or remove items in the middle of
     the list, you must use an iterator. QLinkedList provides both
@@ -288,6 +286,24 @@ QLinkedListData QLinkedListData::shared_null = {
     \sa contains()
 */
 
+/*! \fn bool QLinkedList::startsWith(const T &value) const
+    \since 4.5
+
+    Returns true if the list is not empty and its first
+    item is equal to \a value; otherwise returns false.
+
+    \sa isEmpty(), first()
+*/
+
+/*! \fn bool QLinkedList::endsWith(const T &value) const
+    \since 4.5
+
+    Returns true if the list is not empty and its last
+    item is equal to \a value; otherwise returns false.
+
+    \sa isEmpty(), last()
+*/
+
 /*! \fn QLinkedList::iterator QLinkedList::begin()
 
     Returns an \l{STL-style iterator} pointing to the first item in
@@ -636,7 +652,7 @@ QLinkedListData QLinkedListData::shared_null = {
     Constructs an uninitialized iterator.
 
     Functions like operator*() and operator++() should not be called
-    on an uninitialized iterartor. Use operator=() to assign a value
+    on an uninitialized iterator. Use operator=() to assign a value
     to it before using it.
 
     \sa QLinkedList::begin() QLinkedList::end()
@@ -842,7 +858,7 @@ QLinkedListData QLinkedListData::shared_null = {
     Constructs an uninitialized iterator.
 
     Functions like operator*() and operator++() should not be called
-    on an uninitialized iterartor. Use operator=() to assign a value
+    on an uninitialized iterator. Use operator=() to assign a value
     to it before using it.
 
     \sa QLinkedList::constBegin() QLinkedList::constEnd()
