@@ -1,37 +1,41 @@
 /****************************************************************************
 **
-** Copyright (C) 2008 Nokia Corporation and/or its subsidiary(-ies).
-** Contact: Qt Software Information (qt-info@nokia.com)
+** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial Usage
 ** Licensees holding valid Qt Commercial licenses may use this file in
 ** accordance with the Qt Commercial License Agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and Nokia.
 **
+** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+**
+** In addition, as a special exception, Nokia gives you certain
+** additional rights. These rights are described in the Nokia Qt LGPL
+** Exception version 1.0, included in the file LGPL_EXCEPTION.txt in this
+** package.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License versions 2.0 or 3.0 as published by the Free
-** Software Foundation and appearing in the file LICENSE.GPL included in
-** the packaging of this file.  Please review the following information
-** to ensure GNU General Public Licensing requirements will be met:
-** http://www.fsf.org/licensing/licenses/info/GPLv2.html and
-** http://www.gnu.org/copyleft/gpl.html.  In addition, as a special
-** exception, Nokia gives you certain additional rights. These rights
-** are described in the Nokia Qt GPL Exception version 1.3, included in
-** the file GPL_EXCEPTION.txt in this package.
-**
-** Qt for Windows(R) Licensees
-** As a special exception, Nokia, as the sole copyright holder for Qt
-** Designer, grants users of the Qt/Eclipse Integration plug-in the
-** right for the Qt/Eclipse Integration to link to functionality
-** provided by Qt Designer and its related libraries.
+** General Public License version 3.0 as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU General Public License version 3.0 requirements will be
+** met: http://www.gnu.org/copyleft/gpl.html.
 **
 ** If you are unsure which license is appropriate for your use, please
-** contact the sales department at qt-sales@nokia.com.
+** contact the sales department at http://www.qtsoftware.com/contact.
+** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
@@ -398,7 +402,7 @@ void **QListData::erase(void **xi)
     won't let you, for example, store a QWidget as a value; instead,
     store a QWidget *. A few functions have additional requirements;
     for example, indexOf() and lastIndexOf() expect the value type to
-    support \c operator==(). These requirements are documented on a
+    support \c operator==().  These requirements are documented on a
     per-function basis.
 
     Like the other container classes, QList provides \l{Java-style
@@ -435,7 +439,7 @@ void **QListData::erase(void **xi)
 
     Returns a list whose elements are copied from this list,
     starting at position \a pos. If \a length is -1 (the default), all
-    elements after \a pos are copied; otherwise \a length elements (or
+    elements from \a pos are copied; otherwise \a length elements (or
     all remaining elements if there are less than \a length elements)
     are copied.
 */
@@ -580,6 +584,17 @@ void **QListData::erase(void **xi)
     \sa operator<<(), prepend(), insert()
 */
 
+/*! \fn void QList::append(const QList<T> &value)
+
+    \overload
+
+    \since 4.5
+
+    Appends the items of the \a value list to this list.
+
+    \sa operator<<(), operator+=()
+*/
+
 /*! \fn void QList::prepend(const T &value)
 
     Inserts \a value at the beginning of the list.
@@ -749,6 +764,10 @@ void **QListData::erase(void **xi)
     This function requires the value type to have an implementation of
     \c operator==().
 
+    Note that QList uses 0-based indexes, just like C++ arrays. Negative
+    indexes are not supported with the exception of the value mentioned
+    above.
+
     \sa lastIndexOf(), contains()
 */
 
@@ -765,6 +784,10 @@ void **QListData::erase(void **xi)
     This function requires the value type to have an implementation of
     \c operator==().
 
+    Note that QList uses 0-based indexes, just like C++ arrays. Negative 
+    indexes are not supported with the exception of the value mentioned
+    above.
+
     \sa indexOf()
 */
 
@@ -789,6 +812,24 @@ void **QListData::erase(void **xi)
     \sa contains(), indexOf()
 */
 
+/*! \fn bool QList::startsWith(const T &value) const
+    \since 4.5
+
+    Returns true if this list is not empty and its first
+    item is equal to \a value; otherwise returns false.
+
+    \sa isEmpty(), contains()
+*/
+
+/*! \fn bool QList::endsWith(const T &value) const
+    \since 4.5
+
+    Returns true if this list is not empty and its last
+    item is equal to \a value; otherwise returns false.
+
+    \sa isEmpty(), contains()
+*/
+
 /*! \fn QList::iterator QList::begin()
 
     Returns an \l{STL-style iterator} pointing to the first item in
@@ -907,6 +948,14 @@ void **QListData::erase(void **xi)
     same as size().
 */
 
+/*! \fn int QList::length() const
+    \since 4.5
+
+    This function is identical to count().
+
+    \sa count()
+*/
+
 /*! \fn T& QList::first()
 
     Returns a reference to the first item in the list. The list must
@@ -1124,7 +1173,8 @@ void **QListData::erase(void **xi)
 
 /*! \typedef QList::iterator::iterator_category
 
-    \internal
+  A synonym for \e {std::random_access_iterator_tag} indicating
+  this iterator is a random access iterator.
 */
 
 /*! \typedef QList::iterator::difference_type
@@ -1152,7 +1202,7 @@ void **QListData::erase(void **xi)
     Constructs an uninitialized iterator.
 
     Functions like operator*() and operator++() should not be called
-    on an uninitialized iterartor. Use operator=() to assign a value
+    on an uninitialized iterator. Use operator=() to assign a value
     to it before using it.
 
     \sa QList::begin() QList::end()
@@ -1375,7 +1425,7 @@ void **QListData::erase(void **xi)
     Constructs an uninitialized iterator.
 
     Functions like operator*() and operator++() should not be called
-    on an uninitialized iterartor. Use operator=() to assign a value
+    on an uninitialized iterator. Use operator=() to assign a value
     to it before using it.
 
     \sa QList::constBegin() QList::constEnd()
@@ -1383,7 +1433,8 @@ void **QListData::erase(void **xi)
 
 /*! \typedef QList::const_iterator::iterator_category
 
-    \internal
+  A synonym for \e {std::random_access_iterator_tag} indicating
+  this iterator is a random access iterator.
 */
 
 /*! \typedef QList::const_iterator::difference_type
