@@ -1,46 +1,56 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
+** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtDBus module of the Qt Toolkit.
 **
-** This file may be used under the terms of the GNU General Public
-** License versions 2.0 or 3.0 as published by the Free Software
-** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file.  Alternatively you may (at
-** your option) use any later version of the GNU General Public
-** License if such license has been publicly approved by Trolltech ASA
-** (or its successors, if any) and the KDE Free Qt Foundation. In
-** addition, as a special exception, Trolltech gives you certain
-** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.2, which can be found at
-** http://www.trolltech.com/products/qt/gplexception/ and in the file
-** GPL_EXCEPTION.txt in this package.
+** $QT_BEGIN_LICENSE:LGPL$
+** Commercial Usage
+** Licensees holding valid Qt Commercial licenses may use this file in
+** accordance with the Qt Commercial License Agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Nokia.
 **
-** Please review the following information to ensure GNU General
-** Public Licensing requirements will be met:
-** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
-** you are unsure which license is appropriate for your use, please
-** review the following information:
-** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
-** or contact the sales department at sales@trolltech.com.
+** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Trolltech, as the sole
-** copyright holder for Qt Designer, grants users of the Qt/Eclipse
-** Integration plug-in the right for the Qt/Eclipse Integration to
-** link to functionality provided by Qt Designer and its related
-** libraries.
+** In addition, as a special exception, Nokia gives you certain
+** additional rights. These rights are described in the Nokia Qt LGPL
+** Exception version 1.0, included in the file LGPL_EXCEPTION.txt in this
+** package.
 **
-** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
-** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
-** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
-** granted herein.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3.0 as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU General Public License version 3.0 requirements will be
+** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
-** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+** If you are unsure which license is appropriate for your use, please
+** contact the sales department at http://www.qtsoftware.com/contact.
+** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
+//
+//  W A R N I N G
+//  -------------
+//
+// This file is not part of the public API.  This header file may
+// change from version to version without notice, or even be
+// removed.
+//
+// We mean it.
+//
+//
+
 #ifndef QDBUS_SYMBOLS_P_H
 #define QDBUS_SYMBOLS_P_H
 
@@ -142,6 +152,16 @@ DEFINEFUNC(dbus_bool_t        , dbus_connection_set_watch_functions, (DBusConnec
                                                                       void                       *data,
                                                                       DBusFreeFunction            free_data_function),
            (connection, add_function, remove_function, toggled_function, data, free_data_function), return)
+DEFINEFUNC(void              , dbus_connection_set_wakeup_main_function, (DBusConnection             *connection,
+                                                                          DBusWakeupMainFunction      wakeup_main_function,
+                                                                          void                       *data,
+                                                                          DBusFreeFunction            free_data_function),
+           (connection, wakeup_main_function, data, free_data_function), )
+DEFINEFUNC(void              , dbus_connection_set_dispatch_status_function, (DBusConnection             *connection,
+                                                                              DBusDispatchStatusFunction  function,
+                                                                              void                       *data,
+                                                                              DBusFreeFunction            free_data_function),
+           (connection, function, data, free_data_function), )
 
 DEFINEFUNC(void               , dbus_connection_unref, (DBusConnection             *connection),
            (connection), )
@@ -275,6 +295,12 @@ DEFINEFUNC(dbus_bool_t  , dbus_pending_call_set_notify, (DBusPendingCall
                                                          void                          *user_data,
                                                          DBusFreeFunction               free_user_data),
            (pending, function, user_data, free_user_data), return)
+DEFINEFUNC(void         , dbus_pending_call_block, (DBusPendingCall               *pending),
+           (pending), )
+DEFINEFUNC(void         , dbus_pending_call_cancel, (DBusPendingCall               *pending),
+           (pending), )
+DEFINEFUNC(dbus_bool_t  , dbus_pending_call_get_completed, (DBusPendingCall               *pending),
+           (pending), return)
 DEFINEFUNC(DBusMessage* , dbus_pending_call_steal_reply, (DBusPendingCall               *pending),
            (pending), return)
 DEFINEFUNC(void         , dbus_pending_call_unref, (DBusPendingCall               *pending),
