@@ -1,43 +1,41 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
+** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
-** This file may be used under the terms of the GNU General Public
-** License versions 2.0 or 3.0 as published by the Free Software
-** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file.  Alternatively you may (at
-** your option) use any later version of the GNU General Public
-** License if such license has been publicly approved by Trolltech ASA
-** (or its successors, if any) and the KDE Free Qt Foundation. In
-** addition, as a special exception, Trolltech gives you certain
-** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.2, which can be found at
-** http://www.trolltech.com/products/qt/gplexception/ and in the file
-** GPL_EXCEPTION.txt in this package.
+** $QT_BEGIN_LICENSE:LGPL$
+** Commercial Usage
+** Licensees holding valid Qt Commercial licenses may use this file in
+** accordance with the Qt Commercial License Agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Nokia.
 **
-** Please review the following information to ensure GNU General
-** Public Licensing requirements will be met:
-** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
-** you are unsure which license is appropriate for your use, please
-** review the following information:
-** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
-** or contact the sales department at sales@trolltech.com.
+** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Trolltech, as the sole
-** copyright holder for Qt Designer, grants users of the Qt/Eclipse
-** Integration plug-in the right for the Qt/Eclipse Integration to
-** link to functionality provided by Qt Designer and its related
-** libraries.
+** In addition, as a special exception, Nokia gives you certain
+** additional rights. These rights are described in the Nokia Qt LGPL
+** Exception version 1.0, included in the file LGPL_EXCEPTION.txt in this
+** package.
 **
-** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
-** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
-** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
-** granted herein.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3.0 as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU General Public License version 3.0 requirements will be
+** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
-** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+** If you are unsure which license is appropriate for your use, please
+** contact the sales department at http://www.qtsoftware.com/contact.
+** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
@@ -110,7 +108,7 @@ static void normalizeHints(qreal &minimum, qreal &preferred, qreal &maximum, qre
     \internal
 */
 QGraphicsLayoutItemPrivate::QGraphicsLayoutItemPrivate(QGraphicsLayoutItem *par, bool layout)
-    : parent(par), isLayout(layout)
+    : parent(par), isLayout(layout), ownedByLayout(false), graphicsItem(0)
 {
 }
 
@@ -119,7 +117,7 @@ QGraphicsLayoutItemPrivate::QGraphicsLayoutItemPrivate(QGraphicsLayoutItem *par,
 */
 void QGraphicsLayoutItemPrivate::init()
 {
-    sizeHintCacheDirty = true; 
+    sizeHintCacheDirty = true;
     sizePolicy = QSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
 }
 
@@ -171,6 +169,34 @@ QSizeF *QGraphicsLayoutItemPrivate::effectiveSizeHints(const QSizeF &constraint)
     return cachedSizeHints;
 }
 
+
+/*!
+    \internal
+
+    Returns the parent item of this layout, or 0 if this layout is
+    not installed on any widget.
+
+    If this is the item that the layout is installed on, it will return "itself".
+
+    If the layout is a sub-layout, this function returns the parent
+    widget of the parent layout.
+
+    Note that it will traverse up the layout item hierarchy instead of just calling
+    QGraphicsItem::parentItem(). This is on purpose.
+
+    \sa parent()
+*/
+QGraphicsItem *QGraphicsLayoutItemPrivate::parentItem() const
+{
+    Q_Q(const QGraphicsLayoutItem);
+
+    const QGraphicsLayoutItem *parent = q;
+    while (parent && parent->isLayout()) {
+        parent = parent->parentLayoutItem();
+    }
+    return parent ? parent->graphicsItem() : 0;
+}
+
 /*!
     \class QGraphicsLayoutItem
     \brief The QGraphicsLayoutItem class can be inherited to allow your custom
@@ -207,7 +233,7 @@ QSizeF *QGraphicsLayoutItemPrivate::effectiveSizeHints(const QSizeF &constraint)
     position. Calling setGeometry() will always resize and reposition the item
     immediately. Normally, this function is called by QGraphicsLayout after
     the layout has been activated, but it can also be called by the item's user
-    at any time. 
+    at any time.
 
     The sizeHint() function returns the item' minimum, preferred and maximum
     size hints. You can override these properties by calling setMinimumSize(),
@@ -229,7 +255,7 @@ QSizeF *QGraphicsLayoutItemPrivate::effectiveSizeHints(const QSizeF &constraint)
     passing a QGraphicsLayoutItem pointer to QGraphicsLayoutItem's
     protected constructor, or by calling setParentLayoutItem(). The
     parentLayoutItem() function returns a pointer to the item's layoutItem
-    parent. If the item's parent is 0 or if the the parent does not inherit
+    parent. If the item's parent is 0 or if the parent does not inherit
     from QGraphicsItem, the parentLayoutItem() function then returns 0.
     isLayout() returns true if the QGraphicsLayoutItem subclass is itself a
     layout, or false otherwise.
@@ -270,6 +296,17 @@ QGraphicsLayoutItem::QGraphicsLayoutItem(QGraphicsLayoutItemPrivate &dd)
 */
 QGraphicsLayoutItem::~QGraphicsLayoutItem()
 {
+    QGraphicsLayoutItem *parentLI = parentLayoutItem();
+    if (parentLI && parentLI->isLayout()) {
+        QGraphicsLayout *lay = static_cast<QGraphicsLayout*>(parentLI);
+        // this is not optimal
+        for (int i = lay->count() - 1; i >= 0; --i) {
+            if (lay->itemAt(i) == this) {
+                lay->removeAt(i);
+                break;
+            }
+        }
+    }
     delete d_ptr;
 }
 
@@ -323,7 +360,7 @@ void QGraphicsLayoutItem::setSizePolicy(QSizePolicy::Policy hPolicy,
 }
 
 /*!
-    Returns the current size policy. 
+    Returns the current size policy.
 
     \sa setSizePolicy(), QWidget::sizePolicy()
 */
@@ -589,14 +626,15 @@ void QGraphicsLayoutItem::setMaximumHeight(qreal height)
 */
 
 /*!
-    \fn virtual void QGraphicsLayoutItem::setGeometry(const QRectF &rect) = 0
+    \fn virtual void QGraphicsLayoutItem::setGeometry(const QRectF &rect)
 
-    This pure virtual function sets the geometry of the QGraphicsLayoutItem to
+    This virtual function sets the geometry of the QGraphicsLayoutItem to
     \a rect, which is in parent coordinates (e.g., the top-left corner of \a rect
     is equivalent to the item's position in parent coordinates).
 
-    Reimplement this function in a subclass of QGraphicsLayoutItem to enable
-    your item to receive geometry updates.
+    You must reimplement this function in a subclass of QGraphicsLayoutItem to
+    receive geometry updates. The layout will call this function when it does a 
+    rearrangement.
 
     If \a rect is outside of the bounds of minimumSize and maximumSize, it
     will be adjusted to its closest size so that it is within the legal
@@ -707,7 +745,7 @@ QSizeF QGraphicsLayoutItem::effectiveSizeHint(Qt::SizeHint which, const QSizeF &
     \sa effectiveSizeHint()
 */
 void QGraphicsLayoutItem::updateGeometry()
-{ 
+{
     Q_D(QGraphicsLayoutItem);
     d->sizeHintCacheDirty = true;
 }
@@ -731,7 +769,7 @@ QGraphicsLayoutItem *QGraphicsLayoutItem::parentLayoutItem() const
     \sa parentLayoutItem()
 */
 void QGraphicsLayoutItem::setParentLayoutItem(QGraphicsLayoutItem *parent)
-{ 
+{
     d_func()->parent = parent;
 }
 
@@ -747,6 +785,69 @@ bool QGraphicsLayoutItem::isLayout() const
     return d_func()->isLayout;
 }
 
+/*!
+    Returns whether a layout should delete this item in its destructor.
+    If its true, then the layout will delete it. If its false, then it is
+    assumed that another object has the ownership of it, and the layout won't
+    delete this item.
+
+    If the item inherits both QGraphicsItem and QGraphicsLayoutItem (such
+    as QGraphicsWidget does) the item is really part of two ownership
+    hierarchies. This property informs what the layout should do with its
+    child items when it is destructed. In the case of QGraphicsWidget, it
+    is preferred that when the layout is deleted it won't delete its children
+    (since they are also part of the graphics item hierarchy).
+
+    By default this value is initialized to false in QGraphicsLayoutItem,
+    but it is overridden by QGraphicsLayout to return true. This is because
+    QGraphicsLayout is not normally part of the QGraphicsItem hierarchy, so the
+    parent layout should delete it.
+    Subclasses might override this default behaviour by calling
+    setOwnedByLayout(true).
+
+    \sa setOwnedByLayout()
+*/
+bool QGraphicsLayoutItem::ownedByLayout() const
+{
+    return d_func()->ownedByLayout;
+}
+/*!
+    Sets whether a layout should delete this item in its destructor or not.
+    \a ownership must be true to in order for the layout to delete it.
+    \sa ownedByLayout()
+*/
+void QGraphicsLayoutItem::setOwnedByLayout(bool ownership)
+{
+    d_func()->ownedByLayout = ownership;
+}
+
+/*!
+ * Returns the QGraphicsItem that this layout item represents.
+ * For QGraphicsWidget it will return itself. For custom items it can return an
+ * aggregated value.
+ *
+ * \sa setGraphicsItem()
+ */
+QGraphicsItem *QGraphicsLayoutItem::graphicsItem() const
+{
+    return d_func()->graphicsItem;
+}
+
+/*!
+ * If the QGraphicsLayoutItem represents a QGraphicsItem, and it wants to take
+ * advantage of the automatic reparenting capabilities of QGraphicsLayout it
+ * should set this value.
+ * Note that if you delete \a item and not delete the layout item, you are
+ * responsible of calling setGraphicsItem(0) in order to avoid having a
+ * dangling pointer.
+ *
+ * \sa graphicsItem()
+ */
+void QGraphicsLayoutItem::setGraphicsItem(QGraphicsItem *item)
+{
+    d_func()->graphicsItem = item;
+}
+
 QT_END_NAMESPACE
-        
+
 #endif //QT_NO_GRAPHICSVIEW
