@@ -1,43 +1,41 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
+** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
-** This file may be used under the terms of the GNU General Public
-** License versions 2.0 or 3.0 as published by the Free Software
-** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file.  Alternatively you may (at
-** your option) use any later version of the GNU General Public
-** License if such license has been publicly approved by Trolltech ASA
-** (or its successors, if any) and the KDE Free Qt Foundation. In
-** addition, as a special exception, Trolltech gives you certain
-** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.2, which can be found at
-** http://www.trolltech.com/products/qt/gplexception/ and in the file
-** GPL_EXCEPTION.txt in this package.
+** $QT_BEGIN_LICENSE:LGPL$
+** Commercial Usage
+** Licensees holding valid Qt Commercial licenses may use this file in
+** accordance with the Qt Commercial License Agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Nokia.
 **
-** Please review the following information to ensure GNU General
-** Public Licensing requirements will be met:
-** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
-** you are unsure which license is appropriate for your use, please
-** review the following information:
-** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
-** or contact the sales department at sales@trolltech.com.
+** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Trolltech, as the sole
-** copyright holder for Qt Designer, grants users of the Qt/Eclipse
-** Integration plug-in the right for the Qt/Eclipse Integration to
-** link to functionality provided by Qt Designer and its related
-** libraries.
+** In addition, as a special exception, Nokia gives you certain
+** additional rights. These rights are described in the Nokia Qt LGPL
+** Exception version 1.0, included in the file LGPL_EXCEPTION.txt in this
+** package.
 **
-** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
-** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
-** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
-** granted herein.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3.0 as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU General Public License version 3.0 requirements will be
+** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
-** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+** If you are unsure which license is appropriate for your use, please
+** contact the sales department at http://www.qtsoftware.com/contact.
+** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
@@ -61,12 +59,13 @@
     \snippet doc/src/snippets/code/src_gui_graphicsview_qgraphicslinearlayout.cpp 0
 
     You can add widgets, layouts, stretches (addStretch(), insertStretch() or
-    setStretchFactor()), and gaps (addGap() and setItemSpacing()) to a linear
-    layout. The layout takes ownership of the items, with one natural
-    exception: when a layout is assigned to a widget, all widgets inside the
-    layout are automatically reparented to the widget, effectively signing
-    over ownership from the layout to the layout-managed widget. You can
-    access each item in the layout by calling count() and itemAt(). Calling
+    setStretchFactor()), and spacings (setItemSpacing()) to a linear
+    layout. The layout takes ownership of the items. In some cases when the layout 
+    item also inherits from QGraphicsItem (such as QGraphicsWidget) there will be a
+    ambiguity in ownership because the layout item belongs to two ownership hierarchies.
+	See the documentation of QGraphicsLayoutItem::setOwnedByLayout() how to handle
+	this.
+	You can access each item in the layout by calling count() and itemAt(). Calling
     removeAt() or removeItem() will remove an item from the layout, without
     destroying it.
 
@@ -178,8 +177,8 @@ QLayoutStyleInfo QGraphicsLinearLayoutPrivate::styleInfo() const
     static QWidget *wid = 0;
     if (!wid)
         wid = new QWidget;
-    QGraphicsWidget *w = parentWidget();
-    QStyle *style = w ? w->style() : qApp->style();
+    QGraphicsItem *item = parentItem();
+    QStyle *style = (item && item->isWidget()) ? static_cast<QGraphicsWidget*>(item)->style() : qApp->style();
     return QLayoutStyleInfo(style, wid);
 }
 
@@ -207,16 +206,25 @@ QGraphicsLinearLayout::QGraphicsLinearLayout(QGraphicsLayoutItem *parent)
 */
 QGraphicsLinearLayout::~QGraphicsLinearLayout()
 {
+    for (int i = count() - 1; i >= 0; --i) {
+        QGraphicsLayoutItem *item = itemAt(i);
+        // The following lines can be removed, but this removes the item
+        // from the layout more efficiently than the implementation of 
+        // ~QGraphicsLayoutItem.
+        removeAt(i);
+        if (item) {
+            item->setParentLayoutItem(0);
+            if (item->ownedByLayout())
+                delete item;
+        }
+    }
 }
 
 /*!
-    \property QGraphicsLinearLayout::orientation
-    \brief the orientation of the linear layout
+  Change the layout orientation to \a orientation. Changing the layout
+  orientation will automatically invalidate the layout.
 
-    This property toggles the layout orientation. Changing the layout
-    orientation will automatically invalidate the layout.
-
-    \sa invalidate()
+  \sa orientation()
 */
 void QGraphicsLinearLayout::setOrientation(Qt::Orientation orientation)
 {
@@ -227,6 +235,11 @@ void QGraphicsLinearLayout::setOrientation(Qt::Orientation orientation)
         invalidate();
     }
 }
+
+/*!
+  Returns the layout orientation.
+  \sa setOrientation()
+ */
 Qt::Orientation QGraphicsLinearLayout::orientation() const
 {
     Q_D(const QGraphicsLinearLayout);
@@ -260,10 +273,7 @@ void QGraphicsLinearLayout::insertItem(int index, QGraphicsLayoutItem *item)
         qWarning("QGraphicsLinearLayout::insertItem: cannot insert null item");
         return;
     }
-    if (item->isLayout())
-        d->addChildLayout(static_cast<QGraphicsLayout *>(item));
-    else
-        d->addChildWidget(static_cast<QGraphicsWidget *>(item));
+    d->addChildLayoutItem(item);
 
     Q_ASSERT(item);
     d->fixIndex(&index);
@@ -297,6 +307,7 @@ void QGraphicsLinearLayout::removeItem(QGraphicsLayoutItem *item)
 {
     Q_D(QGraphicsLinearLayout);
     if (QGridLayoutItem *gridItem = d->engine.findLayoutItem(item)) {
+        item->setParentLayoutItem(0);
         d->removeGridItem(gridItem);
         delete gridItem;
         invalidate();
@@ -313,6 +324,8 @@ void QGraphicsLinearLayout::removeAt(int index)
 {
     Q_D(QGraphicsLinearLayout);
     if (QGridLayoutItem *gridItem = d->engine.itemAt(d->gridRow(index), d->gridColumn(index))) {
+        if (QGraphicsLayoutItem *layoutItem = gridItem->layoutItem())
+            layoutItem->setParentLayoutItem(0);
         d->removeGridItem(gridItem);
         delete gridItem;
         invalidate();
@@ -320,13 +333,10 @@ void QGraphicsLinearLayout::removeAt(int index)
 }
 
 /*!
-    \property QGraphicsLinearLayout::spacing
-    \brief the layout's default spacing
+  Sets the layout's spacing to \a spacing. Spacing refers to the
+  vertical and horizontal distances between items.
 
-    This property adjusts the layout's default spacing. The spacing applies to
-    the vertical and horizontal distance between items.
-
-    \sa setItemSpacing(), setStretchFactor(), QGraphicsGridLayout::setSpacing()
+   \sa setItemSpacing(), setStretchFactor(), QGraphicsGridLayout::setSpacing()
 */
 void QGraphicsLinearLayout::setSpacing(qreal spacing)
 {
@@ -339,6 +349,12 @@ void QGraphicsLinearLayout::setSpacing(qreal spacing)
     invalidate();
 }
 
+/*!
+  Returns the layout's spacing. Spacing refers to the
+  vertical and horizontal distances between items.
+
+  \sa setSpacing()
+ */
 qreal QGraphicsLinearLayout::spacing() const
 {
     Q_D(const QGraphicsLinearLayout);
@@ -472,6 +488,10 @@ void QGraphicsLinearLayout::setGeometry(const QRectF &rect)
     QRectF effectiveRect = geometry();
     qreal left, top, right, bottom;
     getContentsMargins(&left, &top, &right, &bottom);
+    Qt::LayoutDirection visualDir = d->visualDirection();
+    d->engine.setVisualDirection(visualDir);
+    if (visualDir == Qt::RightToLeft)
+        qSwap(left, right);
     effectiveRect.adjust(+left, +top, -right, -bottom);
 #ifdef QT_DEBUG
     if (qt_graphicsLayoutDebug()) {
