@@ -1,43 +1,41 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
+** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
-** This file may be used under the terms of the GNU General Public
-** License versions 2.0 or 3.0 as published by the Free Software
-** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file.  Alternatively you may (at
-** your option) use any later version of the GNU General Public
-** License if such license has been publicly approved by Trolltech ASA
-** (or its successors, if any) and the KDE Free Qt Foundation. In
-** addition, as a special exception, Trolltech gives you certain
-** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.2, which can be found at
-** http://www.trolltech.com/products/qt/gplexception/ and in the file
-** GPL_EXCEPTION.txt in this package.
+** $QT_BEGIN_LICENSE:LGPL$
+** Commercial Usage
+** Licensees holding valid Qt Commercial licenses may use this file in
+** accordance with the Qt Commercial License Agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Nokia.
 **
-** Please review the following information to ensure GNU General
-** Public Licensing requirements will be met:
-** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
-** you are unsure which license is appropriate for your use, please
-** review the following information:
-** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
-** or contact the sales department at sales@trolltech.com.
+** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Trolltech, as the sole
-** copyright holder for Qt Designer, grants users of the Qt/Eclipse
-** Integration plug-in the right for the Qt/Eclipse Integration to
-** link to functionality provided by Qt Designer and its related
-** libraries.
+** In addition, as a special exception, Nokia gives you certain
+** additional rights. These rights are described in the Nokia Qt LGPL
+** Exception version 1.0, included in the file LGPL_EXCEPTION.txt in this
+** package.
 **
-** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
-** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
-** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
-** granted herein.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3.0 as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU General Public License version 3.0 requirements will be
+** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
-** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+** If you are unsure which license is appropriate for your use, please
+** contact the sales department at http://www.qtsoftware.com/contact.
+** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
@@ -80,6 +78,11 @@
     formats, in addition to any image format plugins that support
     reading.
 
+    QImageReader autodetects the image format by default, by looking at the
+    provided (optional) format string, the file name suffix, and the data
+    stream contents. You can enable or disable this feature, by calling
+    setAutoDetectImageFormat().
+
     \sa QImageWriter, QImageIOHandler, QImageIOPlugin
 */
 
@@ -192,8 +195,11 @@ static const _qt_BuiltInFormatStruct _qt_BuiltInFormats[] = {
 };
 
 static QImageIOHandler *createReadHandlerHelper(QIODevice *device,
-    const QByteArray &format)
+                                                const QByteArray &format, bool autoDetectImageFormat)
 {
+    if (!autoDetectImageFormat && format.isEmpty())
+        return 0;
+
     QByteArray form = format.toLower();
     QImageIOHandler *handler = 0;
 
@@ -211,7 +217,7 @@ static QImageIOHandler *createReadHandlerHelper(QIODevice *device,
 
 #if !defined (QT_NO_LIBRARY) && !defined(QT_NO_SETTINGS)
     int suffixPluginIndex = -1;
-    if (device && format.isEmpty()) {
+    if (device && format.isEmpty() && autoDetectImageFormat) {
         // if there's no format, see if \a device is a file, and if so, find
         // the file suffix and find support for that format among our plugins.
         // this allows plugins to override our built-in handlers.
@@ -252,7 +258,7 @@ static QImageIOHandler *createReadHandlerHelper(QIODevice *device,
             device->seek(pos);
     }
 
-    if (!handler && !testFormat.isEmpty()) {
+    if (!handler && !testFormat.isEmpty() && autoDetectImageFormat) {
         // check if any plugin supports the format (they are not allowed to
         // read from the device yet).
         const qint64 pos = device ? device->pos() : 0;
@@ -309,7 +315,7 @@ static QImageIOHandler *createReadHandlerHelper(QIODevice *device,
     }
 
 #if !defined (QT_NO_LIBRARY) && !defined(QT_NO_SETTINGS)
-    if (!handler) {
+    if (!handler && autoDetectImageFormat) {
         // check if any of our plugins recognize the file from its contents.
         const qint64 pos = device ? device->pos() : 0;
         for (int i = 0; i < keys.size(); ++i) {
@@ -329,7 +335,7 @@ static QImageIOHandler *createReadHandlerHelper(QIODevice *device,
     }
 #endif
 
-    if (!handler) {
+    if (!handler && autoDetectImageFormat) {
         // check if any of our built-in handlers recognize the file from its
         // contents.
         int currentFormat = 0;
@@ -427,6 +433,7 @@ public:
 
     // device
     QByteArray format;
+    bool autoDetectImageFormat;
     QIODevice *device;
     bool deleteDevice;
     QImageIOHandler *handler;
@@ -451,6 +458,7 @@ public:
     \internal
 */
 QImageReaderPrivate::QImageReaderPrivate(QImageReader *qq)
+    : autoDetectImageFormat(true)
 {
     device = 0;
     deleteDevice = false;
@@ -485,7 +493,7 @@ bool QImageReaderPrivate::initHandler()
     }
 
     // probe the file extension
-    if (deleteDevice && !device->isOpen() && !device->open(QIODevice::ReadOnly)) {
+    if (deleteDevice && !device->isOpen() && !device->open(QIODevice::ReadOnly) && autoDetectImageFormat) {
         QList<QByteArray> extensions = QImageReader::supportedImageFormats();
         if (!format.isEmpty()) {
             // Try the most probable extension first
@@ -508,12 +516,13 @@ bool QImageReaderPrivate::initHandler()
         if (!device->isOpen()) {
             imageReaderError = QImageReader::FileNotFoundError;
             errorString = QLatin1String(QT_TRANSLATE_NOOP(QImageReader, "File not found"));
+            file->setFileName(fileName); // restore the old file name
             return false;
         }
     }
 
     // assign a handler
-    if (!handler && (handler = createReadHandlerHelper(device, format)) == 0) {
+    if (!handler && (handler = createReadHandlerHelper(device, format, autoDetectImageFormat)) == 0) {
         imageReaderError = QImageReader::UnsupportedFormatError;
         errorString = QLatin1String(QT_TRANSLATE_NOOP(QImageReader, "Unsupported image format"));
         return false;
@@ -624,6 +633,61 @@ QByteArray QImageReader::format() const
     return d->format;
 }
 
+/*!
+    If \a enabled is true, image format autodetection is enabled; otherwise,
+    it is disabled. By default, autodetection is enabled.
+
+    QImageReader uses an extensive approach to detecting the image format;
+    firstly, if you pass a file name to QImageReader, it will attempt to
+    detect the file extension if the given file name does not point to an
+    existing file, by appending supported default extensions to the given file
+    name, one at a time. It then uses the following approach to detect the
+    image format:
+
+    \list
+
+    \o Image plugins are queried first, based on either the optional format
+    string, or the file name suffix (if the source device is a file). No
+    content detection is done at this stage. QImageReader will choose the
+    first plugin that supports reading for this format.
+
+    \o If no plugin supports the image format, Qt's built-in handlers are
+    checked based on either the optional format string, or the file name
+    suffix.
+
+    \o If no capable plugins or built-in handlers are found, each plugin is
+    tested by inspecting the content of the data stream.
+
+    \o If no plugins could detect the image format based on data contents,
+    each built-in image handler is tested by inspecting the contents.
+
+    \o Finally, if all above approaches fail, QImageReader will report failure
+    when trying to read the image.
+
+    \endlist    
+
+    By disabling image format autodetection, QImageReader will only query the
+    plugins and built-in handlers based on the format string (i.e., no file
+    name extensions are tested).
+
+    \sa QImageIOHandler::canRead(), QImageIOPlugin::capabilities()
+*/
+void QImageReader::setAutoDetectImageFormat(bool enabled)
+{
+    d->autoDetectImageFormat = enabled;
+}
+
+/*!
+    Returns true if image format autodetection is enabled on this image
+    reader; otherwise returns false. By default, autodetection is enabled.
+
+    \sa setAutoDetectImageFormat()    
+*/
+bool QImageReader::autoDetectImageFormat() const
+{
+    return d->autoDetectImageFormat;
+}
+
 /*!
     Sets QImageReader's device to \a device. If a device has already
     been set, the old device is removed from QImageReader and is
@@ -741,6 +805,29 @@ QSize QImageReader::size() const
     return QSize();
 }
 
+/*!
+    \since 4.5
+
+    Returns the format of the image, without actually reading the image
+    contents. The format describes the image format \l QImageReader::read()
+    returns, not the format of the actual image.
+
+    If the image format does not support this feature, this function returns
+    an invalid format.
+
+    \sa QImageIOHandler::ImageOption, QImageIOHandler::option(), QImageIOHandler::supportsOption()
+*/
+QImage::Format QImageReader::imageFormat() const
+{
+    if (!d->initHandler())
+        return QImage::Format_Invalid;
+
+    if (d->handler->supportsOption(QImageIOHandler::ImageFormat))
+        return (QImage::Format)d->handler->option(QImageIOHandler::ImageFormat).toInt();
+
+    return QImage::Format_Invalid;
+}
+
 /*!
     \since 4.1
 
@@ -1216,11 +1303,13 @@ QByteArray QImageReader::imageFormat(const QString &fileName)
 /*!
     If supported, this function returns the image format of the device
     \a device. Otherwise, an empty string is returned.
+
+    \sa QImageReader::autoDetectImageFormat()
 */
 QByteArray QImageReader::imageFormat(QIODevice *device)
 {
     QByteArray format;
-    QImageIOHandler *handler = createReadHandlerHelper(device, format);
+    QImageIOHandler *handler = createReadHandlerHelper(device, format, /* autoDetectImageFormat = */ true);
     if (handler) {
         if (handler->canRead())
             format = handler->format();
@@ -1248,6 +1337,7 @@ QByteArray QImageReader::imageFormat(QIODevice *device)
     \row    \o TIFF   \o Tagged Image File Format
     \row    \o XBM    \o X11 Bitmap
     \row    \o XPM    \o X11 Pixmap
+    \row    \o SVG    \o Scalable Vector Graphics
     \endtable
 
     Reading and writing SVG files is supported through Qt's
