@@ -1,48 +1,47 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
+** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
-** This file may be used under the terms of the GNU General Public
-** License versions 2.0 or 3.0 as published by the Free Software
-** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file.  Alternatively you may (at
-** your option) use any later version of the GNU General Public
-** License if such license has been publicly approved by Trolltech ASA
-** (or its successors, if any) and the KDE Free Qt Foundation. In
-** addition, as a special exception, Trolltech gives you certain
-** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.2, which can be found at
-** http://www.trolltech.com/products/qt/gplexception/ and in the file
-** GPL_EXCEPTION.txt in this package.
+** $QT_BEGIN_LICENSE:LGPL$
+** Commercial Usage
+** Licensees holding valid Qt Commercial licenses may use this file in
+** accordance with the Qt Commercial License Agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Nokia.
 **
-** Please review the following information to ensure GNU General
-** Public Licensing requirements will be met:
-** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
-** you are unsure which license is appropriate for your use, please
-** review the following information:
-** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
-** or contact the sales department at sales@trolltech.com.
+** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Trolltech, as the sole
-** copyright holder for Qt Designer, grants users of the Qt/Eclipse
-** Integration plug-in the right for the Qt/Eclipse Integration to
-** link to functionality provided by Qt Designer and its related
-** libraries.
+** In addition, as a special exception, Nokia gives you certain
+** additional rights. These rights are described in the Nokia Qt LGPL
+** Exception version 1.0, included in the file LGPL_EXCEPTION.txt in this
+** package.
 **
-** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
-** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
-** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
-** granted herein.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3.0 as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU General Public License version 3.0 requirements will be
+** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
-** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+** If you are unsure which license is appropriate for your use, please
+** contact the sales department at http://www.qtsoftware.com/contact.
+** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
 #include <qplatformdefs.h>
 #include "qitemeditorfactory.h"
+#include "qitemeditorfactory_p.h"
 
 #ifndef QT_NO_ITEMVIEWS
 
@@ -53,29 +52,11 @@
 #include <qspinbox.h>
 #include <limits.h>
 #include <float.h>
-#include <qcoreapplication.h>
+#include <qapplication.h>
 #include <qdebug.h>
 
 QT_BEGIN_NAMESPACE
 
-#ifndef QT_NO_LINEEDIT
-
-class QExpandingLineEdit : public QLineEdit
-{
-    Q_OBJECT
-
-public:
-    QExpandingLineEdit(QWidget *parent);
-    QExpandingLineEdit(const QString &contents, QWidget *parent);
-
-public Q_SLOTS:
-    void resizeToContents();
-
-private:
-    int originalWidth;
-};
-
-#endif // QT_NO_LINEEDIT
 
 #ifndef QT_NO_COMBOBOX
 
@@ -132,7 +113,7 @@ public:
     \row    \o QTime \o QTimeEdit
     \endtable
 
-    Additional editors can be registered with the registerEditor() function. 
+    Additional editors can be registered with the registerEditor() function.
 
     \sa QItemDelegate, {Model/View Programming}, {Color Editor Factory Example}
 */
@@ -251,8 +232,10 @@ QWidget *QDefaultItemEditorFactory::createEditor(QVariant::Type type, QWidget *p
     case QVariant::String:
     default: {
         // the default editor is a lineedit
-        QLineEdit *le = new QExpandingLineEdit(parent);
-        le->setFrame(false);
+        QExpandingLineEdit *le = new QExpandingLineEdit(parent);
+        le->setFrame(le->style()->styleHint(QStyle::SH_ItemView_DrawDelegateFrame, 0, le));
+        if (!le->style()->styleHint(QStyle::SH_ItemView_ShowDecorationSelected, 0, le))
+            le->setWidgetOwnsGeometry(true);
         return le; }
 #else
     default:
@@ -494,32 +477,60 @@ void QItemEditorFactory::setDefaultFactory(QItemEditorFactory *factory)
 #ifndef QT_NO_LINEEDIT
 
 QExpandingLineEdit::QExpandingLineEdit(QWidget *parent)
-    : QLineEdit(parent), originalWidth(-1)
+    : QLineEdit(parent), originalWidth(-1), widgetOwnsGeometry(false)
 {
     connect(this, SIGNAL(textChanged(QString)), this, SLOT(resizeToContents()));
+    updateMinimumWidth();
 }
 
-QExpandingLineEdit::QExpandingLineEdit(const QString &contents, QWidget *parent)
-    : QLineEdit(contents, parent), originalWidth(-1)
+void QExpandingLineEdit::changeEvent(QEvent *e)
 {
-    connect(this, SIGNAL(textChanged(QString)), this, SLOT(resizeToContents()));
+    switch (e->type())
+    {
+    case QEvent::FontChange:
+    case QEvent::StyleChange:
+    case QEvent::ContentsRectChange:
+        updateMinimumWidth();
+        break;
+    default:
+        break;
+    }
+
+    QLineEdit::changeEvent(e);
+}
+
+void QExpandingLineEdit::updateMinimumWidth()
+{
+    int left, right;
+    getTextMargins(&left, 0, &right, 0);
+    int width = left + right + 4 /*horizontalMargin in qlineedit.cpp*/;
+    getContentsMargins(&left, 0, &right, 0);
+    width += left + right;
+
+    QStyleOptionFrameV2 opt;
+    initStyleOption(&opt);
+    
+    int minWidth = style()->sizeFromContents(QStyle::CT_LineEdit, &opt, QSize(width, 0).
+                                      expandedTo(QApplication::globalStrut()), this).width();
+    setMinimumWidth(minWidth);
 }
 
 void QExpandingLineEdit::resizeToContents()
 {
+    int oldWidth = width();
     if (originalWidth == -1)
-        originalWidth = width();
+        originalWidth = oldWidth;
     if (QWidget *parent = parentWidget()) {
         QPoint position = pos();
-	QFontMetrics fm(font());
-	int hintWidth = sizeHint().width() - (fm.width(QLatin1Char('x')) * 17) + fm.width(displayText());
+        int hintWidth = minimumWidth() + fontMetrics().width(displayText());
         int parentWidth = parent->width();
-	int maxWidth = isRightToLeft() ? position.x() + width() : parentWidth - position.x();
-	int newWidth = qBound(originalWidth, hintWidth, maxWidth);
-	if (isRightToLeft())
-	    setGeometry(position.x() - newWidth + width(), position.y(), newWidth, height());
-	else
-	    resize(newWidth, height());
+        int maxWidth = isRightToLeft() ? position.x() + oldWidth : parentWidth - position.x();
+        int newWidth = qBound(originalWidth, hintWidth, maxWidth);
+        if (widgetOwnsGeometry)
+            setMaximumWidth(newWidth);
+        if (isRightToLeft())
+            move(position.x() - newWidth + oldWidth, position.y());
+        resize(newWidth, height());
     }
 }
 
