@@ -1,43 +1,41 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
+** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
-** This file may be used under the terms of the GNU General Public
-** License versions 2.0 or 3.0 as published by the Free Software
-** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file.  Alternatively you may (at
-** your option) use any later version of the GNU General Public
-** License if such license has been publicly approved by Trolltech ASA
-** (or its successors, if any) and the KDE Free Qt Foundation. In
-** addition, as a special exception, Trolltech gives you certain
-** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.2, which can be found at
-** http://www.trolltech.com/products/qt/gplexception/ and in the file
-** GPL_EXCEPTION.txt in this package.
+** $QT_BEGIN_LICENSE:LGPL$
+** Commercial Usage
+** Licensees holding valid Qt Commercial licenses may use this file in
+** accordance with the Qt Commercial License Agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Nokia.
 **
-** Please review the following information to ensure GNU General
-** Public Licensing requirements will be met:
-** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
-** you are unsure which license is appropriate for your use, please
-** review the following information:
-** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
-** or contact the sales department at sales@trolltech.com.
+** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Trolltech, as the sole
-** copyright holder for Qt Designer, grants users of the Qt/Eclipse
-** Integration plug-in the right for the Qt/Eclipse Integration to
-** link to functionality provided by Qt Designer and its related
-** libraries.
+** In addition, as a special exception, Nokia gives you certain
+** additional rights. These rights are described in the Nokia Qt LGPL
+** Exception version 1.0, included in the file LGPL_EXCEPTION.txt in this
+** package.
 **
-** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
-** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
-** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
-** granted herein.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3.0 as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU General Public License version 3.0 requirements will be
+** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
-** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+** If you are unsure which license is appropriate for your use, please
+** contact the sales department at http://www.qtsoftware.com/contact.
+** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
@@ -48,6 +46,7 @@
 #ifndef QT_NO_DRAGANDDROP
 
 #include "qwidget.h"
+#include "qpainter.h"
 #include "qpixmap.h"
 #include "qbitmap.h"
 #include "qdesktopwidget.h"
@@ -270,12 +269,15 @@ static const char* const default_pm[] = {
 "X X X X X X X"
 };
 
-class QShapedPixmapWidget : public QWidget {
-
+class QShapedPixmapWidget : public QWidget
+{
+    Q_OBJECT
 public:
     QShapedPixmapWidget(QWidget* w) :
         QWidget(w,
-                Qt::Tool | Qt::FramelessWindowHint | Qt::X11BypassWindowManagerHint)
+                Qt::Tool | Qt::FramelessWindowHint
+                | Qt::X11BypassWindowManagerHint
+                | Qt::BypassGraphicsProxyWidget)
     {
         setAttribute(Qt::WA_X11NetWmWindowTypeDND);
     }
@@ -289,14 +291,22 @@ public:
             clearMask();
         }
         resize(pm.width(),pm.height());
-	QPalette p = palette();
-	p.setBrush(backgroundRole(), QBrush(pm));
-	setPalette(p);
+        pixmap = pm;
         update();
     }
     QPoint pm_hot;
+
+protected:
+    QPixmap pixmap;
+    void paintEvent(QPaintEvent*)
+    {
+        QPainter p(this);
+        p.drawPixmap(0, 0, pixmap);
+    }
 };
 
+#include "qdnd_x11.moc"
+
 struct XdndData {
     QShapedPixmapWidget *deco;
     QWidget* desktop_proxy;
@@ -743,6 +753,8 @@ static QWidget *find_child(QWidget *tlw, QPoint & p)
                 QWidget *w = qobject_cast<QWidget *>(children.at(i));
                 if (!w)
                     continue;
+                if (w->testAttribute(Qt::WA_TransparentForMouseEvents))
+                    continue;
                 if (w->isVisible() &&
                      w->geometry().contains(p) &&
                      !w->isWindow()) {
@@ -1335,10 +1347,12 @@ bool QDragManager::eventFilter(QObject * o, QEvent * e)
 void QDragManager::updateCursor()
 {
     if (!noDropCursor) {
+#ifndef QT_NO_CURSOR
         noDropCursor = new QCursor(Qt::ForbiddenCursor);
         moveCursor = new QCursor(dragCursor(Qt::MoveAction), 0,0);
         copyCursor = new QCursor(dragCursor(Qt::CopyAction), 0,0);
         linkCursor = new QCursor(dragCursor(Qt::LinkAction), 0,0);
+#endif
     }
 
     QCursor *c;
@@ -1448,6 +1462,10 @@ Window findRealWindow(const QPoint & pos, Window w, int md)
 
 void QDragManager::move(const QPoint & globalPos)
 {
+#ifdef QT_NO_CURSOR
+    Q_UNUSED(globalPos);
+    return;
+#else
     DEBUG() << "QDragManager::move enter";
     if (!object) {
         // perhaps the target crashed?
@@ -1642,6 +1660,7 @@ void QDragManager::move(const QPoint & globalPos)
         }
     }
     DEBUG() << "QDragManager::move leave";
+#endif
 }
 
 
