@@ -1,43 +1,41 @@
 /****************************************************************************
 **
-** Copyright (C) 2007-2008 Trolltech ASA. All rights reserved.
+** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
-** This file may be used under the terms of the GNU General Public
-** License versions 2.0 or 3.0 as published by the Free Software
-** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file.  Alternatively you may (at
-** your option) use any later version of the GNU General Public
-** License if such license has been publicly approved by Trolltech ASA
-** (or its successors, if any) and the KDE Free Qt Foundation. In
-** addition, as a special exception, Trolltech gives you certain
-** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.2, which can be found at
-** http://www.trolltech.com/products/qt/gplexception/ and in the file
-** GPL_EXCEPTION.txt in this package.
+** $QT_BEGIN_LICENSE:LGPL$
+** Commercial Usage
+** Licensees holding valid Qt Commercial licenses may use this file in
+** accordance with the Qt Commercial License Agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Nokia.
 **
-** Please review the following information to ensure GNU General
-** Public Licensing requirements will be met:
-** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
-** you are unsure which license is appropriate for your use, please
-** review the following information:
-** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
-** or contact the sales department at sales@trolltech.com.
+** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Trolltech, as the sole
-** copyright holder for Qt Designer, grants users of the Qt/Eclipse
-** Integration plug-in the right for the Qt/Eclipse Integration to
-** link to functionality provided by Qt Designer and its related
-** libraries.
+** In addition, as a special exception, Nokia gives you certain
+** additional rights. These rights are described in the Nokia Qt LGPL
+** Exception version 1.0, included in the file LGPL_EXCEPTION.txt in this
+** package.
 **
-** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
-** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
-** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
-** granted herein.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3.0 as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU General Public License version 3.0 requirements will be
+** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
-** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+** If you are unsure which license is appropriate for your use, please
+** contact the sales department at http://www.qtsoftware.com/contact.
+** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
@@ -189,9 +187,9 @@ public:
 
     int insertRow(int row);
     void insertRows(int row, int count);
-    void setItem(int row, int column, QLayoutItem *item, bool fullRow = false);
-    void setLayout(int row, int column, QLayout *layout, bool fullRow = false);
-    void setWidget(int row, int column, QWidget *widget, bool fullRow = false);
+    void setItem(int row, QFormLayout::ItemRole role, QLayoutItem *item);
+    void setLayout(int row, QFormLayout::ItemRole role, QLayout *layout);
+    void setWidget(int row, QFormLayout::ItemRole role, QWidget *widget);
 
     void arrangeWidgets(const QVector<QLayoutStruct>& layouts, QRect &rect);
 
@@ -950,8 +948,10 @@ void QFormLayoutPrivate::insertRows(int row, int count)
     }
 }
 
-void QFormLayoutPrivate::setItem(int row, int column, QLayoutItem *item, bool fullRow)
+void QFormLayoutPrivate::setItem(int row, QFormLayout::ItemRole role, QLayoutItem *item)
 {
+    const bool fullRow = role == QFormLayout::SpanningRole;
+    const int column =  role == QFormLayout::SpanningRole ? 1 : static_cast<int>(role);
     if (uint(row) >= uint(m_matrix.rowCount()) || uint(column) > 1U) {
         qWarning("QFormLayoutPrivate::setItem: Invalid cell (%d, %d)", row, column);
         return;
@@ -972,21 +972,21 @@ void QFormLayoutPrivate::setItem(int row, int column, QLayoutItem *item, bool fu
     m_things.append(i);
 }
 
-void QFormLayoutPrivate::setLayout(int row, int column, QLayout *layout, bool fullRow)
+void QFormLayoutPrivate::setLayout(int row, QFormLayout::ItemRole role, QLayout *layout)
 {
     if (layout) {
         Q_Q(QFormLayout);
         q->addChildLayout(layout);
-        setItem(row, column, layout, fullRow);
+        setItem(row, role, layout);
     }
 }
 
-void QFormLayoutPrivate::setWidget(int row, int column, QWidget *widget, bool fullRow)
+void QFormLayoutPrivate::setWidget(int row, QFormLayout::ItemRole role, QWidget *widget)
 {
     if (widget) {
         Q_Q(QFormLayout);
         q->addChildWidget(widget);
-        setItem(row, column, QLayoutPrivate::createWidgetItem(q, widget), fullRow);
+        setItem(row, role, QLayoutPrivate::createWidgetItem(q, widget));
     }
 }
 
@@ -1024,10 +1024,9 @@ QStyle* QFormLayoutPrivate::getStyle() const
     \o \bold{Adherence to the different platform's look and feel guidelines.}
 
         For example, the
-        \l{http://developer.apple.com/documentation/UserExperience/Conceptual/OSXHIGuidelines/}{Mac
-        OS X Aqua} and KDE guidelines specify that the labels should
-        be right-aligned, whereas Windows and GNOME applications
-        normally use left-alignment.
+        \l{Mac OS X Aqua} and KDE guidelines specify that the
+        labels should be right-aligned, whereas Windows and GNOME
+        applications normally use left-alignment.
 
     \o \bold{Support for wrapping long rows.}
 
@@ -1055,7 +1054,7 @@ QStyle* QFormLayoutPrivate::getStyle() const
         \o QCommonStyle derived styles (except QPlastiqueStyle)
         \o QMacStyle
         \o QPlastiqueStyle
-        \o Qtopia styles
+        \o Qt Extended styles
     \row
         \o \inlineimage qformlayout-win.png
         \o \inlineimage qformlayout-mac.png
@@ -1068,16 +1067,14 @@ QStyle* QFormLayoutPrivate::getStyle() const
            corresponds to what we would get using a two-column
            QGridLayout.)
         \o Style based on the
-           \l{http://developer.apple.com/documentation/UserExperience/Conceptual/OSXHIGuidelines/}{Mac
-           OS X Aqua guidelines}. Labels are right-aligned, the
-           fields don't grow beyond their size hint, and the form is
-           horizontally centered.
+           \l{Mac OS X Aqua} guidelines. Labels are right-aligned,
+           the fields don't grow beyond their size hint, and the
+           form is horizontally centered.
         \o Recommended style for
-           \l{http://www.kdedevelopers.org/node/2345}{KDE
-           applications}. Similar to MacStyle, except that the form
+           \l{KDE applications}. Similar to MacStyle, except that the form
            is left-aligned and all fields grow to fill the available
            space.
-        \o Default style for Qtopia styles. Labels are right-aligned,
+        \o Default style for Qt Extended styles. Labels are right-aligned,
            expanding fields grow to fill the available space, and row
            wrapping is enabled for long lines.
     \endtable
@@ -1129,14 +1126,14 @@ QStyle* QFormLayoutPrivate::getStyle() const
 
     \value DontWrapRows
            Fields are always laid out next to their label.  This is
-           the default policy for all styles except Qtopia styles.
+           the default policy for all styles except Qt Extended styles.
 
     \value WrapLongRows
            Labels are given enough horizontal space to fit the widest label,
            and the rest of the space is given to the fields. If the minimum
            size of a field pair is wider than the available space, the field
            is wrapped to the next line.  This is the default policy for
-           Qtopia styles.
+           Qt Extended styles.
 
     \value WrapAllRows
            Fields are always laid out below their label.
@@ -1152,6 +1149,7 @@ QStyle* QFormLayoutPrivate::getStyle() const
 
     \value LabelRole A label widget.
     \value FieldRole A field widget.
+    \value SpanningRole A widget that spans label and field columns.
 
     \sa itemAt(), getItemPosition()
 */
@@ -1260,9 +1258,9 @@ void QFormLayout::insertRow(int row, QWidget *label, QWidget *field)
 
     row = d->insertRow(row);
     if (label)
-        d->setWidget(row, 0, label);
+        d->setWidget(row, LabelRole, label);
     if (field)
-        d->setWidget(row, 1, field);
+        d->setWidget(row, FieldRole, field);
     invalidate();
 }
 
@@ -1275,9 +1273,9 @@ void QFormLayout::insertRow(int row, QWidget *label, QLayout *field)
 
     row = d->insertRow(row);
     if (label)
-        d->setWidget(row, 0, label);
+        d->setWidget(row, LabelRole, label);
     if (field)
-        d->setLayout(row, 1, field);
+        d->setLayout(row, FieldRole, field);
     invalidate();
 }
 
@@ -1328,7 +1326,7 @@ void QFormLayout::insertRow(int row, QWidget *widget)
     }
 
     row = d->insertRow(row);
-    d->setWidget(row, 1, widget, true /* full row */);
+    d->setWidget(row, SpanningRole, widget);
     invalidate();
 }
 
@@ -1349,7 +1347,7 @@ void QFormLayout::insertRow(int row, QLayout *layout)
     }
 
     row = d->insertRow(row);
-    d->setLayout(row, 1, layout, true /* full row */);
+    d->setLayout(row, SpanningRole, layout);
     invalidate();
 }
 
@@ -1361,7 +1359,7 @@ void QFormLayout::addItem(QLayoutItem *item)
     Q_D(QFormLayout);
 
     int row = d->insertRow(d->m_matrix.rowCount());
-    d->setItem(row, 1, item);
+    d->setItem(row, FieldRole, item);
     invalidate();
 }
 
@@ -1565,11 +1563,19 @@ QLayoutItem *QFormLayout::itemAt(int row, ItemRole role) const
     Q_D(const QFormLayout);
     if (uint(row) >= uint(d->m_matrix.rowCount()))
         return 0;
-    int col = (role == LabelRole) ? 0 : 1;
-    QFormLayoutItem *item = d->m_matrix(row, col);
-    if (!item)
-        return 0;
-    return item->item;
+    switch (role) {
+    case SpanningRole:
+        if (QFormLayoutItem *item = d->m_matrix(row, 1))
+            if (item->fullRow)
+                return item->item;
+        break;
+    case LabelRole:
+    case FieldRole:
+        if (QFormLayoutItem *item = d->m_matrix(row, (role == LabelRole) ? 0 : 1))
+            return item->item;
+        break;
+    }
+    return 0;
 }
 
 /*!
@@ -1592,8 +1598,14 @@ void QFormLayout::getItemPosition(int index, int *rowPtr, ItemRole *rolePtr) con
 
     if (rowPtr)
         *rowPtr = row;
-    if (rolePtr && col != -1)
-        *rolePtr = ItemRole(col);
+    if (rolePtr && col != -1) {
+        const bool spanning = col == 1 && d->m_matrix(row, col)->fullRow;
+        if (spanning) {
+            *rolePtr = SpanningRole;
+        } else {
+            *rolePtr = ItemRole(col);
+        }
+    }
 }
 
 /*!
@@ -1676,7 +1688,7 @@ QWidget *QFormLayout::labelForField(QLayout *field) const
     The default value depends on the widget or application style. For
     QMacStyle, the default is FieldsStayAtSizeHint; for QCommonStyle
     derived styles (like Plastique and Windows), the default
-    is ExpandingFieldsGrow; for Qtopia styles, the default is
+    is ExpandingFieldsGrow; for Qt Extended styles, the default is
     AllNonFixedFieldsGrow.
 
     If none of the fields can grow and the form is resized, extra
@@ -1710,7 +1722,7 @@ QFormLayout::FieldGrowthPolicy QFormLayout::fieldGrowthPolicy() const
     \brief the way in which the form's rows wrap
 
     The default value depends on the widget or application style. For
-    Qtopia styles, the default is WrapLongRows; for the other styles,
+    Qt Extended styles, the default is WrapLongRows; for the other styles,
     the default is DontWrapRows.
 
     If you want to display each label above its associated field
@@ -1995,6 +2007,7 @@ void QFormLayout::setLayout(int row, ItemRole role, QLayout *layout)
 
     If the cell is already occupied, the \a item is not inserted and an error message is
     sent to the console.
+    The \a item spans both columns.
 
     \warning Do not use this function to add child layouts or child
     widget items. Use setLayout() or setWidget() instead.
