@@ -1,43 +1,41 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
+** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
-** This file may be used under the terms of the GNU General Public
-** License versions 2.0 or 3.0 as published by the Free Software
-** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file.  Alternatively you may (at
-** your option) use any later version of the GNU General Public
-** License if such license has been publicly approved by Trolltech ASA
-** (or its successors, if any) and the KDE Free Qt Foundation. In
-** addition, as a special exception, Trolltech gives you certain
-** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.2, which can be found at
-** http://www.trolltech.com/products/qt/gplexception/ and in the file
-** GPL_EXCEPTION.txt in this package.
+** $QT_BEGIN_LICENSE:LGPL$
+** Commercial Usage
+** Licensees holding valid Qt Commercial licenses may use this file in
+** accordance with the Qt Commercial License Agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Nokia.
 **
-** Please review the following information to ensure GNU General
-** Public Licensing requirements will be met:
-** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
-** you are unsure which license is appropriate for your use, please
-** review the following information:
-** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
-** or contact the sales department at sales@trolltech.com.
+** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Trolltech, as the sole
-** copyright holder for Qt Designer, grants users of the Qt/Eclipse
-** Integration plug-in the right for the Qt/Eclipse Integration to
-** link to functionality provided by Qt Designer and its related
-** libraries.
+** In addition, as a special exception, Nokia gives you certain
+** additional rights. These rights are described in the Nokia Qt LGPL
+** Exception version 1.0, included in the file LGPL_EXCEPTION.txt in this
+** package.
 **
-** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
-** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
-** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
-** granted herein.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3.0 as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU General Public License version 3.0 requirements will be
+** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
-** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+** If you are unsure which license is appropriate for your use, please
+** contact the sales department at http://www.qtsoftware.com/contact.
+** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
@@ -339,8 +337,6 @@ static void init_direct(QColormapPrivate *d, bool ownColormap)
 
 static QColormap **cmaps = 0;
 
-/*! \internal
-*/
 void QColormap::initialize()
 {
     Display *display = QX11Info::display();
@@ -362,6 +358,8 @@ void QColormap::initialize()
         d->visual = DefaultVisual(display, i);
         d->defaultVisual = true;
 
+        Visual *argbVisual = 0;
+
         if (X11->visual && i == DefaultScreen(display)) {
             // only use the outside colormap on the default screen
             d->visual = find_visual(display, i, X11->visual->c_class,
@@ -384,8 +382,6 @@ void QColormap::initialize()
             XStandardColormap *stdcmap = 0;
             int ncmaps = 0;
 
-            bool foundArgbVisual = false;
-#if 0
 #ifndef QT_NO_XRENDER
             if (X11->use_xrender) {
                 int nvi;
@@ -400,18 +396,14 @@ void QColormap::initialize()
                     XRenderPictFormat *format = XRenderFindVisualFormat(X11->display,
                                                                         xvi[idx].visual);
                     if (format->type == PictTypeDirect && format->direct.alphaMask) {
-                        d->visual = xvi[idx].visual;
-                        d->depth = 32;
-                        d->defaultVisual = false;
-                        foundArgbVisual = true;
+                        argbVisual = xvi[idx].visual;
                         break;
                     }
                 }
+                XFree(xvi);
             }
 #endif
-#endif
-            if (!foundArgbVisual &&
-                XGetRGBColormaps(display, RootWindow(display, i),
+            if (XGetRGBColormaps(display, RootWindow(display, i),
                                  &stdcmap, &ncmaps, XA_RGB_DEFAULT_MAP)) {
                 if (stdcmap) {
                     for (int c = 0; c < ncmaps; ++c) {
@@ -569,6 +561,11 @@ void QColormap::initialize()
         screen->defaultColormap = d->defaultColormap;
         screen->cells = screen->visual->map_entries;
 
+        if (argbVisual) {
+            X11->argbVisuals[i] = argbVisual;
+            X11->argbColormaps[i] = XCreateColormap(display, RootWindow(display, i), argbVisual, AllocNone);
+        }
+
         // ###
         // We assume that 8bpp == pseudocolor, but this is not
         // always the case (according to the X server), so we need
@@ -579,8 +576,6 @@ void QColormap::initialize()
     }
 }
 
-/*! \internal
-*/
 void QColormap::cleanup()
 {
     Display *display = QX11Info::display();
