@@ -1,43 +1,41 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
+** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
-** This file may be used under the terms of the GNU General Public
-** License versions 2.0 or 3.0 as published by the Free Software
-** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file.  Alternatively you may (at
-** your option) use any later version of the GNU General Public
-** License if such license has been publicly approved by Trolltech ASA
-** (or its successors, if any) and the KDE Free Qt Foundation. In
-** addition, as a special exception, Trolltech gives you certain
-** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.2, which can be found at
-** http://www.trolltech.com/products/qt/gplexception/ and in the file
-** GPL_EXCEPTION.txt in this package.
+** $QT_BEGIN_LICENSE:LGPL$
+** Commercial Usage
+** Licensees holding valid Qt Commercial licenses may use this file in
+** accordance with the Qt Commercial License Agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Nokia.
 **
-** Please review the following information to ensure GNU General
-** Public Licensing requirements will be met:
-** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
-** you are unsure which license is appropriate for your use, please
-** review the following information:
-** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
-** or contact the sales department at sales@trolltech.com.
+** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Trolltech, as the sole
-** copyright holder for Qt Designer, grants users of the Qt/Eclipse
-** Integration plug-in the right for the Qt/Eclipse Integration to
-** link to functionality provided by Qt Designer and its related
-** libraries.
+** In addition, as a special exception, Nokia gives you certain
+** additional rights. These rights are described in the Nokia Qt LGPL
+** Exception version 1.0, included in the file LGPL_EXCEPTION.txt in this
+** package.
 **
-** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
-** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
-** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
-** granted herein.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3.0 as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU General Public License version 3.0 requirements will be
+** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
-** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+** If you are unsure which license is appropriate for your use, please
+** contact the sales department at http://www.qtsoftware.com/contact.
+** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
@@ -78,7 +76,6 @@ QT_BEGIN_NAMESPACE
 // This value is used to determine the length of control point vectors
 // when approximating arc segments as curves. The factor is multiplied
 // with the radius of the circle.
-#define KAPPA 0.5522847498
 
 // #define QPP_DEBUG
 // #define QPP_STROKE_DEBUG
@@ -1009,7 +1006,7 @@ void QPainterPath::addPolygon(const QPolygonF &polygon)
 /*!
     \fn void QPainterPath::addEllipse(const QRectF &boundingRectangle)
 
-    Creates an ellipse within the the specified \a boundingRectangle
+    Creates an ellipse within the specified \a boundingRectangle
     and adds it to the painter path as a closed subpath.
 
     The ellipse is composed of a clockwise curve, starting and
@@ -1105,10 +1102,10 @@ void QPainterPath::addText(const QPointF &point, const QFont &f, const QString &
         QScriptItem &si = eng->layoutData->items[item];
 
         if (si.analysis.flags < QScriptAnalysis::TabOrObject) {
-            QGlyphLayout *glyphs = eng->glyphs(&si);
+            QGlyphLayout glyphs = eng->shapedGlyphs(&si);
             QFontEngine *fe = f.d->engineForScript(si.analysis.script);
             Q_ASSERT(fe);
-            fe->addOutlineToPath(x, y, glyphs, si.num_glyphs, this,
+            fe->addOutlineToPath(x, y, glyphs, this,
                                  si.analysis.bidiLevel % 2
                                  ? QTextItem::RenderFlags(QTextItem::RightToLeft)
                                  : QTextItem::RenderFlags(0));
@@ -1189,6 +1186,13 @@ void QPainterPath::connectPath(const QPainterPath &other)
     d->elements += other.d_func()->elements;
 
     d->elements[first].type = LineToElement;
+
+    // avoid duplicate points
+    if (first > 0 && QPointF(d->elements[first]) == QPointF(d->elements[first - 1])) {
+        d->elements.remove(first--);
+        --cStart;
+    }
+
     if (cStart != first)
         d->cStart = cStart;
 }
@@ -1721,7 +1725,7 @@ static void qt_painterpath_isect_curve(const QBezier &bezier, const QPointF &pt,
 */
 bool QPainterPath::contains(const QPointF &pt) const
 {
-    if (isEmpty())
+    if (isEmpty() || !controlPointRect().contains(pt))
         return false;
 
     QPainterPathData *d = d_func();
@@ -2133,6 +2137,105 @@ bool QPainterPath::operator!=(const QPainterPath &path) const
     return !(*this==path);
 }
 
+/*!
+    \since 4.5
+
+    Returns the intersection of this path and the \a other path.
+
+    \sa intersected(), operator&=(), united(), operator|()
+*/
+QPainterPath QPainterPath::operator&(const QPainterPath &other) const
+{
+    return intersected(other);
+}
+
+/*!
+    \since 4.5
+
+    Returns the union of this path and the \a other path.
+
+    \sa united(), operator|=(), intersected(), operator&()
+*/
+QPainterPath QPainterPath::operator|(const QPainterPath &other) const
+{
+    return united(other);
+}
+
+/*!
+    \since 4.5
+
+    Returns the union of this path and the \a other path. This function is equivalent
+    to operator|().
+
+    \sa united(), operator+=(), operator-()
+*/
+QPainterPath QPainterPath::operator+(const QPainterPath &other) const
+{
+    return united(other);
+}
+
+/*!
+    \since 4.5
+
+    Subtracts the \a other path from a copy of this path, and returns the copy.
+
+    \sa subtracted(), operator-=(), operator+()
+*/
+QPainterPath QPainterPath::operator-(const QPainterPath &other) const
+{
+    return subtracted(other);
+}
+
+/*!
+    \since 4.5
+
+    Intersects this path with \a other and returns a reference to this path.
+
+    \sa intersected(), operator&(), operator|=()
+*/
+QPainterPath &QPainterPath::operator&=(const QPainterPath &other)
+{
+    return *this = (*this & other);
+}
+
+/*!
+    \since 4.5
+
+    Unites this path with \a other and returns a reference to this path.
+
+    \sa united(), operator|(), operator&=()
+*/
+QPainterPath &QPainterPath::operator|=(const QPainterPath &other)
+{
+    return *this = (*this | other);
+}
+
+/*!
+    \since 4.5
+
+    Unites this path with \a other, and returns a reference to this path. This
+    is equivalent to operator|=().
+
+    \sa united(), operator+(), operator-=()
+*/
+QPainterPath &QPainterPath::operator+=(const QPainterPath &other)
+{
+    return *this = (*this + other);
+}
+
+/*!
+    \since 4.5
+
+    Subtracts \a other from this path, and returns a reference to this
+    path.
+
+    \sa subtracted(), operator-(), operator+=()
+*/
+QPainterPath &QPainterPath::operator-=(const QPainterPath &other)
+{
+    return *this = (*this - other);
+}
+
 #ifndef QT_NO_DATASTREAM
 /*!
     \fn QDataStream &operator<<(QDataStream &stream, const QPainterPath &path)
@@ -2284,21 +2387,13 @@ void qt_path_stroke_cubic_to(qfixed c1x, qfixed c1y,
     \sa QPen, QBrush
 */
 
-class QPainterPathStrokerPrivate
+QPainterPathStrokerPrivate::QPainterPathStrokerPrivate()
+    : dashOffset(0)
 {
-public:
-    QPainterPathStrokerPrivate()
-        : dashOffset(0)
-    {
-        stroker.setMoveToHook(qt_path_stroke_move_to);
-        stroker.setLineToHook(qt_path_stroke_line_to);
-        stroker.setCubicToHook(qt_path_stroke_cubic_to);
-    }
-
-    QStroker stroker;
-    QVector<qfixed> dashPattern;
-    qreal dashOffset;
-};
+    stroker.setMoveToHook(qt_path_stroke_move_to);
+    stroker.setLineToHook(qt_path_stroke_line_to);
+    stroker.setCubicToHook(qt_path_stroke_cubic_to);
+}
 
 /*!
    Creates a new stroker.
@@ -2342,6 +2437,7 @@ QPainterPath QPainterPathStroker::createStroke(const QPainterPath &path) const
         QDashStroker dashStroker(&d->stroker);
         dashStroker.setDashPattern(d->dashPattern);
         dashStroker.setDashOffset(d->dashOffset);
+        dashStroker.setClipRect(d->stroker.clipRect());
         dashStroker.strokePath(path, &stroke, QTransform());
     }
     stroke.setFillRule(Qt::WindingFill);
@@ -2501,6 +2597,9 @@ qreal QPainterPathStroker::dashOffset() const
 
 /*!
   Sets the dash offset for the generated outlines to \a offset.
+
+  See the documentation for QPen::setDashOffset() for a description of the
+  dash offset.
  */
 void QPainterPathStroker::setDashOffset(qreal offset)
 {
@@ -2596,7 +2695,7 @@ qreal QPainterPath::length() const
 /*!
     Returns percentage of the whole path at the specified length \a len.
 
-    Note that similarly to other percent methods, the percentage measurment
+    Note that similarly to other percent methods, the percentage measurement
     is not linear with regards to the length, if curves are present
     in the path. When curves are present the percentage argument is mapped
     to the t parameter of the Bezier equations.
@@ -2713,7 +2812,7 @@ static inline QBezier bezierAtT(const QPainterPath &path, qreal t, qreal *starti
     Returns the point at at the percentage \a t of the current path.
     The argument \a t has to be between 0 and 1.
 
-    Note that similarly to other percent methods, the percentage measurment
+    Note that similarly to other percent methods, the percentage measurement
     is not linear with regards to the length, if curves are present
     in the path. When curves are present the percentage argument is mapped
     to the t parameter of the Bezier equations.
@@ -2744,7 +2843,7 @@ QPointF QPainterPath::pointAtPercent(qreal t) const
     Positive values for the angles mean counter-clockwise while negative values
     mean the clockwise direction. Zero degrees is at the 3 o'clock position.
 
-    Note that similarly to the other percent methods, the percentage measurment
+    Note that similarly to the other percent methods, the percentage measurement
     is not linear with regards to the length if curves are present
     in the path. When curves are present the percentage argument is mapped
     to the t parameter of the Bezier equations.
@@ -2776,7 +2875,7 @@ qreal QPainterPath::angleAtPercent(qreal t) const
     Returns the slope of the path at the percentage \a t. The
     argument \a t has to be between 0 and 1.
 
-    Note that similarly to other percent methods, the percentage measurment
+    Note that similarly to other percent methods, the percentage measurement
     is not linear with regards to the length, if curves are present
     in the path. When curves are present the percentage argument is mapped
     to the t parameter of the Bezier equations.
@@ -2843,8 +2942,16 @@ void QPainterPath::addRoundedRect(const QRectF &rect, qreal xRadius, qreal yRadi
         qreal w = r.width() / 2;
         qreal h = r.height() / 2;
 
-        xRadius = 100 * qMin(xRadius, w) / w;
-        yRadius = 100 * qMin(yRadius, h) / h;
+        if (w == 0) {
+            xRadius = 0;
+        } else {
+            xRadius = 100 * qMin(xRadius, w) / w;
+        }
+        if (h == 0) {
+            yRadius = 0;
+        } else {
+            yRadius = 100 * qMin(yRadius, h) / h;
+        }
     } else {
         if (xRadius > 100)                          // fix ranges
             xRadius = 100;
@@ -2990,6 +3097,9 @@ void QPainterPath::addRoundRect(const QRectF &r, int xRnd, int yRnd)
 
     Returns a path which is the union of this path's fill area and \a p's fill area.
 
+    Set operations on paths will treat the paths as areas. Non-closed
+    paths will be treated as implicitly closed.
+
     \sa intersected(), subtracted(), subtractedInverted()
 */
 QPainterPath QPainterPath::united(const QPainterPath &p) const
@@ -3017,6 +3127,10 @@ QPainterPath QPainterPath::intersected(const QPainterPath &p) const
     \since 4.3
 
     Returns a path which is \a p's fill area subtracted from this path's fill area.
+
+    Set operations on paths will treat the paths as areas. Non-closed
+    paths will be treated as implicitly closed.
+
 */
 QPainterPath QPainterPath::subtracted(const QPainterPath &p) const
 {
@@ -3060,6 +3174,9 @@ QPainterPath QPainterPath::simplified() const
   Returns true if the current path intersects at any point the given path \a p.
   Also returns true if the current path contains or is contained by any part of \a p.
 
+  Set operations on paths will treat the paths as areas. Non-closed
+  paths will be treated as implicitly closed.
+
   \sa contains()
  */
 bool QPainterPath::intersects(const QPainterPath &p) const
@@ -3079,6 +3196,9 @@ bool QPainterPath::intersects(const QPainterPath &p) const
   the current path. Returns false if any edges of the current path and
   \a p intersect.
 
+  Set operations on paths will treat the paths as areas. Non-closed
+  paths will be treated as implicitly closed.
+
   \sa intersects()
  */
 bool QPainterPath::contains(const QPainterPath &p) const
