@@ -1,52 +1,52 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
+** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
-** This file may be used under the terms of the GNU General Public
-** License versions 2.0 or 3.0 as published by the Free Software
-** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file.  Alternatively you may (at
-** your option) use any later version of the GNU General Public
-** License if such license has been publicly approved by Trolltech ASA
-** (or its successors, if any) and the KDE Free Qt Foundation. In
-** addition, as a special exception, Trolltech gives you certain
-** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.2, which can be found at
-** http://www.trolltech.com/products/qt/gplexception/ and in the file
-** GPL_EXCEPTION.txt in this package.
+** $QT_BEGIN_LICENSE:LGPL$
+** Commercial Usage
+** Licensees holding valid Qt Commercial licenses may use this file in
+** accordance with the Qt Commercial License Agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Nokia.
 **
-** Please review the following information to ensure GNU General
-** Public Licensing requirements will be met:
-** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
-** you are unsure which license is appropriate for your use, please
-** review the following information:
-** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
-** or contact the sales department at sales@trolltech.com.
+** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Trolltech, as the sole
-** copyright holder for Qt Designer, grants users of the Qt/Eclipse
-** Integration plug-in the right for the Qt/Eclipse Integration to
-** link to functionality provided by Qt Designer and its related
-** libraries.
+** In addition, as a special exception, Nokia gives you certain
+** additional rights. These rights are described in the Nokia Qt LGPL
+** Exception version 1.0, included in the file LGPL_EXCEPTION.txt in this
+** package.
 **
-** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
-** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
-** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
-** granted herein.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3.0 as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU General Public License version 3.0 requirements will be
+** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
-** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+** If you are unsure which license is appropriate for your use, please
+** contact the sales department at http://www.qtsoftware.com/contact.
+** $QT_END_LICENSE$
 **
 ****************************************************************************/
 #include "qplatformdefs.h"
 #include <qdebug.h>
 #include "qpdf_p.h"
 #include <qfile.h>
+#include <qtemporaryfile.h>
 #include <private/qmath_p.h>
 #include "private/qcups_p.h"
 #include "qprinterinfo.h"
+#include <qnumeric.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -59,11 +59,19 @@ extern QSizeF qt_paperSizeToQSizeF(QPrinter::PaperSize size);
 /* also adds a space at the end of the number */
 const char *qt_real_to_string(qreal val, char *buf) {
     const char *ret = buf;
+
+    if (qIsNaN(val)) {
+        *(buf++) = '0';
+        *(buf++) = ' ';
+        *buf = 0;
+        return ret;
+    }
+
     if (val < 0) {
         *(buf++) = '-';
         val = -val;
     }
-    int ival = (int) val;
+    unsigned int ival = (unsigned int) val;
     qreal frac = val - (qreal)ival;
 
     int ifrac = (int)(frac * 1000000);
@@ -125,6 +133,137 @@ const char *qt_int_to_string(int val, char *buf) {
     return ret;
 }
 
+
+namespace QPdf {
+    ByteStream::ByteStream(QByteArray *byteArray, bool fileBacking)
+            : dev(new QBuffer(byteArray)),
+            fileBackingEnabled(fileBacking),
+            fileBackingActive(false),
+            handleDirty(false)
+    {
+        dev->open(QIODevice::ReadWrite);
+    }
+
+    ByteStream::ByteStream(bool fileBacking)
+            : dev(new QBuffer(&ba)),
+            fileBackingEnabled(fileBacking),
+            fileBackingActive(false),
+            handleDirty(false)
+    {
+        dev->open(QIODevice::ReadWrite);
+    }
+
+    ByteStream::~ByteStream()
+    {
+        delete dev;
+    }
+
+    ByteStream &ByteStream::operator <<(char chr)
+    {
+        if (handleDirty) prepareBuffer();
+        dev->write(&chr, 1);
+        return *this;
+    }
+
+    ByteStream &ByteStream::operator <<(const char *str)
+    {
+        if (handleDirty) prepareBuffer();
+        dev->write(str, strlen(str));
+        return *this;
+    }
+
+    ByteStream &ByteStream::operator <<(const QByteArray &str)
+    {
+        if (handleDirty) prepareBuffer();
+        dev->write(str);
+        return *this;
+    }
+
+    ByteStream &ByteStream::operator <<(const ByteStream &src)
+    {
+        Q_ASSERT(!src.dev->isSequential());
+        if (handleDirty) prepareBuffer();
+        // We do play nice here, even though it looks ugly.
+        // We save the position and restore it afterwards.
+        ByteStream &s = const_cast<ByteStream&>(src);
+        qint64 pos = s.dev->pos();
+        s.dev->reset();
+        while (!s.dev->atEnd()) {
+            QByteArray buf = s.dev->read(chunkSize());
+            dev->write(buf);
+        }
+        s.dev->seek(pos);
+        return *this;
+    }
+
+    ByteStream &ByteStream::operator <<(qreal val) {
+        char buf[256];
+        qt_real_to_string(val, buf);
+        *this << buf;
+        return *this;
+    }
+
+    ByteStream &ByteStream::operator <<(int val) {
+        char buf[256];
+        qt_int_to_string(val, buf);
+        *this << buf;
+        return *this;
+    }
+
+    ByteStream &ByteStream::operator <<(const QPointF &p) {
+        char buf[256];
+        qt_real_to_string(p.x(), buf);
+        *this << buf;
+        qt_real_to_string(p.y(), buf);
+        *this << buf;
+        return *this;
+    }
+
+    QIODevice *ByteStream::stream()
+    {
+        dev->reset();
+        handleDirty = true;
+        return dev;
+    }
+
+    void ByteStream::clear()
+    {
+        dev->open(QIODevice::ReadWrite | QIODevice::Truncate);
+    }
+
+    void ByteStream::constructor_helper(QByteArray *ba)
+    {
+        delete dev;
+        dev = new QBuffer(ba);
+        dev->open(QIODevice::ReadWrite);
+    }
+
+    void ByteStream::prepareBuffer()
+    {
+        Q_ASSERT(!dev->isSequential());
+        qint64 size = dev->size();
+        if (fileBackingEnabled && !fileBackingActive
+                && size > maxMemorySize()) {
+            // Switch to file backing.
+            QTemporaryFile *newFile = new QTemporaryFile;
+            newFile->open();
+            dev->reset();
+            while (!dev->atEnd()) {
+                QByteArray buf = dev->read(chunkSize());
+                newFile->write(buf);
+            }
+            delete dev;
+            dev = newFile;
+            ba.clear();
+            fileBackingActive = true;
+        }
+        if (dev->pos() != size) {
+            dev->seek(size);
+            handleDirty = false;
+        }
+    }
+}
+
 #define QT_PATH_ELEMENT(elm)
 
 QByteArray QPdf::generatePath(const QPainterPath &path, const QTransform &matrix, PathFlags flags)
@@ -608,7 +747,9 @@ static void cubicToHook(qfixed c1x, qfixed c1y,
 }
 
 QPdf::Stroker::Stroker()
-    : dashStroker(&basicStroker)
+    : stream(0),
+    first(true),
+    dashStroker(&basicStroker)
 {
     stroker = &basicStroker;
     basicStroker.setMoveToHook(moveToHook);
@@ -792,7 +933,7 @@ QByteArray QPdf::stripSpecialCharacters(const QByteArray &string)
 // -------------------------- base engine, shared code between PS and PDF -----------------------
 
 QPdfBaseEngine::QPdfBaseEngine(QPdfBaseEnginePrivate &dd, PaintEngineFeatures f)
-    : QPaintEngine(dd, f)
+    : QAlphaPaintEngine(dd, f)
 {
     Q_D(QPdfBaseEngine);
 #if !defined(QT_NO_CUPS) && !defined(QT_NO_LIBRARY)
@@ -879,26 +1020,33 @@ void QPdfBaseEngine::drawRects (const QRectF *rects, int rectCount)
 
 void QPdfBaseEngine::drawPolygon(const QPointF *points, int pointCount, PolygonDrawMode mode)
 {
+    Q_D(QPdfBaseEngine);
+
+    if (d->useAlphaEngine) {
+        QAlphaPaintEngine::drawPolygon(points, pointCount, mode);
+        if (!continueCall())
+            return;
+    }
+
     if (!points || !pointCount)
         return;
-    Q_D(QPdfBaseEngine);
 
     bool hb = d->hasBrush;
     QPainterPath p;
 
     switch(mode) {
-    case OddEvenMode:
-        p.setFillRule(Qt::OddEvenFill);
-        break;
-    case ConvexMode:
-    case WindingMode:
-        p.setFillRule(Qt::WindingFill);
-        break;
-    case PolylineMode:
-        d->hasBrush = false;
-        break;
-    default:
-        break;
+        case OddEvenMode:
+            p.setFillRule(Qt::OddEvenFill);
+            break;
+        case ConvexMode:
+        case WindingMode:
+            p.setFillRule(Qt::WindingFill);
+            break;
+        case PolylineMode:
+            d->hasBrush = false;
+            break;
+        default:
+            break;
     }
 
     p.moveTo(points[0]);
@@ -915,6 +1063,13 @@ void QPdfBaseEngine::drawPolygon(const QPointF *points, int pointCount, PolygonD
 void QPdfBaseEngine::drawPath (const QPainterPath &p)
 {
     Q_D(QPdfBaseEngine);
+
+    if (d->useAlphaEngine) {
+        QAlphaPaintEngine::drawPath(p);
+        if (!continueCall())
+            return;
+    }
+
     if (d->clipEnabled && d->allClipped)
         return;
     if (!d->hasPen && !d->hasBrush)
@@ -942,9 +1097,20 @@ void QPdfBaseEngine::drawTextItem(const QPointF &p, const QTextItem &textItem)
 {
     Q_D(QPdfBaseEngine);
 
+    if (d->useAlphaEngine) {
+        QAlphaPaintEngine::drawTextItem(p, textItem);
+        if (!continueCall())
+            return;
+    }
+
     if (!d->hasPen || (d->clipEnabled && d->allClipped))
         return;
 
+    if (d->stroker.matrix.type() >= QTransform::TxProject) {
+        QPaintEngine::drawTextItem(p, textItem);
+        return;
+    }
+
     *d->currentPage << "q\n";
     if(!d->simplePen)
         *d->currentPage << QPdf::generateMatrix(d->stroker.matrix);
@@ -967,6 +1133,13 @@ void QPdfBaseEngine::drawTextItem(const QPointF &p, const QTextItem &textItem)
 void QPdfBaseEngine::updateState(const QPaintEngineState &state)
 {
     Q_D(QPdfBaseEngine);
+
+    if (d->useAlphaEngine) {
+        QAlphaPaintEngine::updateState(state);
+        if (!continueCall())
+            return;
+    }
+
     QPaintEngine::DirtyFlags flags = state.state();
 
     if (flags & DirtyTransform)
@@ -1058,6 +1231,8 @@ void QPdfBaseEngine::setupGraphicsState(QPaintEngine::DirtyFlags flags)
         setPen();
 }
 
+extern QPainterPath qt_regionToPath(const QRegion &region);
+
 void QPdfBaseEngine::updateClipPath(const QPainterPath &p, Qt::ClipOperation op)
 {
     Q_D(QPdfBaseEngine);
@@ -1079,6 +1254,31 @@ void QPdfBaseEngine::updateClipPath(const QPainterPath &p, Qt::ClipOperation op)
         d->clips.clear();
         d->clips.append(path);
     }
+
+    if (d->useAlphaEngine) {
+        // if we have an alpha region, we have to subtract that from the
+        // any existing clip region since that region will be filled in
+        // later with images
+        QPainterPath alphaClip = qt_regionToPath(alphaClipping());
+        if (!alphaClip.isEmpty()) {
+            if (!d->clipEnabled) {
+                QRect r = d->fullPage ? d->paperRect() : d->pageRect();
+                QPainterPath dev;
+                dev.addRect(QRect(0, 0, r.width(), r.height()));
+                if (path.isEmpty())
+                    path = dev;
+                else
+                    path = path.intersected(dev);
+                d->clipEnabled = true;
+            } else {
+                path = painter()->clipPath();
+                path = d->stroker.matrix.map(path);
+            }
+            path = path.subtracted(alphaClip);
+            d->clips.clear();
+            d->clips.append(path);
+        }
+    }
 }
 
 void QPdfBaseEngine::setPen()
@@ -1382,11 +1582,13 @@ QVariant QPdfBaseEngine::property(PrintEnginePropertyKey key) const
 
 QPdfBaseEnginePrivate::QPdfBaseEnginePrivate(QPrinter::PrinterMode m)
     : clipEnabled(false), allClipped(false), hasPen(true), hasBrush(false), simplePen(false),
+      useAlphaEngine(false),
       outDevice(0), fd(-1),
       duplex(QPrinter::DuplexNone), collate(false), fullPage(false), embedFonts(true), copies(1),
       pageOrder(QPrinter::FirstPageFirst), orientation(QPrinter::Portrait),
       paperSize(QPrinter::A4), colorMode(QPrinter::Color), paperSource(QPrinter::Auto),
-      hasCustomPageMargins(false)
+      hasCustomPageMargins(false),
+      leftMargin(0), topMargin(0), rightMargin(0), bottomMargin(0)
 {
     resolution = 72;
     if (m == QPrinter::HighResolution)
@@ -1456,8 +1658,10 @@ bool QPdfBaseEnginePrivate::openPrintDevice()
 
     if (!outputFileName.isEmpty()) {
         QFile *file = new QFile(outputFileName);
-        if (! file->open(QFile::WriteOnly|QFile::Truncate))
+        if (! file->open(QFile::WriteOnly|QFile::Truncate)) {
+            delete file;
             return false;
+        }
         outDevice = file;
 #if !defined(QT_NO_CUPS) && !defined(QT_NO_LIBRARY)
     } else if (QCUPSSupport::isAvailable()) {
@@ -1727,7 +1931,7 @@ void QPdfBaseEnginePrivate::drawTextItem(const QPointF &p, const QTextItemInt &t
     QVarLengthArray<QFixedPoint> positions;
     QTransform m;
     m.translate(p.x(), p.y());
-    ti.fontEngine->getGlyphPositions(ti.glyphs, ti.num_glyphs, m, ti.flags,
+    ti.fontEngine->getGlyphPositions(ti.glyphs, m, ti.flags,
                                      glyphs, positions);
     if (glyphs.size() == 0)
         return;
@@ -1843,9 +2047,9 @@ QRect QPdfBaseEnginePrivate::pageRect() const
     QRect r;
 
 #if !defined(QT_NO_CUPS) && !defined(QT_NO_LIBRARY)
-    if (QCUPSSupport::isAvailable() && !cupsPageRect.isNull()) {
+    if (!hasCustomPageMargins && QCUPSSupport::isAvailable() && !cupsPageRect.isNull()) {
         r = cupsPageRect;
-        if (!hasCustomPageMargins && r == cupsPaperRect) {
+        if (r == cupsPaperRect) {
             // if cups doesn't define any margins, give it at least approx 3.5 mm
             r = QRect(10, 10, r.width() - 20, r.height() - 20);
         }
