@@ -1,43 +1,41 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
+** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
-** This file may be used under the terms of the GNU General Public
-** License versions 2.0 or 3.0 as published by the Free Software
-** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file.  Alternatively you may (at
-** your option) use any later version of the GNU General Public
-** License if such license has been publicly approved by Trolltech ASA
-** (or its successors, if any) and the KDE Free Qt Foundation. In
-** addition, as a special exception, Trolltech gives you certain
-** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.2, which can be found at
-** http://www.trolltech.com/products/qt/gplexception/ and in the file
-** GPL_EXCEPTION.txt in this package.
+** $QT_BEGIN_LICENSE:LGPL$
+** Commercial Usage
+** Licensees holding valid Qt Commercial licenses may use this file in
+** accordance with the Qt Commercial License Agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Nokia.
 **
-** Please review the following information to ensure GNU General
-** Public Licensing requirements will be met:
-** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
-** you are unsure which license is appropriate for your use, please
-** review the following information:
-** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
-** or contact the sales department at sales@trolltech.com.
+** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Trolltech, as the sole
-** copyright holder for Qt Designer, grants users of the Qt/Eclipse
-** Integration plug-in the right for the Qt/Eclipse Integration to
-** link to functionality provided by Qt Designer and its related
-** libraries.
+** In addition, as a special exception, Nokia gives you certain
+** additional rights. These rights are described in the Nokia Qt LGPL
+** Exception version 1.0, included in the file LGPL_EXCEPTION.txt in this
+** package.
 **
-** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
-** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
-** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
-** granted herein.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3.0 as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU General Public License version 3.0 requirements will be
+** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
-** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+** If you are unsure which license is appropriate for your use, please
+** contact the sales department at http://www.qtsoftware.com/contact.
+** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
@@ -474,11 +472,11 @@ int QFontMetrics::leftBearing(QChar ch) const
 
     d->alterCharForCapitalization(ch);
 
-    QGlyphLayout glyphs[10];
+    QGlyphLayoutArray<10> glyphs;
     int nglyphs = 9;
-    engine->stringToCMap(&ch, 1, glyphs, &nglyphs, 0);
+    engine->stringToCMap(&ch, 1, &glyphs, &nglyphs, 0);
     // ### can nglyphs != 1 happen at all? Not currently I think
-    glyph_metrics_t gi = engine->boundingBox(glyphs[0].glyph);
+    glyph_metrics_t gi = engine->boundingBox(glyphs.glyphs[0]);
     return qRound(gi.x);
 }
 
@@ -508,11 +506,11 @@ int QFontMetrics::rightBearing(QChar ch) const
 
     d->alterCharForCapitalization(ch);
 
-    QGlyphLayout glyphs[10];
+    QGlyphLayoutArray<10> glyphs;
     int nglyphs = 9;
-    engine->stringToCMap(&ch, 1, glyphs, &nglyphs, 0);
+    engine->stringToCMap(&ch, 1, &glyphs, &nglyphs, 0);
     // ### can nglyphs != 1 happen at all? Not currently I think
-    glyph_metrics_t gi = engine->boundingBox(glyphs[0].glyph);
+    glyph_metrics_t gi = engine->boundingBox(glyphs.glyphs[0]);
     return qRound(gi.xoff - gi.x - gi.width);
 }
 
@@ -537,7 +535,6 @@ int QFontMetrics::width(const QString &text, int len) const
 
     QTextEngine layout(text, d);
     layout.ignoreBidi = true;
-    layout.itemize();
     return qRound(layout.width(0, len));
 }
 
@@ -581,10 +578,10 @@ int QFontMetrics::width(QChar ch) const
 
     d->alterCharForCapitalization(ch);
 
-    QGlyphLayout glyphs[8];
+    QGlyphLayoutArray<8> glyphs;
     int nglyphs = 7;
-    engine->stringToCMap(&ch, 1, glyphs, &nglyphs, 0);
-    return qRound(glyphs[0].advance.x);
+    engine->stringToCMap(&ch, 1, &glyphs, &nglyphs, 0);
+    return qRound(glyphs.advances_x[0]);
 }
 
 /*! \obsolete
@@ -629,10 +626,10 @@ int QFontMetrics::charWidth(const QString &text, int pos) const
 
         d->alterCharForCapitalization(ch);
 
-        QGlyphLayout glyphs[8];
+        QGlyphLayoutArray<8> glyphs;
         int nglyphs = 7;
-        engine->stringToCMap(&ch, 1, glyphs, &nglyphs, 0);
-        width = qRound(glyphs[0].advance.x);
+        engine->stringToCMap(&ch, 1, &glyphs, &nglyphs, 0);
+        width = qRound(glyphs.advances_x[0]);
     }
     return width;
 }
@@ -673,8 +670,8 @@ QRect QFontMetrics::boundingRect(const QString &text) const
     Returns the rectangle that is covered by ink if character \a ch
     were to be drawn at the origin of the coordinate system.
 
-    Note that the bounding rectangle may extend to the left of (0, 0),
-    e.g. for italicized fonts, and that the text output may cover \e
+    Note that the bounding rectangle may extend to the left of (0, 0)
+    (e.g., for italicized fonts), and that the text output may cover \e
     all pixels in the bounding rectangle. For a space character the rectangle
     will usually be empty.
 
@@ -698,10 +695,10 @@ QRect QFontMetrics::boundingRect(QChar ch) const
 
     d->alterCharForCapitalization(ch);
 
-    QGlyphLayout glyphs[10];
+    QGlyphLayoutArray<10> glyphs;
     int nglyphs = 9;
-    engine->stringToCMap(&ch, 1, glyphs, &nglyphs, 0);
-    glyph_metrics_t gm = engine->boundingBox(glyphs[0].glyph);
+    engine->stringToCMap(&ch, 1, &glyphs, &nglyphs, 0);
+    glyph_metrics_t gm = engine->boundingBox(glyphs.glyphs[0]);
     return QRect(qRound(gm.x), qRound(gm.y), qRound(gm.width), qRound(gm.height));
 }
 
@@ -727,7 +724,7 @@ QRect QFontMetrics::boundingRect(QChar ch) const
     \o Qt::AlignCenter (== \c{Qt::AlignHCenter | Qt::AlignVCenter})
     \o Qt::TextSingleLine ignores newline characters in the text.
     \o Qt::TextExpandTabs expands tabs (see below)
-    \o Qt::TextShowMnemonic interprets "&amp;x" as \underline{x}, i.e. underlined.
+    \o Qt::TextShowMnemonic interprets "&x" as \underline{x}; i.e., underlined.
     \o Qt::TextWordWrap breaks the text to fit the rectangle.
     \endlist
 
@@ -784,7 +781,7 @@ QRect QFontMetrics::boundingRect(const QRect &rect, int flags, const QString &te
     \list
     \o Qt::TextSingleLine ignores newline characters.
     \o Qt::TextExpandTabs expands tabs (see below)
-    \o Qt::TextShowMnemonic interprets "&amp;x" as \underline{x}, i.e. underlined.
+    \o Qt::TextShowMnemonic interprets "&x" as \underline{x}; i.e., underlined.
     \o Qt::TextWordBreak breaks the text to fit the rectangle.
     \endlist
 
@@ -855,6 +852,13 @@ QRect QFontMetrics::tightBoundingRect(const QString &text) const
 
     The \a flags argument is optional and currently only supports
     Qt::TextShowMnemonic as value.
+
+    The elide mark will follow the \l{Qt::LayoutDirection}{layout
+    direction}; it will be on the right side of the text for
+    right-to-left layouts, and on the left side for right-to-left
+    layouts. Note that this behavior is independent of the text
+    language.
+
 */
 QString QFontMetrics::elidedText(const QString &text, Qt::TextElideMode mode, int width, int flags) const
 {
@@ -919,7 +923,7 @@ int QFontMetrics::lineWidth() const
  *****************************************************************************/
 
 /*!
-    \class QFontMetricsF qfontmetrics.h
+    \class QFontMetricsF
     \reentrant
 
     \brief The QFontMetricsF class provides font metrics information.
@@ -1303,11 +1307,11 @@ qreal QFontMetricsF::leftBearing(QChar ch) const
 
     d->alterCharForCapitalization(ch);
 
-    QGlyphLayout glyphs[10];
+    QGlyphLayoutArray<10> glyphs;
     int nglyphs = 9;
-    engine->stringToCMap(&ch, 1, glyphs, &nglyphs, 0);
+    engine->stringToCMap(&ch, 1, &glyphs, &nglyphs, 0);
     // ### can nglyphs != 1 happen at all? Not currently I think
-    glyph_metrics_t gi = engine->boundingBox(glyphs[0].glyph);
+    glyph_metrics_t gi = engine->boundingBox(glyphs.glyphs[0]);
     return gi.x.toReal();
 }
 
@@ -1337,11 +1341,11 @@ qreal QFontMetricsF::rightBearing(QChar ch) const
 
     d->alterCharForCapitalization(ch);
 
-    QGlyphLayout glyphs[10];
+    QGlyphLayoutArray<10> glyphs;
     int nglyphs = 9;
-    engine->stringToCMap(&ch, 1, glyphs, &nglyphs, 0);
+    engine->stringToCMap(&ch, 1, &glyphs, &nglyphs, 0);
     // ### can nglyphs != 1 happen at all? Not currently I think
-    glyph_metrics_t gi = engine->boundingBox(glyphs[0].glyph);
+    glyph_metrics_t gi = engine->boundingBox(glyphs.glyphs[0]);
     return (gi.xoff - gi.x - gi.width).toReal();
 }
 
@@ -1403,10 +1407,10 @@ qreal QFontMetricsF::width(QChar ch) const
 
     d->alterCharForCapitalization(ch);
 
-    QGlyphLayout glyphs[8];
+    QGlyphLayoutArray<8> glyphs;
     int nglyphs = 7;
-    engine->stringToCMap(&ch, 1, glyphs, &nglyphs, 0);
-    return glyphs[0].advance.x.toReal();
+    engine->stringToCMap(&ch, 1, &glyphs, &nglyphs, 0);
+    return glyphs.advances_x[0].toReal();
 }
 
 /*!
@@ -1468,10 +1472,10 @@ QRectF QFontMetricsF::boundingRect(QChar ch) const
 
     d->alterCharForCapitalization(ch);
 
-    QGlyphLayout glyphs[10];
+    QGlyphLayoutArray<10> glyphs;
     int nglyphs = 9;
-    engine->stringToCMap(&ch, 1, glyphs, &nglyphs, 0);
-    glyph_metrics_t gm = engine->boundingBox(glyphs[0].glyph);
+    engine->stringToCMap(&ch, 1, &glyphs, &nglyphs, 0);
+    glyph_metrics_t gm = engine->boundingBox(glyphs.glyphs[0]);
     return QRectF(gm.x.toReal(), gm.y.toReal(), gm.width.toReal(), gm.height.toReal());
 }
 
@@ -1496,7 +1500,7 @@ QRectF QFontMetricsF::boundingRect(QChar ch) const
     \o Qt::AlignCenter (== \c{Qt::AlignHCenter | Qt::AlignVCenter})
     \o Qt::TextSingleLine ignores newline characters in the text.
     \o Qt::TextExpandTabs expands tabs (see below)
-    \o Qt::TextShowMnemonic interprets "&amp;x" as \underline{x}, i.e. underlined.
+    \o Qt::TextShowMnemonic interprets "&x" as \underline{x}; i.e., underlined.
     \o Qt::TextWordWrap breaks the text to fit the rectangle.
     \endlist
 
@@ -1555,7 +1559,7 @@ QRectF QFontMetricsF::boundingRect(const QRectF &rect, int flags, const QString&
     \list
     \o Qt::TextSingleLine ignores newline characters.
     \o Qt::TextExpandTabs expands tabs (see below)
-    \o Qt::TextShowMnemonic interprets "&amp;x" as \underline{x}, i.e. underlined.
+    \o Qt::TextShowMnemonic interprets "&x" as \underline{x}; i.e., underlined.
     \o Qt::TextWordBreak breaks the text to fit the rectangle.
     \endlist
 
@@ -1724,7 +1728,7 @@ qreal QFontMetricsF::lineWidth() const
 
     Use the boundingRect() function in combination with
     QString::left() instead.
-
+     
     \oldcode
         QRect rect = boundingRect(text, len);
     \newcode
