@@ -1,43 +1,41 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
+** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
-** This file may be used under the terms of the GNU General Public
-** License versions 2.0 or 3.0 as published by the Free Software
-** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file.  Alternatively you may (at
-** your option) use any later version of the GNU General Public
-** License if such license has been publicly approved by Trolltech ASA
-** (or its successors, if any) and the KDE Free Qt Foundation. In
-** addition, as a special exception, Trolltech gives you certain
-** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.2, which can be found at
-** http://www.trolltech.com/products/qt/gplexception/ and in the file
-** GPL_EXCEPTION.txt in this package.
+** $QT_BEGIN_LICENSE:LGPL$
+** Commercial Usage
+** Licensees holding valid Qt Commercial licenses may use this file in
+** accordance with the Qt Commercial License Agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Nokia.
 **
-** Please review the following information to ensure GNU General
-** Public Licensing requirements will be met:
-** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
-** you are unsure which license is appropriate for your use, please
-** review the following information:
-** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
-** or contact the sales department at sales@trolltech.com.
+** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Trolltech, as the sole
-** copyright holder for Qt Designer, grants users of the Qt/Eclipse
-** Integration plug-in the right for the Qt/Eclipse Integration to
-** link to functionality provided by Qt Designer and its related
-** libraries.
+** In addition, as a special exception, Nokia gives you certain
+** additional rights. These rights are described in the Nokia Qt LGPL
+** Exception version 1.0, included in the file LGPL_EXCEPTION.txt in this
+** package.
 **
-** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
-** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
-** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
-** granted herein.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3.0 as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU General Public License version 3.0 requirements will be
+** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
-** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+** If you are unsure which license is appropriate for your use, please
+** contact the sales department at http://www.qtsoftware.com/contact.
+** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
@@ -412,17 +410,19 @@ bool QTextCursorPrivate::movePosition(QTextCursor::MoveOperation op, QTextCursor
         newPosition = priv->previousCursorPosition(position, QTextLayout::SkipCharacters);
         break;
     case QTextCursor::StartOfWord: {
-        QTextEngine *engine = layout->engine();
-        const HB_CharAttributes *attributes = engine->attributes();
-
         if (relativePos == 0)
-            return false;
+            break;
 
         // skip if already at word start
-        if (attributes[relativePos - 1].whiteSpace
-            && !attributes[relativePos].whiteSpace)
+        QTextEngine *engine = layout->engine();
+        engine->attributes();
+        if ((relativePos == blockIt.length() - 1)
+            && (engine->atSpace(relativePos - 1) || engine->atWordSeparator(relativePos - 1)))
             return false;
 
+        if (relativePos < blockIt.length()-1)
+            ++position;
+
         // FALL THROUGH!
     }
     case QTextCursor::PreviousWord:
@@ -488,18 +488,18 @@ bool QTextCursorPrivate::movePosition(QTextCursor::MoveOperation op, QTextCursor
     }
     case QTextCursor::EndOfWord: {
         QTextEngine *engine = layout->engine();
-        const HB_CharAttributes *attributes = engine->attributes();
-        const QString string = engine->layoutData->string;
-
-        const int len = layout->engine()->layoutData->string.length();
+        engine->attributes();
+        const int len = blockIt.length() - 1;
         if (relativePos >= len)
             return false;
-        relativePos++;
-        while (relativePos < len
-               && !attributes[relativePos].whiteSpace
-               && !engine->atWordSeparator(relativePos))
-            relativePos++;
-
+        if (engine->atWordSeparator(relativePos)) {
+            ++relativePos;
+            while (relativePos < len && engine->atWordSeparator(relativePos))
+                ++relativePos;
+        } else {
+            while (relativePos < len && !engine->atSpace(relativePos) && !engine->atWordSeparator(relativePos))
+                ++relativePos;
+        }
         newPosition = blockIt.position() + relativePos;
         break;
     }
@@ -563,6 +563,49 @@ bool QTextCursorPrivate::movePosition(QTextCursor::MoveOperation op, QTextCursor
         adjustX = false;
         break;
     }
+    case QTextCursor::NextCell: // fall through
+    case QTextCursor::PreviousCell: // fall through
+    case QTextCursor::NextRow: // fall through
+    case QTextCursor::PreviousRow: {
+        QTextTable *table = qobject_cast<QTextTable *>(priv->frameAt(position));
+        if (!table)
+            return false;
+
+        QTextTableCell cell = table->cellAt(position);
+        Q_ASSERT(cell.isValid());
+        int column = cell.column();
+        int row = cell.row();
+        const int currentRow = row;
+        if (op == QTextCursor::NextCell || op == QTextCursor::NextRow) {
+            do {
+                column += cell.columnSpan();
+                if (column >= table->columns()) {
+                    column = 0;
+                    ++row;
+                }
+                cell = table->cellAt(row, column);
+                // note we also continue while we have not reached a cell thats not merged with one above us
+            } while (cell.isValid()
+                    && ((op == QTextCursor::NextRow && currentRow == cell.row())
+                        || cell.row() < row));
+        }
+        else if (op == QTextCursor::PreviousCell || op == QTextCursor::PreviousRow) {
+            do {
+                --column;
+                if (column < 0) {
+                    column = table->columns()-1;
+                    --row;
+                }
+                cell = table->cellAt(row, column);
+                // note we also continue while we have not reached a cell thats not merged with one above us
+            } while (cell.isValid()
+                    && ((op == QTextCursor::PreviousRow && currentRow == cell.row())
+                        || cell.row() < row));
+        }
+        if (cell.isValid())
+            newPosition = cell.firstPosition();
+        break;
+    }
     }
 
     if (mode == QTextCursor::KeepAnchor) {
@@ -928,6 +971,17 @@ QTextLayout *QTextCursorPrivate::blockLayout(QTextBlock &block) const{
     \value Right Move right one character.
     \value WordRight Move right one word.
 
+    \value NextCell  Move to the beginning of the next table cell inside the
+           current table. If the current cell is the last cell in the row, the
+           cursor will move to the first cell in the next row.
+    \value PreviousCell  Move to the beginning of the previous table cell
+           inside the current table. If the current cell is the first cell in
+           the row, the cursor will move to the last cell in the previous row.
+    \value NextRow  Move to the first new cell of the next row in the current
+           table.
+    \value PreviousRow  Move to the last cell of the previous row in the
+           current table.
+
     \sa movePosition()
 */
 
@@ -1020,7 +1074,10 @@ QTextCursor::QTextCursor(const QTextCursor &cursor)
 }
 
 /*!
-    Makes a copy of \a cursor and assigns it to this QTextCursor.
+    Makes a copy of \a cursor and assigns it to this QTextCursor. Note
+    that QTextCursor is an \l{Implicitly Shared Classes}{implicitly
+    shared} class.
+
  */
 QTextCursor &QTextCursor::operator=(const QTextCursor &cursor)
 {
@@ -1153,6 +1210,8 @@ bool QTextCursor::movePosition(MoveOperation op, MoveMode mode, int n)
                 b = b.previous();
             d->setPosition(b.position());
         }
+        if (mode == QTextCursor::MoveAnchor)
+            d->anchor = d->position;
         while (d->movePosition(op, mode)
                && !d->block().isVisible())
             ;
@@ -1242,11 +1301,16 @@ void QTextCursor::insertText(const QString &text, const QTextCharFormat &_format
 
         QTextBlockFormat blockFmt = blockFormat();
 
-        int textStart = 0;
+
+        int textStart = d->priv->text.length();
+        int blockStart = 0;
+        d->priv->text += text;
+        int textEnd = d->priv->text.length();
+
         for (int i = 0; i < text.length(); ++i) {
             QChar ch = text.at(i);
 
-            const int textEnd = i;
+            const int blockEnd = i;
 
             if (ch == QLatin1Char('\r')
                 && (i + 1) < text.length()
@@ -1259,15 +1323,15 @@ void QTextCursor::insertText(const QString &text, const QTextCharFormat &_format
                 || ch == QChar::ParagraphSeparator
                 || ch == QLatin1Char('\r')) {
 
-                if (textEnd > textStart)
-                    d->priv->insert(d->position, QString(text.unicode() + textStart, textEnd - textStart), formatIdx);
+                if (blockEnd > blockStart)
+                    d->priv->insert(d->position, textStart + blockStart, blockEnd - blockStart, formatIdx);
 
-                textStart = i + 1;
                 d->insertBlock(blockFmt, format);
+                blockStart = i + 1;
             }
         }
-        if (textStart < text.length())
-            d->priv->insert(d->position, QString(text.unicode() + textStart, text.length() - textStart), formatIdx);
+        if (textStart + blockStart < textEnd)
+            d->priv->insert(d->position, textStart + blockStart, textEnd - textStart - blockStart, formatIdx);
     }
     d->priv->endEditBlock();
     d->setX();
@@ -1477,7 +1541,7 @@ static void getText(QString &text, QTextDocumentPrivate *priv, const QString &do
         const QTextFragmentData * const frag = fragIt.value();
 
         const int offsetInFragment = qMax(0, pos - fragIt.position());
-        const int len = qMin(int(frag->size - offsetInFragment), end - pos);
+        const int len = qMin(int(frag->size_array[0] - offsetInFragment), end - pos);
 
         text += QString(docText.constData() + frag->stringPosition + offsetInFragment, len);
         pos += len;
@@ -2101,6 +2165,28 @@ void QTextCursor::insertImage(const QString &name)
     insertImage(format);
 }
 
+/*!
+    \since 4.5
+    \overload
+
+    Convenience function for inserting the given \a image with an optional
+    \a name at the current position().
+*/
+void QTextCursor::insertImage(const QImage &image, const QString &name)
+{
+    if (image.isNull()) {
+        qWarning("QTextCursor::insertImage: attempt to add an invalid image");
+        return;
+    }
+    QString imageName = name;
+    if (name.isEmpty())
+        imageName = QString::number(image.serialNumber());
+    d->priv->document()->addResource(QTextDocument::ImageResource, QUrl(imageName), image);
+    QTextImageFormat format;
+    format.setName(imageName);
+    insertImage(format);
+}
+
 /*!
     \fn bool QTextCursor::operator!=(const QTextCursor &other) const
 
@@ -2323,4 +2409,15 @@ int QTextCursor::columnNumber() const
     return relativePos - line.textStart();
 }
 
+/*!
+    \since 4.5
+    Returns the document this cursor is associated with.
+*/
+QTextDocument *QTextCursor::document() const
+{
+    if (d->priv)
+        return d->priv->document();
+    return 0; // document went away
+}
+
 QT_END_NAMESPACE
