@@ -1,43 +1,41 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
+** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
-** This file may be used under the terms of the GNU General Public
-** License versions 2.0 or 3.0 as published by the Free Software
-** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file.  Alternatively you may (at
-** your option) use any later version of the GNU General Public
-** License if such license has been publicly approved by Trolltech ASA
-** (or its successors, if any) and the KDE Free Qt Foundation. In
-** addition, as a special exception, Trolltech gives you certain
-** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.2, which can be found at
-** http://www.trolltech.com/products/qt/gplexception/ and in the file
-** GPL_EXCEPTION.txt in this package.
+** $QT_BEGIN_LICENSE:LGPL$
+** Commercial Usage
+** Licensees holding valid Qt Commercial licenses may use this file in
+** accordance with the Qt Commercial License Agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Nokia.
 **
-** Please review the following information to ensure GNU General
-** Public Licensing requirements will be met:
-** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
-** you are unsure which license is appropriate for your use, please
-** review the following information:
-** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
-** or contact the sales department at sales@trolltech.com.
+** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Trolltech, as the sole
-** copyright holder for Qt Designer, grants users of the Qt/Eclipse
-** Integration plug-in the right for the Qt/Eclipse Integration to
-** link to functionality provided by Qt Designer and its related
-** libraries.
+** In addition, as a special exception, Nokia gives you certain
+** additional rights. These rights are described in the Nokia Qt LGPL
+** Exception version 1.0, included in the file LGPL_EXCEPTION.txt in this
+** package.
 **
-** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
-** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
-** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
-** granted herein.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3.0 as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU General Public License version 3.0 requirements will be
+** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
-** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+** If you are unsure which license is appropriate for your use, please
+** contact the sales department at http://www.qtsoftware.com/contact.
+** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
@@ -243,7 +241,7 @@ QTextCodec *Qt::codecForHtml(const QByteArray &ba)
 #endif
 
 /*!
-    \class QTextDocument qtextdocument.h
+    \class QTextDocument
     \reentrant
 
     \brief The QTextDocument class holds formatted text that can be
@@ -273,8 +271,10 @@ QTextCodec *Qt::codecForHtml(const QByteArray &ba)
     The layout of a document is determined by the documentLayout();
     you can create your own QAbstractTextDocumentLayout subclass and
     set it using setDocumentLayout() if you want to use your own
-    layout logic. The document's title can be obtained by calling the
-    documentTitle() function.
+    layout logic. The document's title and other meta-information can be
+    obtained by calling the metaInformation() function. For documents that
+    are exposed to users through the QTextEdit class, the document title
+    is also available via the QTextEdit::documentTitle() function.
 
     The toPlainText() and toHtml() convenience functions allow you to retrieve the
     contents of the document as plain text and HTML.
@@ -287,7 +287,7 @@ QTextCodec *Qt::codecForHtml(const QByteArray &ba)
     that inform connected editor widgets about the state of the undo/redo
     system.
 
-    \sa QTextCursor QTextEdit \link richtext.html Rich Text Processing\endlink
+    \sa QTextCursor, QTextEdit, \link richtext.html Rich Text Processing\endlink , {Text Object Example}
 */
 
 /*!
@@ -569,6 +569,10 @@ void QTextDocument::markContentsDirty(int from, int length)
     Otherwise, the metrics of the paint device as set on
     QAbstractTextDocumentLayout::setPaintDevice() will be used.
 
+    Using design metrics makes a layout have a width that is no longer dependent on hinting
+    and pixel-rounding. This means that WYSIWYG text layout becomes possible because the width
+    scales much more linearly based on paintdevice metrics than it would otherwise.
+
     By default, this property is false.
 */
 
@@ -657,6 +661,35 @@ qreal QTextDocument::idealWidth() const
     return textWidth();
 }
 
+/*!
+    \property QTextDocument::documentMargin
+    \since 4.5
+
+     The margin around the document. The default is 4.
+*/
+qreal QTextDocument::documentMargin() const
+{
+    Q_D(const QTextDocument);
+    return d->documentMargin;
+}
+
+void QTextDocument::setDocumentMargin(qreal margin)
+{
+    Q_D(QTextDocument);
+    if (d->documentMargin != margin) {
+        d->documentMargin = margin;
+
+        QTextFrame* root = rootFrame();
+        QTextFrameFormat format = root->frameFormat();
+        format.setMargin(margin);
+        root->setFrameFormat(format);
+
+        if (d->lout)
+            d->lout->documentChanged(0, 0, d->length());
+    }
+}
+
+
 /*!
     \property QTextDocument::indentWidth
     \since 4.4
@@ -686,10 +719,16 @@ qreal QTextDocument::indentWidth() const
 void QTextDocument::setIndentWidth(qreal width)
 {
     Q_D(QTextDocument);
-    d->indentWidth = width;
+    if (d->indentWidth != width) {
+        d->indentWidth = width;
+        if (d->lout)
+            d->lout->documentChanged(0, 0, d->length());
+    }
 }
 
 
+
+
 /*!
     \since 4.2
 
@@ -750,6 +789,7 @@ QSizeF QTextDocument::size() const
     The value of this property is undefined in documents with tables or frames.
 
     By default, if defined, this property contains a value of 1.
+    \sa lineCount(), characterCount()
 */
 int QTextDocument::blockCount() const
 {
@@ -757,6 +797,54 @@ int QTextDocument::blockCount() const
     return d->blockMap().numNodes();
 }
 
+
+/*!
+  \since 4.5
+
+  Returns the number of lines of this document (if the layout supports
+  this). Otherwise, this is identical to the number of blocks.
+
+  \sa blockCount(), characterCount()
+ */
+int QTextDocument::lineCount() const
+{
+    Q_D(const QTextDocument);
+    return d->blockMap().length(2);
+}
+
+/*!
+  \since 4.5
+
+  Returns the number of characters of this document.
+
+  \sa blockCount(), characterAt()
+ */
+int QTextDocument::characterCount() const
+{
+    Q_D(const QTextDocument);
+    return d->length();
+}
+
+/*!
+  \since 4.5
+
+  Returns the character at position \a pos, or a null character if the
+  position is out of range.
+
+  \sa characterCount()
+ */
+QChar QTextDocument::characterAt(int pos) const
+{
+    Q_D(const QTextDocument);
+    if (pos < 0 || pos >= d->length())
+        return QChar();
+    QTextDocumentPrivate::FragmentIterator fragIt = d->find(pos);
+    const QTextFragmentData * const frag = fragIt.value();
+    const int offsetInFragment = qMax(0, pos - fragIt.position());
+    return d->text.at(frag->stringPosition + offsetInFragment);
+}
+
+
 /*!
     \property QTextDocument::defaultStyleSheet
     \since 4.2
@@ -779,6 +867,7 @@ void QTextDocument::setDefaultStyleSheet(const QString &sheet)
     d->defaultStyleSheet = sheet;
     QCss::Parser parser(sheet);
     d->parsedDefaultStyleSheet = QCss::StyleSheet();
+    d->parsedDefaultStyleSheet.origin = QCss::StyleSheetOrigin_UserAgent;
     parser.parse(&d->parsedDefaultStyleSheet);
 }
 
@@ -974,11 +1063,25 @@ QString QTextDocument::toPlainText() const
 {
     Q_D(const QTextDocument);
     QString txt = d->plainText();
-    txt.replace(QTextBeginningOfFrame, QLatin1Char('\n'));
-    txt.replace(QTextEndOfFrame, QLatin1Char('\n'));
-    txt.replace(QChar::ParagraphSeparator, QLatin1Char('\n'));
-    txt.replace(QChar::LineSeparator, QLatin1Char('\n'));
-    txt.replace(QChar::Nbsp, QLatin1Char(' '));
+
+    QChar *uc = txt.data();
+    QChar *e = uc + txt.size();
+
+    for (; uc != e; ++uc) {
+        switch (uc->unicode()) {
+        case 0xfdd0: // QTextBeginningOfFrame
+        case 0xfdd1: // QTextEndOfFrame
+        case QChar::ParagraphSeparator:
+        case QChar::LineSeparator:
+            *uc = QLatin1Char('\n');
+            break;
+        case QChar::Nbsp:
+            *uc = QLatin1Char(' ');
+            break;
+        default:
+            ;
+        }
+    }
     return txt;
 }
 
@@ -1313,7 +1416,19 @@ QTextBlock QTextDocument::findBlock(int pos) const
 QTextBlock QTextDocument::findBlockByNumber(int blockNumber) const
 {
     Q_D(const QTextDocument);
-    return QTextBlock(docHandle(), d->blockMap().findNodeByIndex(blockNumber));
+    return QTextBlock(docHandle(), d->blockMap().findNode(blockNumber, 1));
+}
+
+/*!
+    \since 4.5
+    Returns the text block that contains the specified \a lineNumber.
+
+    \sa QTextBlock::firstLineNumber()
+*/
+QTextBlock QTextDocument::findBlockByLineNumber(int lineNumber) const
+{
+    Q_D(const QTextDocument);
+    return QTextBlock(docHandle(), d->blockMap().findNode(lineNumber, 2));
 }
 
 /*!
@@ -1511,6 +1626,9 @@ void QTextDocument::print(QPrinter *printer) const
 {
     Q_D(const QTextDocument);
 
+    if (!printer || !printer->isValid())
+        return;
+
     if (!d->title.isEmpty())
         printer->setDocName(d->title);
 
@@ -1686,6 +1804,8 @@ UserCanceled:
     not be found in the cache, loadResource is called to try to load
     the resource. loadResource should then use addResource to add the
     resource to the cache.
+
+    \sa QTextDocument::ResourceType
 */
 QVariant QTextDocument::resource(int type, const QUrl &name) const
 {
@@ -1898,7 +2018,7 @@ QString QTextHtmlExporter::toHtml(const QByteArray &encoding, ExportMode mode)
     rootFmt.clearProperty(QTextFormat::BackgroundBrush);
 
     QTextFrameFormat defaultFmt;
-    defaultFmt.setMargin(DefaultRootFrameMargin);
+    defaultFmt.setMargin(doc->documentMargin());
 
     if (rootFmt == defaultFmt)
         emitFrame(doc->rootFrame()->begin());
@@ -2315,7 +2435,8 @@ void QTextHtmlExporter::emitBlockAttributes(const QTextBlock &block)
     QLatin1String style(" style=\"");
     html += style;
 
-    if (block.begin().atEnd()) {
+    const bool emptyBlock = block.begin().atEnd();
+    if (emptyBlock) {
         html += QLatin1String("-qt-paragraph-type:empty;");
     }
 
@@ -2340,8 +2461,11 @@ void QTextHtmlExporter::emitBlockAttributes(const QTextBlock &block)
 
     emitPageBreakPolicy(format.pageBreakPolicy());
 
-    const QTextCharFormat blockCharFmt = block.charFormat();
-    QTextCharFormat diff = formatDifference(defaultCharFormat, blockCharFmt).toCharFormat();
+    QTextCharFormat diff;
+    if (emptyBlock) { // only print character properties when we don't expect them to be repeated by actual text in the parag
+        const QTextCharFormat blockCharFmt = block.charFormat();
+        diff = formatDifference(defaultCharFormat, blockCharFmt).toCharFormat();
+    }
 
     diff.clearProperty(QTextFormat::BackgroundBrush);
     if (format.hasProperty(QTextFormat::BackgroundBrush)) {
@@ -2353,15 +2477,6 @@ void QTextHtmlExporter::emitBlockAttributes(const QTextBlock &block)
     if (!diff.properties().isEmpty())
         emitCharFormatStyle(diff);
 
-#if 0
-    QBrush blockCharFmtBg = blockCharFmt.background();
-    if (blockCharFmtBg.style() != Qt::NoBrush) {
-        html += QLatin1String(" -qt-blockcharfmt-background-color:");
-        html += blockCharFmtBg.color().name();
-        html += QLatin1Char(';');
-    }
-#endif
-
     html += QLatin1Char('"');
 
 }
@@ -2449,18 +2564,6 @@ void QTextHtmlExporter::emitBlock(const QTextBlock &block)
 
     html += QLatin1Char('>');
 
-    QTextCharFormat blockCharFmt = block.charFormat();
-    const QTextBlockFormat blockFmt = block.blockFormat();
-
-    blockCharFmt.clearProperty(QTextFormat::BackgroundBrush);
-    if (blockFmt.hasProperty(QTextFormat::BackgroundBrush)) {
-        QBrush bg = blockFmt.background();
-        if (bg.style() != Qt::NoBrush)
-            blockCharFmt.setProperty(QTextFormat::BackgroundBrush, blockFmt.property(QTextFormat::BackgroundBrush));
-    }
-
-    defaultCharFormat.merge(blockCharFmt);
-
     QTextBlock::Iterator it = block.begin();
     if (fragmentMarkers && !it.atEnd() && block == doc->begin())
         html += QLatin1String("<!--StartFragment-->");
