@@ -1,43 +1,41 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
+** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
-** This file may be used under the terms of the GNU General Public
-** License versions 2.0 or 3.0 as published by the Free Software
-** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file.  Alternatively you may (at
-** your option) use any later version of the GNU General Public
-** License if such license has been publicly approved by Trolltech ASA
-** (or its successors, if any) and the KDE Free Qt Foundation. In
-** addition, as a special exception, Trolltech gives you certain
-** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.2, which can be found at
-** http://www.trolltech.com/products/qt/gplexception/ and in the file
-** GPL_EXCEPTION.txt in this package.
+** $QT_BEGIN_LICENSE:LGPL$
+** Commercial Usage
+** Licensees holding valid Qt Commercial licenses may use this file in
+** accordance with the Qt Commercial License Agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Nokia.
 **
-** Please review the following information to ensure GNU General
-** Public Licensing requirements will be met:
-** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
-** you are unsure which license is appropriate for your use, please
-** review the following information:
-** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
-** or contact the sales department at sales@trolltech.com.
+** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Trolltech, as the sole
-** copyright holder for Qt Designer, grants users of the Qt/Eclipse
-** Integration plug-in the right for the Qt/Eclipse Integration to
-** link to functionality provided by Qt Designer and its related
-** libraries.
+** In addition, as a special exception, Nokia gives you certain
+** additional rights. These rights are described in the Nokia Qt LGPL
+** Exception version 1.0, included in the file LGPL_EXCEPTION.txt in this
+** package.
 **
-** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
-** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
-** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
-** granted herein.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3.0 as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU General Public License version 3.0 requirements will be
+** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
-** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+** If you are unsure which license is appropriate for your use, please
+** contact the sales department at http://www.qtsoftware.com/contact.
+** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
@@ -359,6 +357,15 @@ void QTextFormatPrivate::recalcFont() const
                 if (f.fixedPitch() != value)
                     f.setFixedPitch(value);
                 break; }
+            case QTextFormat::FontStyleHint:
+                f.setStyleHint(static_cast<QFont::StyleHint>(props.at(i).value.toInt()), f.styleStrategy());
+                break;
+            case QTextFormat::FontStyleStrategy:
+                f.setStyleStrategy(static_cast<QFont::StyleStrategy>(props.at(i).value.toInt()));
+                break;
+            case QTextFormat::FontKerning:
+                f.setKerning(props.at(i).value.toBool());
+                break;
             default:
                 break;
             }
@@ -495,6 +502,9 @@ Q_GUI_EXPORT QDataStream &operator>>(QDataStream &stream, QTextFormat &fmt)
                                                 specified in percentage, with 100 as the default value.
     \value FontWordSpacing  Changes the default spacing between individual words. A positive value increases the word spacing
                                                  by the corresponding pixels; a negative value decreases the spacing.
+    \value FontStyleHint        Corresponds to the QFont::StyleHint property
+    \value FontStyleStrategy    Corresponds to the QFont::StyleStrategy property
+    \value FontKerning          Specifies whether the font has kerning turned on.
 
     \omitvalue FirstFontProperty
     \omitvalue LastFontProperty
@@ -508,7 +518,6 @@ Q_GUI_EXPORT QDataStream &operator>>(QDataStream &stream, QTextFormat &fmt)
     \value IsAnchor
     \value AnchorHref
     \value AnchorName
-
     \value ObjectType
 
     List properties
@@ -1165,8 +1174,8 @@ bool QTextFormat::operator==(const QTextFormat &rhs) const
                             normal text.
     \value AlignSubScript   Characters are placed below the baseline for
                             normal text.
-    \value AlignMiddle This is currently only implemented for inline objects. The center
-                       of the object is vertically aligned with the base line.
+    \value AlignMiddle The center of the object is vertically aligned with the base line.
+                       Currently, this is only implemented for inline objects.
     \value AlignBottom The bottom edge of the object is vertically aligned with
                        the base line.
     \value AlignTop    The top edge of the object is vertically aligned with
@@ -1379,6 +1388,74 @@ void QTextCharFormat::setUnderlineStyle(UnderlineStyle style)
 */
 
 
+/*!
+    \since 4.5
+    \fn void QTextCharFormat::setFontStyleHint(QFont::StyleHint hint, QFont::StyleStrategy strategy)
+
+    Sets the font style \a hint and \a strategy.
+
+    Qt does not support style hints on X11 since this information is not provided by the window system.
+
+    \sa setFont()
+    \sa QFont::setStyleHint()
+*/
+
+
+/*!
+    \since 4.5
+    \fn void QTextCharFormat::setFontStyleStrategy(QFont::StyleStrategy strategy)
+
+    Sets the font style \a strategy.
+
+    \sa setFont()
+    \sa QFont::setStyleStrategy()
+*/
+
+
+/*!
+    \since 4.5
+    \fn void QTextCharFormat::setFontKerning(bool enable)
+    Enables kerning for this font if \a enable is true; otherwise disables it.
+
+    When kerning is enabled, glyph metrics do not add up anymore, even for
+    Latin text. In other words, the assumption that width('a') + width('b')
+    is equal to width("ab") is not neccesairly true.
+
+    \sa setFont()
+*/
+
+
+/*!
+    \fn QTextCharFormat::StyleHint QTextCharFormat::fontStyleHint() const
+    \since 4.5
+
+    Returns the font style hint.
+
+    \sa setFontStyleHint(), font()
+*/
+
+
+/*!
+    \since 4.5
+    \fn QTextCharFormat::StyleStrategy QTextCharFormat::fontStyleStrategy() const
+
+    Returns the current font style strategy.
+
+    \sa setFontStyleStrategy()
+    \sa font()
+*/
+
+
+/*!
+    \since 4.5
+    \fn  bool QTextCharFormat::fontKerning() const
+    Returns true if the font kerning is enabled.
+
+    \sa setFontKerning()
+    \sa font()
+*/
+
+
 /*!
     \fn void QTextCharFormat::setFontFixedPitch(bool fixedPitch)
 
@@ -1485,7 +1562,7 @@ void QTextCharFormat::setUnderlineStyle(UnderlineStyle style)
     \fn void QTextCharFormat::setAnchorHref(const QString &value)
 
     Sets the hypertext link for the text format to the given \a value.
-    This is typically a URL like "http://www.trolltech.com/index.html".
+    This is typically a URL like "http://qtsoftware.com/index.html".
 
     The anchor will be displayed with the \a value as its display text;
     if you want to display different text call setAnchorNames().
@@ -1657,6 +1734,9 @@ void QTextCharFormat::setFont(const QFont &font)
     setFontWordSpacing(font.wordSpacing());
     if (font.letterSpacingType() == QFont::PercentageSpacing)
         setFontLetterSpacing(font.letterSpacing());
+    setFontStyleHint(font.styleHint());
+    setFontStyleStrategy(font.styleStrategy());
+    setFontKerning(font.kerning());
 }
 
 /*!
@@ -2048,6 +2128,8 @@ QTextListFormat::QTextListFormat(const QTextFormat &fmt)
     \fn void QTextListFormat::setIndent(int indentation)
 
     Sets the list format's \a indentation.
+    The indentation is multiplied by the QTextDocument::indentWidth
+    property to get the effective indent in pixels.
 
     \sa indent()
 */
@@ -2057,6 +2139,8 @@ QTextListFormat::QTextListFormat(const QTextFormat &fmt)
     \fn int QTextListFormat::indent() const
 
     Returns the list format's indentation.
+    The indentation is multiplied by the QTextDocument::indentWidth
+    property to get the effective indent in pixels.
 
     \sa setIndent()
 */
@@ -2655,7 +2739,6 @@ QTextImageFormat::QTextImageFormat(const QTextFormat &fmt)
     \sa setName()
 */
 
-
 /*!
     \fn void QTextImageFormat::setWidth(qreal width)
 
@@ -2665,6 +2748,7 @@ QTextImageFormat::QTextImageFormat(const QTextFormat &fmt)
 */
 
 
+// ### Qt5 qreal replace with a QTextLength
 /*!
     \fn qreal QTextImageFormat::width() const
 
@@ -2683,6 +2767,7 @@ QTextImageFormat::QTextImageFormat(const QTextFormat &fmt)
 */
 
 
+// ### Qt5 qreal replace with a QTextLength
 /*!
     \fn qreal QTextImageFormat::height() const
 
