@@ -1,43 +1,41 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
+** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
-** This file may be used under the terms of the GNU General Public
-** License versions 2.0 or 3.0 as published by the Free Software
-** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file.  Alternatively you may (at
-** your option) use any later version of the GNU General Public
-** License if such license has been publicly approved by Trolltech ASA
-** (or its successors, if any) and the KDE Free Qt Foundation. In
-** addition, as a special exception, Trolltech gives you certain
-** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.2, which can be found at
-** http://www.trolltech.com/products/qt/gplexception/ and in the file
-** GPL_EXCEPTION.txt in this package.
+** $QT_BEGIN_LICENSE:LGPL$
+** Commercial Usage
+** Licensees holding valid Qt Commercial licenses may use this file in
+** accordance with the Qt Commercial License Agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Nokia.
 **
-** Please review the following information to ensure GNU General
-** Public Licensing requirements will be met:
-** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
-** you are unsure which license is appropriate for your use, please
-** review the following information:
-** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
-** or contact the sales department at sales@trolltech.com.
+** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Trolltech, as the sole
-** copyright holder for Qt Designer, grants users of the Qt/Eclipse
-** Integration plug-in the right for the Qt/Eclipse Integration to
-** link to functionality provided by Qt Designer and its related
-** libraries.
+** In addition, as a special exception, Nokia gives you certain
+** additional rights. These rights are described in the Nokia Qt LGPL
+** Exception version 1.0, included in the file LGPL_EXCEPTION.txt in this
+** package.
 **
-** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
-** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
-** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
-** granted herein.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3.0 as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU General Public License version 3.0 requirements will be
+** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
-** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+** If you are unsure which license is appropriate for your use, please
+** contact the sales department at http://www.qtsoftware.com/contact.
+** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
@@ -47,13 +45,14 @@
 #include <qdebug.h>
 #include "qtexttable_p.h"
 #include "qvarlengtharray.h"
+#include "private/qfunctions_p.h"
 
 #include <stdlib.h>
 
 QT_BEGIN_NAMESPACE
 
 /*!
-    \class QTextTableCell qtexttable.h
+    \class QTextTableCell
     \reentrant
 
     \brief The QTextTableCell class represents the properties of a
@@ -121,6 +120,7 @@ void QTextTableCell::setFormat(const QTextCharFormat &format)
 {
     QTextCharFormat fmt = format;
     fmt.clearProperty(QTextFormat::ObjectIndex);
+    fmt.setObjectType(QTextFormat::TableCellObject);
     QTextDocumentPrivate *p = table->docHandle();
     QTextDocumentPrivate::FragmentIterator frag(&p->fragmentMap(), fragment);
 
@@ -140,11 +140,24 @@ QTextCharFormat QTextTableCell::format() const
     QTextDocumentPrivate *p = table->docHandle();
     QTextFormatCollection *c = p->formatCollection();
 
-    QTextCharFormat fmt = c->charFormat(QTextDocumentPrivate::FragmentIterator(&p->fragmentMap(), fragment)->format);
+    QTextCharFormat fmt = c->charFormat(tableCellFormatIndex());
     fmt.setObjectType(QTextFormat::TableCellObject);
     return fmt;
 }
 
+/*!
+    \since 4.5
+
+    Returns the index of the tableCell's format in the document's internal list of formats.
+
+    \sa QTextDocument::allFormats()
+*/
+int QTextTableCell::tableCellFormatIndex() const
+{
+    QTextDocumentPrivate *p = table->docHandle();
+    return QTextDocumentPrivate::FragmentIterator(&p->fragmentMap(), fragment)->format;
+}
+
 /*!
     Returns the number of the row in the table that contains this cell.
 
@@ -325,6 +338,9 @@ QTextTable *QTextTablePrivate::createTable(QTextDocumentPrivate *pieceTable, int
     // add block after table
     QTextCharFormat charFmt;
     charFmt.setObjectIndex(table->objectIndex());
+    charFmt.setObjectType(QTextFormat::TableCellObject);
+
+
     int charIdx = pieceTable->formatCollection()->indexForFormat(charFmt);
     int cellIdx = pieceTable->formatCollection()->indexForFormat(QTextBlockFormat());
 
@@ -361,12 +377,12 @@ struct QFragmentFindHelper
     const QTextDocumentPrivate::FragmentMap &fragmentMap;
 };
 
-static inline bool operator<(int fragment, const QFragmentFindHelper &helper)
+Q_STATIC_GLOBAL_INLINE_OPERATOR bool operator<(int fragment, const QFragmentFindHelper &helper)
 {
     return helper.fragmentMap.position(fragment) < helper.pos;
 }
 
-static inline bool operator<(const QFragmentFindHelper &helper, int fragment)
+Q_STATIC_GLOBAL_INLINE_OPERATOR bool operator<(const QFragmentFindHelper &helper, int fragment)
 {
     return helper.pos < helper.fragmentMap.position(fragment);
 }
@@ -471,7 +487,7 @@ void QTextTablePrivate::update() const
 
 
 /*!
-    \class QTextTable qtexttable.h
+    \class QTextTable
     \reentrant
 
     \brief The QTextTable class represents a table in a QTextDocument.
@@ -509,7 +525,12 @@ void QTextTablePrivate::update() const
     Rows and columns within a QTextTable can be merged and split using
     the mergeCells() and splitCell() functions. However, only cells that span multiple
     rows or columns can be split. (Merging or splitting does not increase or decrease
-    the number of rows and columns.)
+    the number of rows and columns.) 
+
+    Note that if you have merged multiple columns and rows into one cell, you will not
+    be able to split the merged cell into new cells spanning over more than one row 
+    or column. To be able to split cells spanning over several rows and columns you 
+    need to do this over several iterations.
 
     \table 80%
     \row
@@ -537,7 +558,7 @@ void QTextTablePrivate::update() const
 /*! \internal
  */
 QTextTable::QTextTable(QTextDocument *doc)
-    : QTextFrame(*new QTextTablePrivate, doc)
+    : QTextFrame(*new QTextTablePrivate(doc), doc)
 {
 }
 
@@ -645,7 +666,7 @@ void QTextTable::resize(int rows, int cols)
 
     Inserts a number of \a rows before the row with the specified \a index.
 
-    \sa resize() insertColumns() removeRows() removeColumns()
+    \sa resize() insertColumns() removeRows() removeColumns() appendRows() appendColumns()
 */
 void QTextTable::insertRows(int pos, int num)
 {
@@ -707,7 +728,7 @@ void QTextTable::insertRows(int pos, int num)
 
     Inserts a number of \a columns before the column with the specified \a index.
 
-    \sa insertRows() resize() removeRows() removeColumns()
+    \sa insertRows() resize() removeRows() removeColumns() appendRows() appendColumns()
 */
 void QTextTable::insertColumns(int pos, int num)
 {
@@ -764,12 +785,34 @@ void QTextTable::insertColumns(int pos, int num)
     p->endEditBlock();
 }
 
+/*!
+    \since 4.5
+    Appends \a count rows at the bottom of the table.
+
+    \sa insertColumns() insertRows() resize() removeRows() removeColumns() appendColumns()
+*/
+void QTextTable::appendRows(int count)
+{
+    insertRows(rows(), count);
+}
+
+/*!
+    \since 4.5
+    Appends \a count columns at the right side of the table.
+
+    \sa insertColumns() insertRows() resize() removeRows() removeColumns() appendRows()
+*/
+void QTextTable::appendColumns(int count)
+{
+    insertColumns(columns(), count);
+}
+
 /*!
     \fn void QTextTable::removeRows(int index, int rows)
 
     Removes a number of \a rows starting with the row at the specified \a index.
 
-    \sa insertRows(), insertColumns(), resize(), removeColumns()
+    \sa insertRows(), insertColumns(), resize(), removeColumns() appendRows() appendColumns()
 */
 void QTextTable::removeRows(int pos, int num)
 {
@@ -831,7 +874,7 @@ void QTextTable::removeRows(int pos, int num)
     Removes a number of \a columns starting with the column at the specified
     \a index.
 
-    \sa insertRows() insertColumns() removeRows() resize()
+    \sa insertRows() insertColumns() removeRows() resize() appendRows() appendColumns()
 */
 void QTextTable::removeColumns(int pos, int num)
 {
