@@ -1,43 +1,41 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
+** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
-** This file may be used under the terms of the GNU General Public
-** License versions 2.0 or 3.0 as published by the Free Software
-** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file.  Alternatively you may (at
-** your option) use any later version of the GNU General Public
-** License if such license has been publicly approved by Trolltech ASA
-** (or its successors, if any) and the KDE Free Qt Foundation. In
-** addition, as a special exception, Trolltech gives you certain
-** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.2, which can be found at
-** http://www.trolltech.com/products/qt/gplexception/ and in the file
-** GPL_EXCEPTION.txt in this package.
+** $QT_BEGIN_LICENSE:LGPL$
+** Commercial Usage
+** Licensees holding valid Qt Commercial licenses may use this file in
+** accordance with the Qt Commercial License Agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Nokia.
 **
-** Please review the following information to ensure GNU General
-** Public Licensing requirements will be met:
-** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
-** you are unsure which license is appropriate for your use, please
-** review the following information:
-** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
-** or contact the sales department at sales@trolltech.com.
+** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Trolltech, as the sole
-** copyright holder for Qt Designer, grants users of the Qt/Eclipse
-** Integration plug-in the right for the Qt/Eclipse Integration to
-** link to functionality provided by Qt Designer and its related
-** libraries.
+** In addition, as a special exception, Nokia gives you certain
+** additional rights. These rights are described in the Nokia Qt LGPL
+** Exception version 1.0, included in the file LGPL_EXCEPTION.txt in this
+** package.
 **
-** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
-** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
-** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
-** granted herein.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3.0 as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU General Public License version 3.0 requirements will be
+** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
-** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+** If you are unsure which license is appropriate for your use, please
+** contact the sales department at http://www.qtsoftware.com/contact.
+** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
@@ -333,9 +331,9 @@ QString QCalendarMonthValidator::text(const QDate &date, int repeat) const
             str += QLatin1String("0");
         return str + QString::number(date.month());
     } else if (repeat == 3) {
-        return m_locale.monthName(date.month(), QLocale::ShortFormat);
+        return m_locale.standaloneMonthName(date.month(), QLocale::ShortFormat);
     } else if (repeat >= 4) {
-        return m_locale.monthName(date.month(), QLocale::LongFormat);
+        return m_locale.standaloneMonthName(date.month(), QLocale::LongFormat);
     }
     return QString();
 }
@@ -910,6 +908,7 @@ public:
     virtual void keyboardSearch(const QString & search) { Q_UNUSED(search) }
 
 signals:
+    void showDate(const QDate &date);
     void changeDate(const QDate &date, bool changeMonth);
     void clicked(const QDate &date);
     void editingFinished();
@@ -919,6 +918,9 @@ protected:
     void mousePressEvent(QMouseEvent *event);
     void mouseMoveEvent(QMouseEvent *event);
     void mouseReleaseEvent(QMouseEvent *event);
+#ifndef QT_NO_WHEELEVENT
+    void wheelEvent(QWheelEvent *event);
+#endif
     void keyPressEvent(QKeyEvent *event);
     bool event(QEvent *event);
 
@@ -1054,16 +1056,12 @@ void QCalendarModel::cellForDate(const QDate &date, int *row, int *column) const
 QString QCalendarModel::dayName(Qt::DayOfWeek day) const
 {
     switch (m_horizontalHeaderFormat) {
-        case QCalendarWidget::SingleLetterDayNames:
-            if (m_view->locale().language() == QLocale::Chinese) {
-                return m_view->locale().dayName(day, QLocale::ShortFormat).right(1);
-            } else if (m_view->locale().language() == QLocale::Arabic) {
-                //some Arabic locales have the same letter at beginning which
-                //makes the various days indistinguishable
-                QLocale l(QLocale::Arabic, QLocale::Egypt);
-                return l.dayName(day, QLocale::ShortFormat).left(1);
-            }
-            return m_view->locale().dayName(day, QLocale::ShortFormat).left(1);
+        case QCalendarWidget::SingleLetterDayNames: {
+            QString standaloneDayName = m_view->locale().standaloneDayName(day, QLocale::NarrowFormat);
+            if (standaloneDayName == m_view->locale().dayName(day, QLocale::NarrowFormat))
+                return standaloneDayName.left(1);
+            return standaloneDayName;
+        }
         case QCalendarWidget::ShortDayNames:
             return m_view->locale().dayName(day, QLocale::ShortFormat);
         case QCalendarWidget::LongDayNames:
@@ -1375,6 +1373,18 @@ void QCalendarView::keyPressEvent(QKeyEvent *event)
     QTableView::keyPressEvent(event);
 }
 
+#ifndef QT_NO_WHEELEVENT
+void QCalendarView::wheelEvent(QWheelEvent *event)
+{
+    const int numDegrees = event->delta() / 8;
+    const int numSteps = numDegrees / 15;
+    const QModelIndex index = currentIndex();
+    QDate currentDate = static_cast<QCalendarModel*>(model())->dateForCell(index.row(), index.column());
+    currentDate = currentDate.addMonths(-numSteps);
+    emit showDate(currentDate);
+}
+#endif
+
 bool QCalendarView::event(QEvent *event)
 {
 #ifdef QT_KEYPAD_NAVIGATION
@@ -1574,6 +1584,7 @@ public:
     void update();
     void paintCell(QPainter *painter, const QRect &rect, const QDate &date) const;
 
+    void _q_slotShowDate(const QDate &date);
     void _q_slotChangeDate(const QDate &date);
     void _q_slotChangeDate(const QDate &date, bool changeMonth);
     void _q_editingFinished();
@@ -1588,7 +1599,7 @@ public:
     void updateMonthMenu();
     void updateMonthMenuNames();
     void updateNavigationBar();
-    void updateCurrentPage(QDate &newDate);
+    void updateCurrentPage(const QDate &newDate);
     inline QDate getCurrentDate();
     void setNavigatorEnabled(bool enable);
 
@@ -1611,6 +1622,7 @@ public:
 
     bool navBarVisible;
     mutable QSize cachedSizeHint;
+    Qt::FocusPolicy oldFocusPolicy;
 };
 
 void QCalendarDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option,
@@ -1646,6 +1658,7 @@ QCalendarWidgetPrivate::QCalendarWidgetPrivate()
     m_navigator = 0;
     m_dateEditEnabled = false;
     navBarVisible = true;
+    oldFocusPolicy = Qt::StrongFocus;
 }
 
 void QCalendarWidgetPrivate::setNavigatorEnabled(bool enable)
@@ -1691,9 +1704,6 @@ void QCalendarWidgetPrivate::createNavigationBar(QWidget *widget)
     updateButtonIcons();
     prevMonth->setAutoRepeat(true);
     nextMonth->setAutoRepeat(true);
-    prevMonth->setFocusProxy(m_view);
-    nextMonth->setFocusProxy(m_view);
-
 
     monthButton = new QCalToolButton(navBarBackground);
     monthButton->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Minimum);
@@ -1701,7 +1711,7 @@ void QCalendarWidgetPrivate::createNavigationBar(QWidget *widget)
     monthButton->setPopupMode(QToolButton::InstantPopup);
     monthMenu = new QMenu(monthButton);
     for (int i = 1; i <= 12; i++) {
-        QString monthName(q->locale().monthName(i, QLocale::LongFormat));
+        QString monthName(q->locale().standaloneMonthName(i, QLocale::LongFormat));
         QAction *act = monthMenu->addAction(monthName);
         act->setData(i);
         monthToAction[i] = act;
@@ -1788,15 +1798,16 @@ void QCalendarWidgetPrivate::updateMonthMenuNames()
     Q_Q(QCalendarWidget);
 
     for (int i = 1; i <= 12; i++) {
-        QString monthName(q->locale().monthName(i, QLocale::LongFormat));
+        QString monthName(q->locale().standaloneMonthName(i, QLocale::LongFormat));
         monthToAction[i]->setText(monthName);
     }
 }
 
-void QCalendarWidgetPrivate::updateCurrentPage(QDate &newDate)
+void QCalendarWidgetPrivate::updateCurrentPage(const QDate &date)
 {
     Q_Q(QCalendarWidget);
 
+    QDate newDate = date;
     QDate minDate = q->minimumDate();
     QDate maxDate = q->maximumDate();
     if (minDate.isValid()&& minDate.daysTo(newDate) < 0)
@@ -1841,8 +1852,11 @@ void QCalendarWidgetPrivate::_q_nextMonthClicked()
 
 void QCalendarWidgetPrivate::_q_yearEditingFinished()
 {
+    Q_Q(QCalendarWidget);
     yearButton->setText(yearEdit->text());
     yearEdit->hide();
+    q->setFocusPolicy(oldFocusPolicy);
+    qApp->removeEventFilter(q);
     spaceHolder->changeSize(0, 0);
     yearButton->show();
     QDate currentDate = getCurrentDate();
@@ -1852,12 +1866,16 @@ void QCalendarWidgetPrivate::_q_yearEditingFinished()
 
 void QCalendarWidgetPrivate::_q_yearClicked()
 {
+    Q_Q(QCalendarWidget);
     //show the spinbox on top of the button
     yearEdit->setGeometry(yearButton->x(), yearButton->y(),
                           yearEdit->sizeHint().width(), yearButton->height());
     spaceHolder->changeSize(yearButton->width(), 0);
     yearButton->hide();
+    oldFocusPolicy = q->focusPolicy();
+    q->setFocusPolicy(Qt::NoFocus);
     yearEdit->show();
+    qApp->installEventFilter(q);
     yearEdit->raise();
     yearEdit->selectAll();
     yearEdit->setFocus(Qt::MouseFocusReason);
@@ -1881,7 +1899,7 @@ void QCalendarWidgetPrivate::updateNavigationBar()
 {
     Q_Q(QCalendarWidget);
 
-    QString monthName = q->locale().monthName(m_model->m_shownMonth, QLocale::LongFormat);
+    QString monthName = q->locale().standaloneMonthName(m_model->m_shownMonth, QLocale::LongFormat);
 
     monthButton->setText(monthName);
     yearButton->setText(QString::number(m_model->m_shownYear));
@@ -1907,6 +1925,11 @@ void QCalendarWidgetPrivate::paintCell(QPainter *painter, const QRect &rect, con
     q->paintCell(painter, rect, date);
 }
 
+void QCalendarWidgetPrivate::_q_slotShowDate(const QDate &date)
+{
+    updateCurrentPage(date);
+}
+
 void QCalendarWidgetPrivate::_q_slotChangeDate(const QDate &date)
 {
     _q_slotChangeDate(date, true);
@@ -2058,6 +2081,8 @@ QCalendarWidget::QCalendarWidget(QWidget *parent)
     setFocusProxy(d->m_view);
     setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
 
+    connect(d->m_view, SIGNAL(showDate(QDate)),
+            this, SLOT(_q_slotShowDate(QDate)));
     connect(d->m_view, SIGNAL(changeDate(QDate,bool)),
             this, SLOT(_q_slotChangeDate(QDate,bool)));
     connect(d->m_view, SIGNAL(clicked(QDate)),
@@ -2136,6 +2161,13 @@ QSize QCalendarWidget::minimumSizeHint() const
     if (verticalHeaderFormat() == QCalendarWidget::NoVerticalHeader) {
         cols = 7;
         startCol = 1;
+    } else {
+        for (int i = 1; i <= 6; i++) {
+            QFontMetrics fm(d->m_model->formatForCell(i, 0).font());
+            for (int j = 1; j < end; j++)
+                w = qMax(w, fm.width(QString::number(j)) + marginH);
+            h = qMax(h, fm.height());
+        }
     }
 
     QFontMetrics fm(d->m_model->formatForCell(1, 1).font());
@@ -2167,7 +2199,7 @@ QSize QCalendarWidget::minimumSizeHint() const
         QFontMetrics fm = d->monthButton->fontMetrics();
         int monthW = 0;
         for (int i = 1; i < 12; i++) {
-            QString monthName = locale().monthName(i, QLocale::LongFormat);
+            QString monthName = locale().standaloneMonthName(i, QLocale::LongFormat);
             monthW = qMax(monthW, fm.boundingRect(monthName).width());
         }
         const int buttonDecoMargin = d->monthButton->sizeHint().width() - fm.boundingRect(d->monthButton->text()).width();
@@ -2555,11 +2587,12 @@ QCalendarWidget::HorizontalHeaderFormat QCalendarWidget::horizontalHeaderFormat(
 }
 
 
-/*! \enum QCalendarWidget::VerticalHeaderFormat
+/*! 
+    \enum QCalendarWidget::VerticalHeaderFormat
 
     This enum type defines the various formats the vertical header can display.
 
-    \value ISOWeekNumbers The header displays a ISO week numbers \l QDate::weekNumber().
+    \value ISOWeekNumbers The header displays ISO week numbers as described by \l QDate::weekNumber().
     \value NoVerticalHeader The header is hidden.
 
     \sa verticalHeaderFormat(), HorizontalHeaderFormat
@@ -2758,7 +2791,7 @@ QTextCharFormat QCalendarWidget::dateTextFormat(const QDate &date) const
 void QCalendarWidget::setDateTextFormat(const QDate &date, const QTextCharFormat &format)
 {
     Q_D(QCalendarWidget);
-    if ( date.isNull() && !format.isValid() )
+    if (date.isNull())
         d->m_model->m_dateFormats.clear();
     else
         d->m_model->m_dateFormats[date] = format;
@@ -2902,9 +2935,12 @@ void QCalendarWidget::updateCells()
 /*!
     \fn void QCalendarWidget::clicked(const QDate &date)
 
-    This signal is emitted when a mouse button is clicked. The date the
-    mouse was clicked on is specified by \a date. The signal is only
-    emitted when clicked on a valid date.
+    This signal is emitted when a mouse button is clicked. The date
+    the mouse was clicked on is specified by \a date. The signal is
+    only emitted when clicked on a valid date, e.g., dates are not
+    outside the minimumDate() and maximumDate(). If the selection mode
+    is NoSelection, this signal will not be emitted.
+
 */
 
 /*!
@@ -2991,6 +3027,21 @@ bool QCalendarWidget::event(QEvent *event)
     return QWidget::event(event);
 }
 
+/*!
+  \reimp
+*/
+bool QCalendarWidget::eventFilter(QObject *watched, QEvent *event)
+{
+    Q_D(QCalendarWidget);
+    if (event->type() == QEvent::MouseButtonPress && d->yearEdit->hasFocus() && !QRect(d->yearEdit->mapToGlobal(QPoint(0, 0)), d->yearEdit->size()).contains(static_cast<QMouseEvent *>(event)->globalPos())) {
+        event->accept();
+        d->_q_yearEditingFinished();
+        setFocus();
+        return true;
+    }
+    return QWidget::eventFilter(watched, event);
+}
+
 /*!
   \reimp
 */
