@@ -1,43 +1,41 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
+** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
-** This file may be used under the terms of the GNU General Public
-** License versions 2.0 or 3.0 as published by the Free Software
-** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file.  Alternatively you may (at
-** your option) use any later version of the GNU General Public
-** License if such license has been publicly approved by Trolltech ASA
-** (or its successors, if any) and the KDE Free Qt Foundation. In
-** addition, as a special exception, Trolltech gives you certain
-** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.2, which can be found at
-** http://www.trolltech.com/products/qt/gplexception/ and in the file
-** GPL_EXCEPTION.txt in this package.
+** $QT_BEGIN_LICENSE:LGPL$
+** Commercial Usage
+** Licensees holding valid Qt Commercial licenses may use this file in
+** accordance with the Qt Commercial License Agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Nokia.
 **
-** Please review the following information to ensure GNU General
-** Public Licensing requirements will be met:
-** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
-** you are unsure which license is appropriate for your use, please
-** review the following information:
-** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
-** or contact the sales department at sales@trolltech.com.
+** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Trolltech, as the sole
-** copyright holder for Qt Designer, grants users of the Qt/Eclipse
-** Integration plug-in the right for the Qt/Eclipse Integration to
-** link to functionality provided by Qt Designer and its related
-** libraries.
+** In addition, as a special exception, Nokia gives you certain
+** additional rights. These rights are described in the Nokia Qt LGPL
+** Exception version 1.0, included in the file LGPL_EXCEPTION.txt in this
+** package.
 **
-** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
-** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
-** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
-** granted herein.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3.0 as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU General Public License version 3.0 requirements will be
+** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
-** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+** If you are unsure which license is appropriate for your use, please
+** contact the sales department at http://www.qtsoftware.com/contact.
+** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
@@ -135,8 +133,8 @@ QSize QDockWidgetTitleButton::sizeHint() const
     int size = 2*style()->pixelMetric(QStyle::PM_DockWidgetTitleBarButtonMargin, 0, this);
     if (!icon().isNull()) {
         int iconSize = style()->pixelMetric(QStyle::PM_SmallIconSize, 0, this);
-        const QPixmap pm = icon().pixmap(iconSize);
-        size += qMax(pm.width(), pm.height());
+        QSize sz = icon().actualSize(QSize(iconSize, iconSize));
+        size += qMax(sz.width(), sz.height());
     }
 
     return QSize(size, size);
@@ -163,9 +161,7 @@ void QDockWidgetTitleButton::paintEvent(QPaintEvent *)
     opt.init(this);
     opt.state |= QStyle::State_AutoRaise;
 
-#ifdef Q_WS_MAC
-    if (!qobject_cast<QMacStyle *>(style()))
-#endif
+    if (style()->styleHint(QStyle::SH_DockWidget_ButtonsHaveFrame, 0, this))
     {
         if (isEnabled() && underMouse() && !isChecked() && !isDown())
             opt.state |= QStyle::State_Raised;
@@ -201,14 +197,21 @@ QDockWidgetLayout::~QDockWidgetLayout()
 }
 
 bool QDockWidgetLayout::nativeWindowDeco() const
+{
+    return nativeWindowDeco(parentWidget()->isWindow());
+}
+
+bool QDockWidgetLayout::nativeWindowDeco(bool floating) const
 {
 #if defined(Q_WS_X11) || defined(Q_WS_QWS) || defined(Q_OS_WINCE)
+    Q_UNUSED(floating);
     return false;
 #else
-    return parentWidget()->isWindow() && item_list[QDockWidgetLayout::TitleBar] == 0;
+    return floating && item_list[QDockWidgetLayout::TitleBar] == 0;
 #endif
 }
 
+
 void QDockWidgetLayout::addItem(QLayoutItem*)
 {
     qWarning() << "QDockWidgetLayout::addItem(): please use QDockWidgetLayout::setWidget()";
@@ -267,11 +270,7 @@ QSize QDockWidgetLayout::sizeFromContent(const QSize &content, bool floating) co
     }
 
     QDockWidget *w = qobject_cast<QDockWidget*>(parentWidget());
-    bool customTitleBar = item_list[TitleBar] != 0;
-    bool nativeDeco = floating && !customTitleBar;
-#if defined(Q_WS_X11) || defined(Q_WS_QWS) || defined(Q_OS_WINCE)
-    nativeDeco = false;
-#endif
+    const bool nativeDeco = nativeWindowDeco(floating);
 
     int fw = floating && !nativeDeco
             ? w->style()->pixelMetric(QStyle::PM_DockWidgetFrameWidth, 0, w)
@@ -293,8 +292,11 @@ QSize QDockWidgetLayout::sizeFromContent(const QSize &content, bool floating) co
     if (content.height() < 0)
         result.setHeight(-1);
 
-    QSize min = w->minimumSize();
-    QSize max = w->maximumSize();
+    int left, top, right, bottom;
+    w->getContentsMargins(&left, &top, &right, &bottom);
+    //we need to substract the contents margin (it will be added by the caller)
+    QSize min = w->minimumSize() - QSize(left + right, top + bottom);
+    QSize max = w->maximumSize() - QSize(left + right, top + bottom);
 
     /* A floating dockwidget will automatically get its minimumSize set to the layout's
        minimum size + deco. We're *not* interested in this, we only take minimumSize()
@@ -404,10 +406,14 @@ int QDockWidgetLayout::minimumTitleWidth() const
 
     QSize closeSize(0, 0);
     QSize floatSize(0, 0);
-    if (QLayoutItem *item = item_list[CloseButton])
-        closeSize = item->sizeHint();
-    if (QLayoutItem *item = item_list[FloatButton])
-        floatSize = item->sizeHint();
+    if (hasFeature(q, QDockWidget::DockWidgetClosable)) {
+        if (QLayoutItem *item = item_list[CloseButton])
+            closeSize = item->widget()->sizeHint();
+    }
+    if (hasFeature(q, QDockWidget::DockWidgetFloatable)) {
+        if (QLayoutItem *item = item_list[FloatButton])
+            floatSize = item->widget()->sizeHint();
+    }
 
     int titleHeight = this->titleHeight();
 
@@ -439,6 +445,7 @@ int QDockWidgetLayout::titleHeight() const
     QFontMetrics titleFontMetrics = q->fontMetrics();
 #ifdef Q_WS_MAC
     if (qobject_cast<QMacStyle *>(q->style())) {
+        //### this breaks on proxy styles.  (But is this code still called?)
         QFont font = qt_app_fonts_hash()->value("QToolButton", q->font());
         titleFontMetrics = QFontMetrics(font);
     }
@@ -553,7 +560,7 @@ QSize QDockWidgetItem::maximumSize() const
 QSize QDockWidgetItem::sizeHint() const
 {
     if (QLayoutItem *item = dockWidgetChildItem()) {
-        return dockWidgetLayout()->sizeFromContent(item->sizeHint(), false);
+         return dockWidgetLayout()->sizeFromContent(item->sizeHint(), false);
     } else {
         return QWidgetItem::sizeHint();
     }
@@ -782,7 +789,7 @@ bool QDockWidgetPrivate::isAnimating() const
     return (void*)mainWinLayout->pluggingWidget == (void*)q;
 }
 
-void QDockWidgetPrivate::mousePressEvent(QMouseEvent *event)
+bool QDockWidgetPrivate::mousePressEvent(QMouseEvent *event)
 {
 #if !defined(QT_NO_MAINWINDOW)
     Q_Q(QDockWidget);
@@ -793,35 +800,29 @@ void QDockWidgetPrivate::mousePressEvent(QMouseEvent *event)
     if (!layout->nativeWindowDeco()) {
         QRect titleArea = layout->titleArea();
 
-        if (event->button() != Qt::LeftButton)
-            return;
-        if (!titleArea.contains(event->pos()))
-            return;
-        // check if the tool window is movable... do nothing if it is not
-        if (!hasFeature(q, QDockWidget::DockWidgetMovable))
-            return;
-
-        if (qobject_cast<QMainWindow*>(q->parentWidget()) == 0)
-            return;
-
-        if (isAnimating())
-            return;
-
-        if (state != 0)
-            return;
+        if (event->button() != Qt::LeftButton ||
+            !titleArea.contains(event->pos()) ||
+            // check if the tool window is movable... do nothing if it
+            // is not (but allow moving if the window is floating)
+            (!hasFeature(q, QDockWidget::DockWidgetMovable) && !q->isFloating()) ||
+            qobject_cast<QMainWindow*>(q->parentWidget()) == 0 ||
+            isAnimating() || state != 0) {
+            return false;
+        }
 
         initDrag(event->pos(), false);
 
-        if (state == 0)
-            return;
+        if (state)
+            state->ctrlDrag = hasFeature(q, QDockWidget::DockWidgetFloatable) && event->modifiers() & Qt::ControlModifier;
 
-        state->ctrlDrag = event->modifiers() & Qt::ControlModifier;
+        return true;
     }
 
 #endif // !defined(QT_NO_MAINWINDOW)
+    return false;
 }
 
-void QDockWidgetPrivate::mouseDoubleClickEvent(QMouseEvent *event)
+bool QDockWidgetPrivate::mouseDoubleClickEvent(QMouseEvent *event)
 {
     Q_Q(QDockWidget);
 
@@ -830,23 +831,23 @@ void QDockWidgetPrivate::mouseDoubleClickEvent(QMouseEvent *event)
     if (!layout->nativeWindowDeco()) {
         QRect titleArea = layout->titleArea();
 
-        if (event->button() != Qt::LeftButton)
-            return;
-        if (!titleArea.contains(event->pos()))
-            return;
-        if (!hasFeature(q, QDockWidget::DockWidgetFloatable))
-            return;
-        _q_toggleTopLevel();
+        if (event->button() == Qt::LeftButton && titleArea.contains(event->pos()) &&
+            hasFeature(q, QDockWidget::DockWidgetFloatable)) {
+            _q_toggleTopLevel();
+            return true;
+        }
     }
+    return false;
 }
 
-void QDockWidgetPrivate::mouseMoveEvent(QMouseEvent *event)
+bool QDockWidgetPrivate::mouseMoveEvent(QMouseEvent *event)
 {
+    bool ret = false;
 #if !defined(QT_NO_MAINWINDOW)
     Q_Q(QDockWidget);
 
     if (!state)
-        return;
+        return ret;
 
     QDockWidgetLayout *dwlayout
         = qobject_cast<QDockWidgetLayout*>(q->layout());
@@ -863,6 +864,7 @@ void QDockWidgetPrivate::mouseMoveEvent(QMouseEvent *event)
 #else
             q->grabMouse();
 #endif
+            ret = true;
         }
     }
 
@@ -872,25 +874,25 @@ void QDockWidgetPrivate::mouseMoveEvent(QMouseEvent *event)
 
         if (!state->ctrlDrag)
             mwlayout->hover(state->widgetItem, event->globalPos());
+
+        ret = true;
     }
 
 #endif // !defined(QT_NO_MAINWINDOW)
+    return ret;
 }
 
-void QDockWidgetPrivate::mouseReleaseEvent(QMouseEvent *event)
+bool QDockWidgetPrivate::mouseReleaseEvent(QMouseEvent *event)
 {
 #if !defined(QT_NO_MAINWINDOW)
 
-    if (event->button() != Qt::LeftButton)
-        return;
-    if (!state)
-        return;
-    if (state->nca)
-        return;
-
-    endDrag();
+    if (event->button() == Qt::LeftButton && state && !state->nca) {
+        endDrag();
+        return true; //filter out the event
+    }
 
 #endif // !defined(QT_NO_MAINWINDOW)
+    return false;
 }
 
 void QDockWidgetPrivate::nonClientAreaMouseEvent(QMouseEvent *event)
@@ -994,10 +996,9 @@ void QDockWidgetPrivate::unplug(const QRect &rect)
     Q_Q(QDockWidget);
     QRect r = rect;
     r.moveTopLeft(q->mapToGlobal(QPoint(0, 0)));
-#ifndef Q_WS_X11
     QDockWidgetLayout *layout = qobject_cast<QDockWidgetLayout*>(q->layout());
-    r.adjust(0, layout->titleHeight(), 0, 0);
-#endif
+    if (layout->nativeWindowDeco(true))
+        r.adjust(0, layout->titleHeight(), 0, 0);
     setWindowState(true, true, r);
 }
 
@@ -1019,15 +1020,12 @@ void QDockWidgetPrivate::setWindowState(bool floating, bool unplug, const QRect
     Qt::WindowFlags flags = floating ? Qt::Tool : Qt::Widget;
 
     QDockWidgetLayout *layout = qobject_cast<QDockWidgetLayout*>(q->layout());
-    bool nativeDeco = floating && layout->widgetForRole(QDockWidgetLayout::TitleBar) == 0;
-#if defined(Q_WS_X11) || defined(Q_WS_QWS) || defined(Q_OS_WINCE)
-    nativeDeco = false;
-#endif
+    const bool nativeDeco = layout->nativeWindowDeco(floating);
 
     if (nativeDeco) {
         flags |= Qt::CustomizeWindowHint | Qt::WindowTitleHint;
         if (hasFeature(q, QDockWidget::DockWidgetClosable))
-            flags |= Qt::WindowSystemMenuHint;
+            flags |= Qt::WindowCloseButtonHint;
     } else {
         flags |= Qt::FramelessWindowHint;
     }
@@ -1037,7 +1035,7 @@ void QDockWidgetPrivate::setWindowState(bool floating, bool unplug, const QRect
 
     q->setWindowFlags(flags);
 
-#ifdef Q_WS_MAC
+#if defined(Q_WS_MAC) && !defined(QT_MAC_USE_COCOA)
     if (floating && nativeDeco && (q->features() & QDockWidget::DockWidgetVerticalTitleBar)) {
         ChangeWindowAttributes(HIViewGetWindow(HIViewRef(q->winId())), kWindowSideTitlebarAttribute, 0);
     }
@@ -1053,7 +1051,7 @@ void QDockWidgetPrivate::setWindowState(bool floating, bool unplug, const QRect
 
     if (floating != wasFloating) {
         emit q->topLevelChanged(floating);
-        if (!floating) {
+        if (!floating && q->parentWidget()) {
             QMainWindowLayout *mwlayout = qobject_cast<QMainWindowLayout *>(q->parentWidget()->layout());
             if (mwlayout)
                 emit q->dockLocationChanged(mwlayout->dockWidgetArea(q));
@@ -1111,7 +1109,9 @@ void QDockWidgetPrivate::setWindowState(bool floating, bool unplug, const QRect
 /*!
     \enum QDockWidget::DockWidgetFeature
 
-    \value DockWidgetClosable   The dock widget can be closed.
+    \value DockWidgetClosable   The dock widget can be closed. On some systems the dock
+	                            widget always has a close button when it's floating
+								(for example on MacOS 10.5).
     \value DockWidgetMovable    The dock widget can be moved between docks
                                 by the user.
     \value DockWidgetFloatable  The dock widget can be detached from the
@@ -1193,7 +1193,7 @@ QWidget *QDockWidget::widget() const
     Sets the widget for the dock widget to \a widget.
 
     If the dock widget is visible when \a widget is added, you must
-    \l{QWidget::}{show()} it explicitly. 
+    \l{QWidget::}{show()} it explicitly.
 
     Note that you must add the layout of the \a widget before you call
     this function; if not, the \a widget will not be visible.
@@ -1336,6 +1336,9 @@ void QDockWidget::changeEvent(QEvent *event)
 /*! \reimp */
 void QDockWidget::closeEvent(QCloseEvent *event)
 {
+    Q_D(QDockWidget);
+    if (d->state)
+        d->endDrag(true);
     QWidget::closeEvent(event);
 }
 
@@ -1419,14 +1422,17 @@ bool QDockWidget::event(QEvent *event)
         // return true after calling the handler since we don't want
         // them to be passed onto the default handlers
     case QEvent::MouseButtonPress:
-        d->mousePressEvent(static_cast<QMouseEvent *>(event));
-        return true;
+        if (d->mousePressEvent(static_cast<QMouseEvent *>(event)))
+            return true;
+        break;
     case QEvent::MouseButtonDblClick:
-        d->mouseDoubleClickEvent(static_cast<QMouseEvent *>(event));
-        return true;
+        if (d->mouseDoubleClickEvent(static_cast<QMouseEvent *>(event)))
+            return true;
+        break;
     case QEvent::MouseMove:
-        d->mouseMoveEvent(static_cast<QMouseEvent *>(event));
-        return true;
+        if (d->mouseMoveEvent(static_cast<QMouseEvent *>(event)))
+            return true;
+        break;
 #ifdef Q_OS_WIN
     case QEvent::Leave:
         if (d->state != 0 && d->state->dragging && !d->state->nca) {
@@ -1439,8 +1445,9 @@ bool QDockWidget::event(QEvent *event)
         break;
 #endif
     case QEvent::MouseButtonRelease:
-        d->mouseReleaseEvent(static_cast<QMouseEvent *>(event));
-        return true;
+        if (d->mouseReleaseEvent(static_cast<QMouseEvent *>(event)))
+            return true;
+        break;
     case QEvent::NonClientAreaMouseMove:
     case QEvent::NonClientAreaMouseButtonPress:
     case QEvent::NonClientAreaMouseButtonRelease:
