@@ -1,37 +1,41 @@
 /****************************************************************************
 **
-** Copyright (C) 2008 Nokia Corporation and/or its subsidiary(-ies).
-** Contact: Qt Software Information (qt-info@nokia.com)
+** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial Usage
 ** Licensees holding valid Qt Commercial licenses may use this file in
 ** accordance with the Qt Commercial License Agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and Nokia.
 **
+** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+**
+** In addition, as a special exception, Nokia gives you certain
+** additional rights. These rights are described in the Nokia Qt LGPL
+** Exception version 1.0, included in the file LGPL_EXCEPTION.txt in this
+** package.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License versions 2.0 or 3.0 as published by the Free
-** Software Foundation and appearing in the file LICENSE.GPL included in
-** the packaging of this file.  Please review the following information
-** to ensure GNU General Public Licensing requirements will be met:
-** http://www.fsf.org/licensing/licenses/info/GPLv2.html and
-** http://www.gnu.org/copyleft/gpl.html.  In addition, as a special
-** exception, Nokia gives you certain additional rights. These rights
-** are described in the Nokia Qt GPL Exception version 1.3, included in
-** the file GPL_EXCEPTION.txt in this package.
-**
-** Qt for Windows(R) Licensees
-** As a special exception, Nokia, as the sole copyright holder for Qt
-** Designer, grants users of the Qt/Eclipse Integration plug-in the
-** right for the Qt/Eclipse Integration to link to functionality
-** provided by Qt Designer and its related libraries.
+** General Public License version 3.0 as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU General Public License version 3.0 requirements will be
+** met: http://www.gnu.org/copyleft/gpl.html.
 **
 ** If you are unsure which license is appropriate for your use, please
-** contact the sales department at qt-sales@nokia.com.
+** contact the sales department at http://www.qtsoftware.com/contact.
+** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
@@ -42,7 +46,7 @@
 #include <qtoolbutton.h>
 #include <qmenu.h>
 #include <qdebug.h>
-#include <math.h>
+#include <qmath.h>
 
 #include "qmainwindowlayout_p.h"
 #include "qtoolbarextension_p.h"
@@ -284,7 +288,7 @@ void QToolBarLayout::updateGeomArray() const
 
         that->expanding = expanding || exp & o;
 
-        
+
         if (item->widget()) {
             if ((item->widget()->sizePolicy().horizontalPolicy() & QSizePolicy::ExpandFlag)) {
                 that->expandFlag = true;
@@ -293,11 +297,12 @@ void QToolBarLayout::updateGeomArray() const
 
         if (!empty) {
             if (count == 0) // the minimum size only displays one widget
-                rpick(o, that->minSize) += spacing + pick(o, min);
+                rpick(o, that->minSize) += pick(o, min);
             int s = perp(o, minSize);
             rperp(o, that->minSize) = qMax(s, perp(o, min));
 
-            rpick(o, that->hint) += spacing + pick(o, hint);
+            //we only add spacing before item (ie never before the first one)
+            rpick(o, that->hint) += (count == 0 ? 0 : spacing) + pick(o, hint);
             s = perp(o, that->hint);
             rperp(o, that->hint) = qMax(s, perp(o, hint));
             ++count;
@@ -362,7 +367,6 @@ void QToolBarLayout::setGeometry(const QRect &rect)
     Qt::Orientation o = tb->orientation();
 
     QLayout::setGeometry(rect);
-
     if (movable()) {
         if (o == Qt::Horizontal) {
             handRect = QRect(margin, margin, handleExtent, rect.height() - 2*margin);
@@ -406,6 +410,18 @@ void QToolBarLayout::setGeometry(const QRect &rect)
         if (!extension->isHidden())
             extension->hide();
     }
+#ifdef Q_WS_MAC
+    // Nothing to do for Carbon... probably   
+#  ifdef QT_MAC_USE_COCOA
+    if (QMainWindow *win = qobject_cast<QMainWindow*>(tb->parentWidget())) {
+        Qt::ToolBarArea area = win->toolBarArea(tb);
+        if (win->unifiedTitleAndToolBarOnMac() && area == Qt::TopToolBarArea) {
+            static_cast<QMainWindowLayout *>(win->layout())->fixSizeInUnifiedToolbar(tb);
+        }
+    }
+#  endif
+#endif
+    
 }
 
 bool QToolBarLayout::layoutActions(const QSize &size)
@@ -456,7 +472,7 @@ bool QToolBarLayout::layoutActions(const QSize &size)
             if (a[i].empty)
                 continue;
 
-            int newSize = size + spacing + a[i].minimumSize;
+            int newSize = size + (count == 0 ? 0 : spacing) + a[i].minimumSize;
             if (prev != -1 && newSize > space) {
                 if (rows == 0)
                     ranOutOfSpace = true;
@@ -503,7 +519,7 @@ bool QToolBarLayout::layoutActions(const QSize &size)
             }
 
             QPoint pos;
-            rpick(o, pos) = margin + handleExtent + spacing + a[j].pos;
+            rpick(o, pos) = margin + handleExtent + a[j].pos;
             rperp(o, pos) = rowPos;
             QSize size;
             rpick(o, size) = a[j].size;
@@ -579,15 +595,15 @@ QSize QToolBarLayout::expandedSize(const QSize &size) const
     int count = 0;
     for (int x = 0; x < items.count(); ++x) {
         if (!geomArray[x].empty) {
+            total_w += (count == 0 ? 0 : spacing) + geomArray[x].minimumSize;
             ++count;
-            total_w += spacing + geomArray[x].minimumSize;
         }
     }
     if (count == 0)
         return QSize(0, 0);
 
     int min_w = pick(o, size);
-    int rows = (int)sqrt(count + 0.0);
+    int rows = (int)qSqrt(qreal(count));
     if (rows == 1)
         ++rows;      // we want to expand to at least two rows
     int space = total_w/rows + spacing + extensionExtent;
@@ -607,7 +623,7 @@ QSize QToolBarLayout::expandedSize(const QSize &size) const
             if (geomArray[i].empty)
                 continue;
 
-            int newSize = size + spacing + geomArray[i].minimumSize;
+            int newSize = size + (count == 0 ? 0 : spacing) + geomArray[i].minimumSize;
             rowHeight = qMax(rowHeight, perp(o, items.at(i)->sizeHint()));
             if (prev != -1 && newSize > space) {
                 if (count > 1 && size + spacing + extensionExtent > space) {
@@ -630,7 +646,7 @@ QSize QToolBarLayout::expandedSize(const QSize &size) const
     w = qMax(w, min_w);
     if (win != 0)
         w = qMin(w, pick(o, win->size()));
-    h += 2*margin;
+    h += 2*margin - spacing; //there is no spacing before the first row
 
     QSize result;
     rpick(o, result) = w;
@@ -690,8 +706,10 @@ QToolBarItem *QToolBarLayout::createItem(QAction *action)
 
     if (QWidgetAction *widgetAction = qobject_cast<QWidgetAction *>(action)) {
         widget = widgetAction->requestWidget(tb);
-        if (widget != 0)
+        if (widget != 0) {
+            widget->setAttribute(Qt::WA_LayoutUsesWidgetRect);
             customWidget = true;
+        }
     } else if (action->isSeparator()) {
         QToolBarSeparator *sep = new QToolBarSeparator(tb);
         connect(tb, SIGNAL(orientationChanged(Qt::Orientation)),
