@@ -1,43 +1,41 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
+** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtNetwork module of the Qt Toolkit.
 **
-** This file may be used under the terms of the GNU General Public
-** License versions 2.0 or 3.0 as published by the Free Software
-** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file.  Alternatively you may (at
-** your option) use any later version of the GNU General Public
-** License if such license has been publicly approved by Trolltech ASA
-** (or its successors, if any) and the KDE Free Qt Foundation. In
-** addition, as a special exception, Trolltech gives you certain
-** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.2, which can be found at
-** http://www.trolltech.com/products/qt/gplexception/ and in the file
-** GPL_EXCEPTION.txt in this package.
+** $QT_BEGIN_LICENSE:LGPL$
+** Commercial Usage
+** Licensees holding valid Qt Commercial licenses may use this file in
+** accordance with the Qt Commercial License Agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Nokia.
 **
-** Please review the following information to ensure GNU General
-** Public Licensing requirements will be met:
-** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
-** you are unsure which license is appropriate for your use, please
-** review the following information:
-** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
-** or contact the sales department at sales@trolltech.com.
+** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Trolltech, as the sole
-** copyright holder for Qt Designer, grants users of the Qt/Eclipse
-** Integration plug-in the right for the Qt/Eclipse Integration to
-** link to functionality provided by Qt Designer and its related
-** libraries.
+** In addition, as a special exception, Nokia gives you certain
+** additional rights. These rights are described in the Nokia Qt LGPL
+** Exception version 1.0, included in the file LGPL_EXCEPTION.txt in this
+** package.
 **
-** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
-** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
-** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
-** granted herein.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3.0 as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU General Public License version 3.0 requirements will be
+** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
-** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+** If you are unsure which license is appropriate for your use, please
+** contact the sales department at http://www.qtsoftware.com/contact.
+** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
@@ -51,7 +49,7 @@
 
     \reentrant
     \ingroup io
-    \module network
+    \inmodule QtNetwork
 
     QAbstractSocket is the base class for QTcpSocket and QUdpSocket
     and contains all common functionality of these two classes. If
@@ -96,9 +94,11 @@
     Read or write data by calling read() or write(), or use the
     convenience functions readLine() and readAll(). QAbstractSocket
     also inherits getChar(), putChar(), and ungetChar() from
-    QIODevice, which work on single bytes. For every chunk of data
-    that has been written to the socket, the bytesWritten() signal is
-    emitted.
+    QIODevice, which work on single bytes. The bytesWritten() signal
+    is emitted when data has been written to the socket (i.e., when
+    the client has read the data). Note that Qt does not limit the
+    write buffer size. You can monitor its size by listening to this
+    signal.
 
     The readyRead() signal is emitted every time a new chunk of data
     has arrived. bytesAvailable() then returns the number of bytes
@@ -110,14 +110,14 @@
     size of the read buffer, call setReadBufferSize().
 
     To close the socket, call disconnectFromHost(). QAbstractSocket enters
-    QAbstractSocket::ClosingState, then emits closing(). After all pending
-    data has been written to the socket, QAbstractSocket actually closes the
-    socket, enters QAbstractSocket::ClosedState, and emits disconnected(). If
-    you want to abort a connection immediately, discarding all pending data,
-    call abort() instead. If the remote host closes the connection,
-    QAbstractSocket will emit error(QAbstractSocket::RemoteHostClosedError),
-    during which the socket state will still be ConnectedState, and then the
-    disconnected() signal will be emitted.
+    QAbstractSocket::ClosingState. After all pending data has been written to
+    the socket, QAbstractSocket actually closes the socket, enters
+    QAbstractSocket::ClosedState, and emits disconnected(). If you want to
+    abort a connection immediately, discarding all pending data, call abort()
+    instead. If the remote host closes the connection, QAbstractSocket will
+    emit error(QAbstractSocket::RemoteHostClosedError), during which the socket
+    state will still be ConnectedState, and then the disconnected() signal
+    will be emitted.
 
     The port and address of the connected peer is fetched by calling
     peerPort() and peerAddress(). peerName() returns the host name of
@@ -199,9 +199,10 @@
     parameter describes the type of error that occurred.
 
     QAbstractSocket::SocketError is not a registered metatype, so for queued
-    connections, you will have to register it with Q_REGISTER_METATYPE.
+    connections, you will have to register it with Q_DECLARE_METATYPE() and
+    qRegisterMetaType().
 
-    \sa error(), errorString()
+    \sa error(), errorString(), {Creating Custom Qt Types}
 */
 
 /*!
@@ -211,9 +212,10 @@
     The \a socketState parameter is the new state.
 
     QAbstractSocket::SocketState is not a registered metatype, so for queued
-    connections, you will have to register it with Q_REGISTER_METATYPE.
+    connections, you will have to register it with Q_REGISTER_METATYPE() and
+    qRegisterMetaType().
 
-    \sa state()
+    \sa state(), {Creating Custom Qt Types}
 */
 
 /*!
@@ -289,11 +291,21 @@
            the proxy requires authentication.
     \value SslHandshakeFailedError The SSL/TLS handshake failed, so
            the connection was closed (only used in QSslSocket)
-    \value UnknownSocketError An unidentified error occurred.
     \value UnfinishedSocketOperationError Used by QAbstractSocketEngine only,
            The last operation attempted has not finished yet (still in progress in
             the background).
+    \value ProxyConnectionRefusedError Could not contact the proxy server because
+           the connection to that server was denied
+    \value ProxyConnectionClosedError The connection to the proxy server was closed
+           unexpectedly (before the connection to the final peer was established)
+    \value ProxyConnectionTimeoutError The connection to the proxy server timed out
+           or the proxy server stopped responding in the authentication phase.
+    \value ProxyNotFoundError The proxy address set with setProxy() (or the application
+           proxy) was not found.
+    \value ProxyProtocolError The connection negotiation with the proxy server
+           because the response from the proxy server could not be understood.
 
+    \value UnknownSocketError An unidentified error occurred.
     \sa QAbstractSocket::error()
 */
 
@@ -392,6 +404,21 @@ static QByteArray qt_prettyDebug(const char *data, int len, int maxLength)
 }
 #endif
 
+static bool isProxyError(QAbstractSocket::SocketError error)
+{
+    switch (error) {
+    case QAbstractSocket::ProxyAuthenticationRequiredError:
+    case QAbstractSocket::ProxyConnectionRefusedError:
+    case QAbstractSocket::ProxyConnectionClosedError:
+    case QAbstractSocket::ProxyConnectionTimeoutError:
+    case QAbstractSocket::ProxyNotFoundError:
+    case QAbstractSocket::ProxyProtocolError:
+        return true;
+    default:
+        return false;
+    }
+}
+
 /*! \internal
 
     Constructs a QAbstractSocketPrivate. Initializes all members.
@@ -402,6 +429,7 @@ QAbstractSocketPrivate::QAbstractSocketPrivate()
       readSocketNotifierStateSet(false),
       emittedReadyRead(false),
       emittedBytesWritten(false),
+      abortCalled(false),
       closeCalled(false),
       pendingClose(false),
       port(0),
@@ -419,9 +447,6 @@ QAbstractSocketPrivate::QAbstractSocketPrivate()
       hostLookupId(-1),
       state(QAbstractSocket::UnconnectedState),
       socketError(QAbstractSocket::UnknownSocketError)
-#ifndef QT_NO_NETWORKPROXY
-    , proxy(0)
-#endif
 {
 }
 
@@ -432,9 +457,6 @@ QAbstractSocketPrivate::QAbstractSocketPrivate()
 */
 QAbstractSocketPrivate::~QAbstractSocketPrivate()
 {
-#ifndef QT_NO_NETWORKPROXY
-    delete proxy;
-#endif
 }
 
 /*! \internal
@@ -466,23 +488,33 @@ void QAbstractSocketPrivate::resetSocketLayer()
     network layer protocol \a protocol. Resets the socket layer first
     if it's already initialized. Sets up the socket notifiers.
 */
-bool QAbstractSocketPrivate::initSocketLayer(const QHostAddress &host, QAbstractSocket::SocketType type)
+bool QAbstractSocketPrivate::initSocketLayer(QAbstractSocket::NetworkLayerProtocol protocol)
 {
+#ifdef QT_NO_NETWORKPROXY
+    // this is here to avoid a duplication of the call to createSocketEngine below
+    static const QNetworkProxy &proxyInUse = *(QNetworkProxy *)0;
+#endif
+
     Q_Q(QAbstractSocket);
 #if defined (QABSTRACTSOCKET_DEBUG)
     QString typeStr;
-    if (type == QAbstractSocket::TcpSocket) typeStr = "TcpSocket";
-    else if (type == QAbstractSocket::UdpSocket) typeStr = "UdpSocket";
+    if (q->socketType() == QAbstractSocket::TcpSocket) typeStr = "TcpSocket";
+    else if (q->socketType() == QAbstractSocket::UdpSocket) typeStr = "UdpSocket";
     else typeStr = "UnknownSocketType";
     QString protocolStr;
-    if (host.protocol() == QAbstractSocket::IPv4Protocol) protocolStr = "IPv4Protocol";
-    else if (host.protocol() == QAbstractSocket::IPv6Protocol) protocolStr = "IPv6Protocol";
+    if (protocol == QAbstractSocket::IPv4Protocol) protocolStr = "IPv4Protocol";
+    else if (protocol == QAbstractSocket::IPv6Protocol) protocolStr = "IPv6Protocol";
     else protocolStr = "UnknownNetworkLayerProtocol";
 #endif
 
     resetSocketLayer();
-    socketEngine = QAbstractSocketEngine::createSocketEngine(host, type, q);
-    if (!socketEngine->initialize(type, host.protocol())) {
+    socketEngine = QAbstractSocketEngine::createSocketEngine(q->socketType(), proxyInUse, q);
+    if (!socketEngine) {
+        socketError = QAbstractSocket::UnsupportedSocketOperationError;
+        q->setErrorString(QAbstractSocket::tr("Operation on socket is not supported"));
+        return false;
+    }
+    if (!socketEngine->initialize(q->socketType(), protocol)) {
 #if defined (QABSTRACTSOCKET_DEBUG)
         qDebug("QAbstractSocketPrivate::initSocketLayer(%s, %s) failed (%s)",
                typeStr.toLatin1().constData(), protocolStr.toLatin1().constData(),
@@ -493,8 +525,6 @@ bool QAbstractSocketPrivate::initSocketLayer(const QHostAddress &host, QAbstract
         return false;
     }
 
-    cachedSocketDescriptor = socketEngine->socketDescriptor();
-
     if (threadData->eventDispatcher)
         socketEngine->setReceiver(this);
 
@@ -611,16 +641,6 @@ bool QAbstractSocketPrivate::canWriteNotification()
         socketEngine->setWriteNotificationEnabled(false);
 #endif
 
-    // If in connecting state, check if the connection has been
-    // established, otherwise flush pending data.
-    if (state == QAbstractSocket::ConnectingState) {
-#if defined (QABSTRACTSOCKET_DEBUG)
-        qDebug("QAbstractSocketPrivate::canWriteNotification() testing connection");
-#endif
-        _q_testConnection();
-        return false;
-    }
-
 #if defined (QABSTRACTSOCKET_DEBUG)
     qDebug("QAbstractSocketPrivate::canWriteNotification() flushing");
 #endif
@@ -640,6 +660,23 @@ bool QAbstractSocketPrivate::canWriteNotification()
     return (writeBuffer.size() < tmp);
 }
 
+/*! \internal
+
+    Slot connected to a notification of connection status
+    change. Either we finished connecting or we failed to connect.
+*/
+void QAbstractSocketPrivate::connectionNotification()
+{
+    // If in connecting state, check if the connection has been
+    // established, otherwise flush pending data.
+    if (state == QAbstractSocket::ConnectingState) {
+#if defined (QABSTRACTSOCKET_DEBUG)
+        qDebug("QAbstractSocketPrivate::connectionNotification() testing connection");
+#endif
+        _q_testConnection();
+    }
+}
+
 /*! \internal
 
     Writes pending data in the write buffers to the socket. The
@@ -697,11 +734,103 @@ bool QAbstractSocketPrivate::flush()
     if (writeBuffer.isEmpty() && socketEngine && socketEngine->isWriteNotificationEnabled())
         socketEngine->setWriteNotificationEnabled(false);
     if (state == QAbstractSocket::ClosingState)
-        q->close();
+        q->disconnectFromHost();
 
     return true;
 }
 
+#ifndef QT_NO_NETWORKPROXY
+/*! \internal
+
+    Resolve the proxy to its final value.
+*/
+void QAbstractSocketPrivate::resolveProxy(const QString &hostname, quint16 port)
+{
+    QHostAddress parsed;
+    if (hostname == QLatin1String("localhost")
+        || hostname.startsWith(QLatin1String("localhost."))
+        || (parsed.setAddress(hostname)
+            && (parsed == QHostAddress::LocalHost
+                || parsed == QHostAddress::LocalHostIPv6))) {
+        proxyInUse = QNetworkProxy::NoProxy;
+        return;
+    }
+
+    QList<QNetworkProxy> proxies;
+
+    if (proxy.type() != QNetworkProxy::DefaultProxy) {
+        // a non-default proxy was set with setProxy
+        proxies << proxy;
+    } else {
+        // try the application settings instead
+        QNetworkProxyQuery query(hostname, port, QString(),
+                                 socketType == QAbstractSocket::TcpSocket ?
+                                 QNetworkProxyQuery::TcpSocket :
+                                 QNetworkProxyQuery::UdpSocket);
+        proxies = QNetworkProxyFactory::proxyForQuery(query);
+    }
+
+    // return the first that we can use
+    foreach (const QNetworkProxy &p, proxies) {
+        if (socketType == QAbstractSocket::UdpSocket &&
+            (p.capabilities() & QNetworkProxy::UdpTunnelingCapability) == 0)
+            continue;
+
+        if (socketType == QAbstractSocket::TcpSocket &&
+            (p.capabilities() & QNetworkProxy::TunnelingCapability) == 0)
+            continue;
+
+        proxyInUse = p;
+        return;
+    }
+
+    // no proxy found
+    // DefaultProxy here will raise an error
+    proxyInUse = QNetworkProxy();
+}
+
+/*!
+    \internal
+
+    Starts the connection to \a host, like _q_startConnecting below,
+    but without hostname resolution.
+*/
+void QAbstractSocketPrivate::startConnectingByName(const QString &host)
+{
+    Q_Q(QAbstractSocket);
+    if (state == QAbstractSocket::ConnectingState || state == QAbstractSocket::ConnectedState)
+        return;
+
+#if defined(QABSTRACTSOCKET_DEBUG)
+    qDebug("QAbstractSocketPrivate::startConnectingByName(host == %s)", qPrintable(host));
+#endif
+
+    // ### Let the socket engine drive this?
+    state = QAbstractSocket::ConnectingState;
+    emit q->stateChanged(state);
+
+    connectTimeElapsed = 0;
+
+    if (initSocketLayer(QAbstractSocket::UnknownNetworkLayerProtocol)) {
+        if (socketEngine->connectToHostByName(host, port) ||
+            socketEngine->state() == QAbstractSocket::ConnectingState) {
+            cachedSocketDescriptor = socketEngine->socketDescriptor();
+
+            return;
+        }
+
+        // failed to connect
+        socketError = socketEngine->error();
+        q->setErrorString(socketEngine->errorString());
+    }
+
+    state = QAbstractSocket::UnconnectedState;
+    emit q->error(socketError);
+    emit q->stateChanged(state);
+}
+
+#endif
+
 /*! \internal
 
     Slot connected to QHostInfo::lookupHost() in connectToHost(). This
@@ -712,7 +841,7 @@ bool QAbstractSocketPrivate::flush()
 void QAbstractSocketPrivate::_q_startConnecting(const QHostInfo &hostInfo)
 {
     Q_Q(QAbstractSocket);
-    if (state == QAbstractSocket::ConnectingState || state == QAbstractSocket::ConnectedState)
+    if (state != QAbstractSocket::HostLookupState)
         return;
 
     addresses = hostInfo.addresses();
@@ -738,7 +867,7 @@ void QAbstractSocketPrivate::_q_startConnecting(const QHostInfo &hostInfo)
 #endif
         state = QAbstractSocket::UnconnectedState;
         socketError = QAbstractSocket::HostNotFoundError;
-        q->setErrorString(QLatin1String(QT_TRANSLATE_NOOP("QAbstractSocket", "Host not found")));
+        q->setErrorString(QAbstractSocket::tr("Host not found"));
         emit q->stateChanged(state);
         emit q->error(QAbstractSocket::HostNotFoundError);
         return;
@@ -788,16 +917,14 @@ void QAbstractSocketPrivate::_q_connectToNextAddress()
 #endif
                     ) && socketEngine->state() == QAbstractSocket::ConnectingState) {
                     socketError = QAbstractSocket::ConnectionRefusedError;
-                    q->setErrorString(QLatin1String(QT_TRANSLATE_NOOP("QAbstractSocket",
-                                                                      "Connection refused")));
+                    q->setErrorString(QAbstractSocket::tr("Connection refused"));
                 } else {
                     socketError = socketEngine->error();
                     q->setErrorString(socketEngine->errorString());
                 }
             } else {
-                socketError = QAbstractSocket::ConnectionRefusedError;
-                q->setErrorString(QLatin1String(QT_TRANSLATE_NOOP("QAbstractSocket",
-                                                                  "Connection refused")));
+//                socketError = QAbstractSocket::ConnectionRefusedError;
+//                q->setErrorString(QAbstractSocket::tr("Connection refused"));
             }
             emit q->stateChanged(state);
             emit q->error(socketError);
@@ -807,8 +934,8 @@ void QAbstractSocketPrivate::_q_connectToNextAddress()
         // Pick the first host address candidate
         host = addresses.takeFirst();
 #if defined(QABSTRACTSOCKET_DEBUG)
-        qDebug("QAbstractSocketPrivate::_q_connectToNextAddress(), connecting to %s:%i",
-               host.toString().toLatin1().constData(), port);
+        qDebug("QAbstractSocketPrivate::_q_connectToNextAddress(), connecting to %s:%i, %d left to try",
+               host.toString().toLatin1().constData(), port, addresses.count());
 #endif
 
 #if defined(QT_NO_IPV6)
@@ -822,7 +949,7 @@ void QAbstractSocketPrivate::_q_connectToNextAddress()
         }
 #endif
 
-        if (!initSocketLayer(host, q->socketType())) {
+        if (!initSocketLayer(host.protocol())) {
             // hope that the next address is better
 #if defined(QABSTRACTSOCKET_DEBUG)
             qDebug("QAbstractSocketPrivate::_q_connectToNextAddress(), failed to initialize sock layer");
@@ -834,10 +961,14 @@ void QAbstractSocketPrivate::_q_connectToNextAddress()
         // (localhost address on BSD or any UDP connect), emit
         // connected() and return.
         if (socketEngine->connectToHost(host, port)) {
+            //_q_testConnection();
             fetchConnectionParameters();
             return;
         }
 
+        // cache the socket descriptor even if we're not fully connected yet
+        cachedSocketDescriptor = socketEngine->socketDescriptor();
+
         // Check that we're in delayed connection state. If not, try
         // the next address
         if (socketEngine->state() != QAbstractSocket::ConnectingState) {
@@ -874,17 +1005,6 @@ void QAbstractSocketPrivate::_q_connectToNextAddress()
 void QAbstractSocketPrivate::_q_testConnection()
 {
     if (socketEngine) {
-        if (socketEngine->state() != QAbstractSocket::ConnectedState) {
-            // Try connecting if we're not already connected.
-            if (!socketEngine->connectToHost(host, port)) {
-                if (socketEngine->error() == QAbstractSocket::UnfinishedSocketOperationError) {
-                    // Connection in progress; wait for the next notification.
-                    socketEngine->setWriteNotificationEnabled(true);
-                    return;
-                }
-            }
-        }
-
         if (threadData->eventDispatcher) {
             if (connectTimer)
                 connectTimer->stop();
@@ -901,8 +1021,8 @@ void QAbstractSocketPrivate::_q_testConnection()
             return;
         }
 
-        // don't retry the other addresses if we couldn't authenticate to the proxy.
-        if (socketEngine->error() == QAbstractSocket::ProxyAuthenticationRequiredError)
+        // don't retry the other addresses if we had a proxy error
+        if (isProxyError(socketEngine->error()))
             addresses.clear();
     }
 
@@ -927,14 +1047,22 @@ void QAbstractSocketPrivate::_q_testConnection()
 */
 void QAbstractSocketPrivate::_q_abortConnectionAttempt()
 {
+    Q_Q(QAbstractSocket);
 #if defined(QABSTRACTSOCKET_DEBUG)
     qDebug("QAbstractSocketPrivate::_q_abortConnectionAttempt() (timed out)");
 #endif
-    if (socketEngine) {
+    if (socketEngine)
         socketEngine->setWriteNotificationEnabled(false);
+    connectTimer->stop();
 
-        if (socketEngine->isValid())
-            _q_testConnection();
+    if (addresses.isEmpty()) {
+        state = QAbstractSocket::UnconnectedState;
+        socketError = QAbstractSocket::SocketTimeoutError;
+        q->setErrorString(QAbstractSocket::tr("Connection timed out"));
+        emit q->stateChanged(state);
+        emit q->error(socketError);
+    } else {
+        _q_connectToNextAddress();
     }
 }
 
@@ -1001,7 +1129,7 @@ bool QAbstractSocketPrivate::readFromSocket()
 
 /*! \internal
 
-    Sets up the the internal state after the connection has succeeded.
+    Sets up the internal state after the connection has succeeded.
 */
 void QAbstractSocketPrivate::fetchConnectionParameters()
 {
@@ -1105,7 +1233,7 @@ bool QAbstractSocket::isValid() const
 
     \a hostName may be an IP address in string form (e.g.,
     "43.195.83.32"), or it may be a host name (e.g.,
-    "www.trolltech.com"). QAbstractSocket will do a lookup only if
+    "qtsoftware.com"). QAbstractSocket will do a lookup only if
     required. \a port is in native byte order.
 
     \sa state(), peerName(), peerAddress(), peerPort(), waitForConnected()
@@ -1137,16 +1265,17 @@ void QAbstractSocket::connectToHostImplementation(const QString &hostName, quint
            (int) openMode);
 #endif
 
-    if (d->state == ConnectedState || d->state == ConnectingState) {
+    if (d->state == ConnectedState || d->state == ConnectingState || d->state == ClosingState) {
         qWarning("QAbstractSocket::connectToHost() called when already connecting/connected to \"%s\"", qPrintable(hostName));
         return;
     }
 
     d->hostName = hostName;
     d->port = port;
-    d->state = HostLookupState;
+    d->state = UnconnectedState;
     d->readBuffer.clear();
     d->writeBuffer.clear();
+    d->abortCalled = false;
     d->closeCalled = false;
     d->pendingClose = false;
     d->localPort = 0;
@@ -1163,9 +1292,22 @@ void QAbstractSocket::connectToHostImplementation(const QString &hostName, quint
         d->hostLookupId = -1;
     }
 
+#ifndef QT_NO_NETWORKPROXY
+    // Get the proxy information
+    d->resolveProxy(hostName, port);
+    if (d->proxyInUse.type() == QNetworkProxy::DefaultProxy) {
+        // failed to setup the proxy
+        d->socketError = QAbstractSocket::UnsupportedSocketOperationError;
+        setErrorString(QAbstractSocket::tr("Operation on socket is not supported"));
+        emit error(d->socketError);
+        return;
+    }
+#endif
+
     if (!d_func()->isBuffered)
         openMode |= QAbstractSocket::Unbuffered;
     QIODevice::open(openMode);
+    d->state = HostLookupState;
     emit stateChanged(d->state);
 
     QHostAddress temp;
@@ -1173,6 +1315,12 @@ void QAbstractSocket::connectToHostImplementation(const QString &hostName, quint
         QHostInfo info;
         info.setAddresses(QList<QHostAddress>() << temp);
         d->_q_startConnecting(info);
+#ifndef QT_NO_NETWORKPROXY
+    } else if (d->proxyInUse.capabilities() & QNetworkProxy::HostNameLookupCapability) {
+        // the proxy supports connection by name, so use it
+        d->startConnectingByName(hostName);
+        return;
+#endif
     } else {
         if (d->threadData->eventDispatcher)
             d->hostLookupId = QHostInfo::lookupHost(hostName, this, SLOT(_q_startConnecting(QHostInfo)));
@@ -1356,6 +1504,11 @@ bool QAbstractSocket::setSocketDescriptor(int socketDescriptor, SocketState sock
 
     d->resetSocketLayer();
     d->socketEngine = QAbstractSocketEngine::createSocketEngine(socketDescriptor, this);
+    if (!d->socketEngine) {
+        d->socketError = UnsupportedSocketOperationError;
+        setErrorString(tr("Operation on socket is not supported"));
+        return false;
+    }
     bool result = d->socketEngine->initialize(socketDescriptor, socketState);
     if (!result) {
         d->socketError = d->socketEngine->error();
@@ -1460,11 +1613,6 @@ bool QAbstractSocket::waitForConnected(int msecs)
         QHostInfo::abortHostLookup(d->hostLookupId);
         d->hostLookupId = -1;
         d->_q_startConnecting(QHostInfo::fromName(d->hostName));
-    } else {
-#if defined (QABSTRACTSOCKET_DEBUG)
-        qDebug("QAbstractSocket::waitForConnected(%i) testing connection", msecs);
-#endif
-        d->_q_testConnection();
     }
     if (state() == UnconnectedState)
         return false;
@@ -1483,7 +1631,7 @@ bool QAbstractSocket::waitForConnected(int msecs)
 #endif
         timedOut = false;
 
-        if (d->socketEngine->waitForWrite(timeout, &timedOut) && !timedOut) {
+        if (d->socketEngine && d->socketEngine->waitForWrite(timeout, &timedOut) && !timedOut) {
             d->_q_testConnection();
         } else {
             d->_q_connectToNextAddress();
@@ -1492,7 +1640,8 @@ bool QAbstractSocket::waitForConnected(int msecs)
 
     if ((timedOut && state() != ConnectedState) || state() == ConnectingState) {
         d->socketError = SocketTimeoutError;
-        setSocketState(UnconnectedState);
+        d->state = UnconnectedState;
+        emit stateChanged(d->state);
         d->resetSocketLayer();
         setErrorString(tr("Socket operation timed out"));
     }
@@ -1717,9 +1866,9 @@ bool QAbstractSocket::waitForDisconnected(int msecs)
 }
 
 /*!
-    Aborts the current connection and resets the socket. Unlike
-    disconnectFromHost(), this function immediately closes the socket, discarding
-    any pending data in the write buffer.
+    Aborts the current connection and resets the socket. Unlike disconnectFromHost(),
+    this function immediately closes the socket, discarding any pending data in the
+    write buffer.
 
     \sa disconnectFromHost(), close()
 */
@@ -1744,6 +1893,7 @@ void QAbstractSocket::abort()
     }
 
     d->writeBuffer.clear();
+    d->abortCalled = true;
     close();
 }
 
@@ -2015,7 +2165,12 @@ void QAbstractSocket::setPeerName(const QString &name)
 }
 
 /*!
-    Disconnects the socket's connection with the host.
+    Closes the I/O device for the socket, disconnects the socket's connection with the
+    host, closes the socket, and resets the name, address, port number and underlying
+    socket descriptor.
+
+    See QIODevice::close() for a description of the actions that occur when an I/O
+    device is closed.
 
     \sa abort()
 */
@@ -2072,7 +2227,7 @@ void QAbstractSocket::disconnectFromHostImplementation()
         return;
     }
 
-    if (d->state == ConnectingState) {
+    if (!d->abortCalled && (d->state == ConnectingState || d->state == HostLookupState)) {
 #if defined(QABSTRACTSOCKET_DEBUG)
         qDebug("QAbstractSocket::disconnectFromHost() but we're still connecting");
 #endif
@@ -2088,33 +2243,40 @@ void QAbstractSocket::disconnectFromHostImplementation()
     if (d->socketEngine)
         d->socketEngine->setReadNotificationEnabled(false);
 
-    // Perhaps emit closing()
-    if (d->state != ClosingState) {
-        d->state = ClosingState;
+    if (d->abortCalled) {
 #if defined(QABSTRACTSOCKET_DEBUG)
-        qDebug("QAbstractSocket::disconnectFromHost() emits stateChanged()(ClosingState)");
+        qDebug("QAbstractSocket::disconnectFromHost() aborting immediately");
 #endif
-        emit stateChanged(d->state);
     } else {
+        // Perhaps emit closing()
+        if (d->state != ClosingState) {
+            d->state = ClosingState;
 #if defined(QABSTRACTSOCKET_DEBUG)
-        qDebug("QAbstractSocket::disconnectFromHost() return from delayed close");
-#endif
-    }
-
-    // Wait for pending data to be written.
-    if (d->socketEngine && d->socketEngine->isValid() && d->writeBuffer.size() > 0) {
-        d->socketEngine->setWriteNotificationEnabled(true);
-
-#if defined(QABSTRACTSOCKET_DEBUG)
-        qDebug("QAbstractSocket::disconnectFromHost() delaying disconnect");
-#endif
-        return;
-    } else {
-#if defined(QABSTRACTSOCKET_DEBUG)
-        qDebug("QAbstractSocket::disconnectFromHost() disconnecting immediately");
+            qDebug("QAbstractSocket::disconnectFromHost() emits stateChanged()(ClosingState)");
 #endif
+            emit stateChanged(d->state);
+        } else {
+#if defined(QABSTRACTSOCKET_DEBUG)
+            qDebug("QAbstractSocket::disconnectFromHost() return from delayed close");
+#endif
+        }
+
+        // Wait for pending data to be written.
+        if (d->socketEngine && d->socketEngine->isValid() && d->writeBuffer.size() > 0) {
+            d->socketEngine->setWriteNotificationEnabled(true);
+
+#if defined(QABSTRACTSOCKET_DEBUG)
+            qDebug("QAbstractSocket::disconnectFromHost() delaying disconnect");
+#endif
+            return;
+        } else {
+#if defined(QABSTRACTSOCKET_DEBUG)
+            qDebug("QAbstractSocket::disconnectFromHost() disconnecting immediately");
+#endif
+        }
     }
 
+    SocketState previousState = d->state;
     d->resetSocketLayer();
     d->state = UnconnectedState;
     emit stateChanged(d->state);
@@ -2123,7 +2285,9 @@ void QAbstractSocket::disconnectFromHostImplementation()
 #ifdef QT3_SUPPORT
     emit delayedCloseFinished(); // compat signal
 #endif
-    emit disconnected();
+    // only emit disconnected if we were connected before
+    if (previousState == ConnectedState || previousState == ClosingState)
+        emit disconnected();
 
     d->localPort = 0;
     d->peerPort = 0;
@@ -2267,36 +2431,40 @@ void QAbstractSocket::setSocketError(SocketError socketError)
 
     \snippet doc/src/snippets/code/src_network_socket_qabstractsocket.cpp 3
 
+    The default value for the proxy is QNetworkProxy::DefaultProxy,
+    which means the socket will use the application settings: if a
+    proxy is set with QNetworkProxy::setApplicationProxy, it will use
+    that; otherwise, if a factory is set with
+    QNetworkProxyFactory::setApplicationProxyFactory, it will query
+    that factory with type QNetworkProxyQuery::TcpSocket.
 
-    \sa proxy(), QNetworkProxy
+    \sa proxy(), QNetworkProxy, QNetworkProxyFactory::queryProxy()
 */
 void QAbstractSocket::setProxy(const QNetworkProxy &networkProxy)
 {
     Q_D(QAbstractSocket);
-    if (!d->proxy)
-        d->proxy = new QNetworkProxy();
-    *d->proxy = networkProxy;
+    d->proxy = networkProxy;
 }
 
 /*!
     \since 4.1
 
     Returns the network proxy for this socket.
-    By default QNetworkProxy::DefaultProxy is used.
+    By default QNetworkProxy::DefaultProxy is used, which means this
+    socket will query the default proxy settings for the application.
 
-    \sa setProxy(), QNetworkProxy
+    \sa setProxy(), QNetworkProxy, QNetworkProxyFactory
 */
 QNetworkProxy QAbstractSocket::proxy() const
 {
     Q_D(const QAbstractSocket);
-    if (d->proxy)
-        return *d->proxy;
-    return QNetworkProxy();
+    return d->proxy;
 }
 #endif // QT_NO_NETWORKPROXY
 
 #ifdef QT3_SUPPORT
-/*! \enum QAbstractSocket::Error
+/*! 
+    \enum QAbstractSocket::Error
     \compat
 
     Use QAbstractSocket::SocketError instead.
@@ -2412,6 +2580,21 @@ Q_NETWORK_EXPORT QDebug operator<<(QDebug debug, QAbstractSocket::SocketError er
     case QAbstractSocket::UnknownSocketError:
         debug << "QAbstractSocket::UnknownSocketError";
         break;
+    case QAbstractSocket::ProxyConnectionRefusedError:
+        debug << "QAbstractSocket::ProxyConnectionRefusedError";
+        break;
+    case QAbstractSocket::ProxyConnectionClosedError:
+        debug << "QAbstractSocket::ProxyConnectionClosedError";
+        break;
+    case QAbstractSocket::ProxyConnectionTimeoutError:
+        debug << "QAbstractSocket::ProxyConnectionTimeoutError";
+        break;
+    case QAbstractSocket::ProxyNotFoundError:
+        debug << "QAbstractSocket::ProxyNotFoundError";
+        break;
+    case QAbstractSocket::ProxyProtocolError:
+        debug << "QAbstractSocket::ProxyProtocolError";
+        break;
     default:
         debug << "QAbstractSocket::SocketError(" << int(error) << ")";
         break;
