@@ -1,43 +1,41 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
+** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the Qt3Support module of the Qt Toolkit.
 **
-** This file may be used under the terms of the GNU General Public
-** License versions 2.0 or 3.0 as published by the Free Software
-** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file.  Alternatively you may (at
-** your option) use any later version of the GNU General Public
-** License if such license has been publicly approved by Trolltech ASA
-** (or its successors, if any) and the KDE Free Qt Foundation. In
-** addition, as a special exception, Trolltech gives you certain
-** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.2, which can be found at
-** http://www.trolltech.com/products/qt/gplexception/ and in the file
-** GPL_EXCEPTION.txt in this package.
+** $QT_BEGIN_LICENSE:LGPL$
+** Commercial Usage
+** Licensees holding valid Qt Commercial licenses may use this file in
+** accordance with the Qt Commercial License Agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Nokia.
 **
-** Please review the following information to ensure GNU General
-** Public Licensing requirements will be met:
-** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
-** you are unsure which license is appropriate for your use, please
-** review the following information:
-** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
-** or contact the sales department at sales@trolltech.com.
+** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Trolltech, as the sole
-** copyright holder for Qt Designer, grants users of the Qt/Eclipse
-** Integration plug-in the right for the Qt/Eclipse Integration to
-** link to functionality provided by Qt Designer and its related
-** libraries.
+** In addition, as a special exception, Nokia gives you certain
+** additional rights. These rights are described in the Nokia Qt LGPL
+** Exception version 1.0, included in the file LGPL_EXCEPTION.txt in this
+** package.
 **
-** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
-** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
-** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
-** granted herein.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3.0 as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU General Public License version 3.0 requirements will be
+** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
-** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+** If you are unsure which license is appropriate for your use, please
+** contact the sales department at http://www.qtsoftware.com/contact.
+** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
@@ -67,8 +65,8 @@
 
 #include <ctype.h>
 #if defined(Q_OS_WINCE)
-#include "qfunctions_wince.h"
 #include <winsock.h>
+#include "qfunctions_wince.h"
 #endif
 
 QT_BEGIN_NAMESPACE
@@ -424,7 +422,7 @@ QWidget * Q3DragObject::source()
 
 
 /*!
-    \class Q3DragObject qdragobject.h
+    \class Q3DragObject
 
     \brief The Q3DragObject class encapsulates MIME-based data
     transfer.
@@ -459,7 +457,7 @@ void stripws(QByteArray& s)
 }
 
 /*!
-    \class Q3TextDrag qdragobject.h
+    \class Q3TextDrag
 
     \brief The Q3TextDrag class is a drag and drop object for
     transferring plain and Unicode text.
@@ -749,7 +747,7 @@ bool Q3TextDrag::decode(const QMimeSource* e, QString& str)
 */
 
 /*!
-    \class Q3ImageDrag qdragobject.h
+    \class Q3ImageDrag
 
     \brief The Q3ImageDrag class provides a drag and drop object for
     transferring images.
@@ -936,7 +934,7 @@ bool Q3ImageDrag::decode(const QMimeSource* e, QPixmap& pm)
 
 
 /*!
-    \class Q3StoredDrag qdragobject.h
+    \class Q3StoredDrag
     \brief The Q3StoredDrag class provides a simple stored-value drag object for arbitrary MIME data.
 
     \compat
@@ -1023,7 +1021,7 @@ QByteArray Q3StoredDrag::encodedData(const char* m) const
 
 
 /*!
-    \class Q3UriDrag qdragobject.h
+    \class Q3UriDrag
     \brief The Q3UriDrag class provides a drag object for a list of URI references.
 
     \compat
