@@ -1,43 +1,41 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
+** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtSql module of the Qt Toolkit.
 **
-** This file may be used under the terms of the GNU General Public
-** License versions 2.0 or 3.0 as published by the Free Software
-** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file.  Alternatively you may (at
-** your option) use any later version of the GNU General Public
-** License if such license has been publicly approved by Trolltech ASA
-** (or its successors, if any) and the KDE Free Qt Foundation. In
-** addition, as a special exception, Trolltech gives you certain
-** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.2, which can be found at
-** http://www.trolltech.com/products/qt/gplexception/ and in the file
-** GPL_EXCEPTION.txt in this package.
+** $QT_BEGIN_LICENSE:LGPL$
+** Commercial Usage
+** Licensees holding valid Qt Commercial licenses may use this file in
+** accordance with the Qt Commercial License Agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Nokia.
 **
-** Please review the following information to ensure GNU General
-** Public Licensing requirements will be met:
-** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
-** you are unsure which license is appropriate for your use, please
-** review the following information:
-** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
-** or contact the sales department at sales@trolltech.com.
+** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Trolltech, as the sole
-** copyright holder for Qt Designer, grants users of the Qt/Eclipse
-** Integration plug-in the right for the Qt/Eclipse Integration to
-** link to functionality provided by Qt Designer and its related
-** libraries.
+** In addition, as a special exception, Nokia gives you certain
+** additional rights. These rights are described in the Nokia Qt LGPL
+** Exception version 1.0, included in the file LGPL_EXCEPTION.txt in this
+** package.
 **
-** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
-** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
-** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
-** granted herein.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3.0 as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU General Public License version 3.0 requirements will be
+** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
-** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+** If you are unsure which license is appropriate for your use, please
+** contact the sales department at http://www.qtsoftware.com/contact.
+** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
@@ -81,7 +79,13 @@ QT_BEGIN_NAMESPACE
 class QMYSQLDriverPrivate
 {
 public:
-    QMYSQLDriverPrivate() : mysql(0), tc(0), preparedQuerys(false), preparedQuerysEnabled(false) {}
+    QMYSQLDriverPrivate() : mysql(0),
+#ifndef QT_NO_TEXTCODEC
+        tc(QTextCodec::codecForLocale()),
+#else
+        tc(0),
+#endif
+        preparedQuerys(false), preparedQuerysEnabled(false) {}
     MYSQL *mysql;
     QTextCodec *tc;
 
@@ -159,25 +163,23 @@ static inline QVariant qDateTimeFromString(QString &val)
 #endif
 }
 
-class QMYSQLResultPrivate : public QMYSQLDriverPrivate
+class QMYSQLResultPrivate : public QObject
 {
+    Q_OBJECT
 public:
-    QMYSQLResultPrivate() : QMYSQLDriverPrivate(), result(0),
-#ifndef QT_NO_TEXTCODEC
-        tc(QTextCodec::codecForLocale()),
-#else
-        tc(0),
-#endif
+    QMYSQLResultPrivate(const QMYSQLDriver* dp) : driver(dp), result(0),
         rowsAffected(0), hasBlobs(false)
 #if MYSQL_VERSION_ID >= 40108
         , stmt(0), meta(0), inBinds(0), outBinds(0)
 #endif
         , precisionPolicy(QSql::HighPrecision)
-        {}
+        {
+            connect(dp, SIGNAL(destroyed()), this, SLOT(driverDestroyed()));
+        }
 
+    const QMYSQLDriver* driver;
     MYSQL_RES *result;
     MYSQL_ROW row;
-    QTextCodec *tc;
 
     int rowsAffected;
 
@@ -208,6 +210,8 @@ public:
     MYSQL_BIND *outBinds;
 #endif
     QSql::NumericalPrecisionPolicy precisionPolicy;
+private Q_SLOTS:
+    void driverDestroyed() { driver = NULL; }
 };
 
 #ifndef QT_NO_TEXTCODEC
@@ -225,7 +229,7 @@ static QTextCodec* codec(MYSQL* mysql)
 static QSqlError qMakeError(const QString& err, QSqlError::ErrorType type,
                             const QMYSQLDriverPrivate* p)
 {
-    const char *cerr = mysql_error(p->mysql);
+    const char *cerr = p->mysql ? mysql_error(p->mysql) : 0;
     return QSqlError(QLatin1String("QMYSQL: ") + err,
                      p->tc ? toUnicode(p->tc, cerr) : QString::fromLatin1(cerr),
                      type, mysql_errno(p->mysql));
@@ -380,10 +384,7 @@ bool QMYSQLResultPrivate::bindInValues()
 QMYSQLResult::QMYSQLResult(const QMYSQLDriver* db)
 : QSqlResult(db)
 {
-    d = new QMYSQLResultPrivate();
-    d->mysql = db->d->mysql;
-    d->tc = db->d->tc;
-    d->preparedQuerysEnabled = db->d->preparedQuerysEnabled;
+    d = new QMYSQLResultPrivate(db);
 }
 
 QMYSQLResult::~QMYSQLResult()
@@ -395,10 +396,11 @@ QMYSQLResult::~QMYSQLResult()
 QVariant QMYSQLResult::handle() const
 {
 #if MYSQL_VERSION_ID >= 40108
-    return d->meta ? qVariantFromValue(d->meta) : qVariantFromValue(d->stmt);
-#else
-    return qVariantFromValue(d->result);
+    if(d->driver && d->driver->d->preparedQuerys)
+        return d->meta ? qVariantFromValue(d->meta) : qVariantFromValue(d->stmt);
+    else
 #endif
+        return qVariantFromValue(d->result);
 }
 
 void QMYSQLResult::cleanup()
@@ -409,8 +411,8 @@ void QMYSQLResult::cleanup()
 // must iterate trough leftover result sets from multi-selects or stored procedures
 // if this isn't done subsequent queries will fail with "Commands out of sync"
 #if MYSQL_VERSION_ID >= 40100
-    while (mysql_next_result(d->mysql) == 0) {
-        MYSQL_RES *res = mysql_store_result(d->mysql);
+    while (d->driver && d->driver->d->mysql && mysql_next_result(d->driver->d->mysql) == 0) {
+        MYSQL_RES *res = mysql_store_result(d->driver->d->mysql);
         if (res)
             mysql_free_result(res);
     }
@@ -450,15 +452,18 @@ void QMYSQLResult::cleanup()
     setAt(-1);
     setActive(false);
 
-    d->preparedQuerys = d->preparedQuerysEnabled;
+    if(d->driver)
+        d->driver->d->preparedQuerys = d->driver->d->preparedQuerysEnabled;
 }
 
 bool QMYSQLResult::fetch(int i)
 {
+    if(!d->driver)
+        return false;
     if (isForwardOnly()) { // fake a forward seek
         if (at() < i) {
             int x = i - at();
-            while (--x && fetchNext());
+            while (--x && fetchNext()) {};
             return fetchNext();
         } else {
             return false;
@@ -466,7 +471,7 @@ bool QMYSQLResult::fetch(int i)
     }
     if (at() == i)
         return true;
-    if (d->preparedQuerys) {
+    if (d->driver->d->preparedQuerys) {
 #if MYSQL_VERSION_ID >= 40108
         mysql_stmt_data_seek(d->stmt, i);
 
@@ -497,7 +502,9 @@ bool QMYSQLResult::fetch(int i)
 
 bool QMYSQLResult::fetchNext()
 {
-    if (d->preparedQuerys) {
+    if(!d->driver)
+        return false;
+    if (d->driver->d->preparedQuerys) {
 #if MYSQL_VERSION_ID >= 40108
         if (mysql_stmt_fetch(d->stmt))
             return false;
@@ -515,14 +522,16 @@ bool QMYSQLResult::fetchNext()
 
 bool QMYSQLResult::fetchLast()
 {
+    if(!d->driver)
+        return false;
     if (isForwardOnly()) { // fake this since MySQL can't seek on forward only queries
         bool success = fetchNext(); // did we move at all?
-        while (fetchNext());
+        while (fetchNext()) {};
         return success;
     }
 
     my_ulonglong numRows;
-    if (d->preparedQuerys) {
+    if (d->driver->d->preparedQuerys) {
 #if MYSQL_VERSION_ID >= 40108
         numRows = mysql_stmt_num_rows(d->stmt);
 #else
@@ -556,15 +565,18 @@ QVariant QMYSQLResult::data(int field)
         return QVariant();
     }
 
+    if (!d->driver)
+        return QVariant();
+
     int fieldLength = 0;
     const QMYSQLResultPrivate::QMyField &f = d->fields.at(field);
     QString val;
-    if (d->preparedQuerys) {
+    if (d->driver->d->preparedQuerys) {
         if (f.nullIndicator)
             return QVariant(f.type);
 
         if (f.type != QVariant::ByteArray)
-            val = toUnicode(d->tc, f.outField, f.bufLength);
+            val = toUnicode(d->driver->d->tc, f.outField, f.bufLength);
     } else {
         if (d->row[field] == NULL) {
             // NULL value
@@ -572,7 +584,7 @@ QVariant QMYSQLResult::data(int field)
         }
         fieldLength = mysql_fetch_lengths(d->result)[field];
         if (f.type != QVariant::ByteArray)
-            val = toUnicode(d->tc, d->row[field], fieldLength);
+            val = toUnicode(d->driver->d->tc, d->row[field], fieldLength);
     }
 
     switch(f.type) {
@@ -590,14 +602,18 @@ QVariant QMYSQLResult::data(int field)
         switch(d->precisionPolicy) {
             case QSql::LowPrecisionInt32:
                 v=val.toInt(&ok);
+                break;
             case QSql::LowPrecisionInt64:
                 v = val.toLongLong(&ok);
+                break;
             case QSql::LowPrecisionDouble:
                 v = val.toDouble(&ok);
+                break;
             case QSql::HighPrecision:
             default:
                 v = val;
                 ok = true;
+                break;
         }
         if(ok)
             return v;
@@ -613,7 +629,7 @@ QVariant QMYSQLResult::data(int field)
     case QVariant::ByteArray: {
 
         QByteArray ba;
-        if (d->preparedQuerys) {
+        if (d->driver->d->preparedQuerys) {
             ba = QByteArray(f.outField, f.bufLength);
         } else {
             ba = QByteArray(d->row[field], fieldLength);
@@ -630,7 +646,7 @@ QVariant QMYSQLResult::data(int field)
 
 bool QMYSQLResult::isNull(int field)
 {
-   if (d->preparedQuerys)
+   if (d->driver->d->preparedQuerys)
        return d->fields.at(field).nullIndicator;
    else
        return d->row[field] == NULL;
@@ -638,44 +654,48 @@ bool QMYSQLResult::isNull(int field)
 
 bool QMYSQLResult::reset (const QString& query)
 {
-    if (!driver() || !driver()->isOpen() || driver()->isOpenError())
+    if (!driver() || !driver()->isOpen() || driver()->isOpenError() || !d->driver)
         return false;
 
-    cleanup();
-    d->preparedQuerys = false;
+    if(d->driver->d->preparedQuerysEnabled && prepare(query)) {
+        d->driver->d->preparedQuerys = true;
+        return exec();
+    }
+    d->driver->d->preparedQuerys = false;
 
-    const QByteArray encQuery(fromUnicode(d->tc, query));
-    if (mysql_real_query(d->mysql, encQuery.data(), encQuery.length())) {
+    const QByteArray encQuery(fromUnicode(d->driver->d->tc, query));
+    if (mysql_real_query(d->driver->d->mysql, encQuery.data(), encQuery.length())) {
         setLastError(qMakeError(QCoreApplication::translate("QMYSQLResult", "Unable to execute query"),
-                     QSqlError::StatementError, d));
+                     QSqlError::StatementError, d->driver->d));
         return false;
     }
-    d->result = mysql_store_result(d->mysql);
-    if (!d->result && mysql_field_count(d->mysql) > 0) {
+    d->result = mysql_store_result(d->driver->d->mysql);
+    if (!d->result && mysql_field_count(d->driver->d->mysql) > 0) {
         setLastError(qMakeError(QCoreApplication::translate("QMYSQLResult", "Unable to store result"),
-                    QSqlError::StatementError, d));
+                    QSqlError::StatementError, d->driver->d));
         return false;
     }
-    int numFields = mysql_field_count(d->mysql);
+    int numFields = mysql_field_count(d->driver->d->mysql);
     setSelect(numFields != 0);
     d->fields.resize(numFields);
-    d->rowsAffected = mysql_affected_rows(d->mysql);
+    d->rowsAffected = mysql_affected_rows(d->driver->d->mysql);
     if (isSelect()) {
         for(int i = 0; i < numFields; i++) {
             MYSQL_FIELD* field = mysql_fetch_field_direct(d->result, i);
             d->fields[i].type = qDecodeMYSQLType(field->type, field->flags);
         }
+        setAt(QSql::BeforeFirstRow);
     }
     setActive(true);
-    return true;
+    return isActive();
 }
 
 int QMYSQLResult::size()
 {
-    if (isSelect())
-        if (d->preparedQuerys)
+    if (d->driver && isSelect())
+        if (d->driver->d->preparedQuerys)
 #if MYSQL_VERSION_ID >= 40108
-            return int(mysql_stmt_num_rows(d->stmt));
+            return mysql_stmt_num_rows(d->stmt);
 #else
             return -1;
 #endif
@@ -692,17 +712,17 @@ int QMYSQLResult::numRowsAffected()
 
 QVariant QMYSQLResult::lastInsertId() const
 {
-    if (!isActive())
+    if (!isActive() || !d->driver)
         return QVariant();
 
-    if (d->preparedQuerys) {
+    if (d->driver->d->preparedQuerys) {
 #if MYSQL_VERSION_ID >= 40108
         quint64 id = mysql_stmt_insert_id(d->stmt);
         if (id)
             return QVariant(id);
 #endif
     } else {
-        quint64 id = mysql_insert_id(d->mysql);
+        quint64 id = mysql_insert_id(d->driver->d->mysql);
         if (id)
             return QVariant(id);
     }
@@ -713,20 +733,20 @@ QSqlRecord QMYSQLResult::record() const
 {
     QSqlRecord info;
     MYSQL_RES *res;
-    if (!isActive() || !isSelect())
+    if (!isActive() || !isSelect() || !d->driver)
         return info;
 
 #if MYSQL_VERSION_ID >= 40108
-    res = d->preparedQuerys ? d->meta : d->result;
+    res = d->driver->d->preparedQuerys ? d->meta : d->result;
 #else
     res = d->result;
 #endif
 
-    if (!mysql_errno(d->mysql)) {
+    if (!mysql_errno(d->driver->d->mysql)) {
         mysql_field_seek(res, 0);
         MYSQL_FIELD* field = mysql_fetch_field(res);
         while(field) {
-            info.append(qToField(field, d->tc));
+            info.append(qToField(field, d->driver->d->tc));
             field = mysql_fetch_field(res);
         }
     }
@@ -736,6 +756,8 @@ QSqlRecord QMYSQLResult::record() const
 
 bool QMYSQLResult::nextResult()
 {
+    if(!d->driver)
+        return false;
 #if MYSQL_VERSION_ID >= 40100 
     setAt(-1);
     setActive(false);
@@ -749,26 +771,26 @@ bool QMYSQLResult::nextResult()
         delete[] d->fields[i].outField;
     d->fields.clear();
 
-    int status = mysql_next_result(d->mysql);
+    int status = mysql_next_result(d->driver->d->mysql);
     if (status > 0) {
         setLastError(qMakeError(QCoreApplication::translate("QMYSQLResult", "Unable to execute next query"),
-                     QSqlError::StatementError, d));
+                     QSqlError::StatementError, d->driver->d));
         return false;
     } else if (status == -1) {
         return false;   // No more result sets
     }
 
-    d->result = mysql_store_result(d->mysql);
-    int numFields = mysql_field_count(d->mysql);
+    d->result = mysql_store_result(d->driver->d->mysql);
+    int numFields = mysql_field_count(d->driver->d->mysql);
     if (!d->result && numFields > 0) {
         setLastError(qMakeError(QCoreApplication::translate("QMYSQLResult", "Unable to store next result"),
-                     QSqlError::StatementError, d));
+                     QSqlError::StatementError, d->driver->d));
         return false;
     }
 
     setSelect(numFields > 0);
     d->fields.resize(numFields);
-    d->rowsAffected = mysql_affected_rows(d->mysql);
+    d->rowsAffected = mysql_affected_rows(d->driver->d->mysql);
 
     if (isSelect()) {
         for (int i = 0; i < numFields; i++) {
@@ -828,8 +850,11 @@ static MYSQL_TIME *toMySqlDate(QDate date, QTime time, QVariant::Type type)
 
 bool QMYSQLResult::prepare(const QString& query)
 {
+    if(!d->driver)
+        return false;
+#if MYSQL_VERSION_ID >= 40108
     cleanup();
-    if (!d->preparedQuerys)
+    if (!d->driver->d->preparedQuerys)
         return QSqlResult::prepare(query);
 
     int r;
@@ -838,14 +863,14 @@ bool QMYSQLResult::prepare(const QString& query)
         return false;
 
     if (!d->stmt)
-        d->stmt = mysql_stmt_init(d->mysql);
+        d->stmt = mysql_stmt_init(d->driver->d->mysql);
     if (!d->stmt) {
         setLastError(qMakeError(QCoreApplication::translate("QMYSQLResult", "Unable to prepare statement"),
-                     QSqlError::StatementError, d));
+                     QSqlError::StatementError, d->driver->d));
         return false;
     }
 
-    const QByteArray encQuery(fromUnicode(d->tc, query));
+    const QByteArray encQuery(fromUnicode(d->driver->d->tc, query));
     r = mysql_stmt_prepare(d->stmt, encQuery.constData(), encQuery.length());
     if (r != 0) {
         setLastError(qMakeStmtError(QCoreApplication::translate("QMYSQLResult",
@@ -860,11 +885,16 @@ bool QMYSQLResult::prepare(const QString& query)
 
     setSelect(d->bindInValues());
     return true;
+#else
+    return false;
+#endif
 }
 
 bool QMYSQLResult::exec()
 {
-    if (!d->preparedQuerys)
+    if (!d->driver)
+        return false;
+    if (!d->driver->d->preparedQuerys)
         return QSqlResult::exec();
     if (!d->stmt)
         return false;
@@ -897,6 +927,7 @@ bool QMYSQLResult::exec()
             nullVector[i] = static_cast<my_bool>(val.isNull());
             currBind->is_null = &nullVector[i];
             currBind->length = 0;
+            currBind->is_unsigned = 0;
 
             switch (val.type()) {
                 case QVariant::ByteArray:
@@ -943,7 +974,6 @@ bool QMYSQLResult::exec()
                     currBind->buffer_type = MYSQL_TYPE_DOUBLE;
                     currBind->buffer = data;
                     currBind->buffer_length = sizeof(double);
-                    currBind->is_unsigned = 0;
                     break;
                 case QVariant::LongLong:
                 case QVariant::ULongLong:
@@ -954,12 +984,11 @@ bool QMYSQLResult::exec()
                     break;
                 case QVariant::String:
                 default: {
-                    QByteArray ba = fromUnicode(d->tc, val.toString());
+                    QByteArray ba = fromUnicode(d->driver->d->tc, val.toString());
                     stringVector.append(ba);
                     currBind->buffer_type = MYSQL_TYPE_STRING;
                     currBind->buffer = const_cast<char *>(ba.constData());
                     currBind->buffer_length = ba.length();
-                    currBind->is_unsigned = 0;
                     break; }
             }
         }
@@ -1229,6 +1258,7 @@ bool QMYSQLDriver::open(const QString& db,
         setLastError(qMakeError(tr("Unable to connect"),
                      QSqlError::ConnectionError, d));
         mysql_close(d->mysql);
+        d->mysql = NULL;
         setOpenError(true);
         return false;
     }
@@ -1248,6 +1278,11 @@ bool QMYSQLDriver::open(const QString& db,
     d->preparedQuerysEnabled = false;
 #endif
 
+#ifndef QT_NO_THREAD
+    mysql_thread_init();
+#endif
+
+
     setOpen(true);
     setOpenError(false);
     return true;
@@ -1256,7 +1291,11 @@ bool QMYSQLDriver::open(const QString& db,
 void QMYSQLDriver::close()
 {
     if (isOpen()) {
+#ifndef QT_NO_THREAD
+        mysql_thread_end();
+#endif
         mysql_close(d->mysql);
+        d->mysql = NULL;
         setOpen(false);
         setOpenError(false);
     }
@@ -1425,9 +1464,13 @@ QString QMYSQLDriver::formatValue(const QSqlField &field, bool trimStrings) cons
 QString QMYSQLDriver::escapeIdentifier(const QString &identifier, IdentifierType) const
 {
     QString res = identifier;
-    res.prepend(QLatin1Char('`')).append(QLatin1Char('`'));
-    res.replace(QLatin1Char('.'), QLatin1String("`.`"));
+    if(!identifier.isEmpty() && identifier.left(1) != QString(QLatin1Char('`')) && identifier.right(1) != QString(QLatin1Char('`')) ) {
+        res.prepend(QLatin1Char('`')).append(QLatin1Char('`'));
+        res.replace(QLatin1Char('.'), QLatin1String("`.`"));
+    }
     return res;
 }
 
 QT_END_NAMESPACE
+
+#include "qsql_mysql.moc"
\ No newline at end of file
