@@ -1,9 +1,9 @@
 /****************************************************************************
 **
 ** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
-** Contact: Qt Software Information (qt-info@nokia.com)
+** Contact: Nokia Corporation (qt-info@nokia.com)
 **
-** This file is part of the QtSVG module of the Qt Toolkit.
+** This file is part of the QtSvg module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial Usage
@@ -34,7 +34,7 @@
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
 ** If you are unsure which license is appropriate for your use, please
-** contact the sales department at qt-sales@nokia.com.
+** contact the sales department at http://www.qtsoftware.com/contact.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -261,7 +261,7 @@ public:
                 constantAlpha &= (stops.at(i).second.alpha() == alpha);
 
             if (!constantAlpha) {
-                const qreal spacing = 0.02;
+                const qreal spacing = qreal(0.02);
                 QGradientStops newStops;
                 QRgb fromColor = PREMUL(stops.at(0).second.rgba());
                 QRgb toColor;
@@ -516,7 +516,34 @@ public:
     \brief The QSvgGenerator class provides a paint device that is used to create SVG drawings.
     \reentrant
 
-    \sa QSvgRenderer, QSvgWidget
+    This paint device represents a Scalable Vector Graphics (SVG) drawing. Like QPrinter, it is
+    designed as a write-only device that generates output in a specific format.
+
+    To write an SVG file, you first need to configure the output by setting the \l fileName
+    or \l outputDevice properties. It is usually necessary to specify the size of the drawing
+    by setting the \l size property, and in some cases where the drawing will be included in
+    another, the \l viewBox property also needs to be set.
+
+    \snippet examples/painting/svggenerator/window.cpp configure SVG generator
+
+    Other meta-data can be specified by setting the \a title, \a description and \a resolution
+    properties.
+
+    As with other QPaintDevice subclasses, a QPainter object is used to paint onto an instance
+    of this class:
+
+    \snippet examples/painting/svggenerator/window.cpp begin painting
+    \dots
+    \snippet examples/painting/svggenerator/window.cpp end painting
+
+    Painting is performed in the same way as for any other paint device. However,
+    it is necessary to use the QPainter::begin() and \l{QPainter::}{end()} to
+    explicitly begin and end painting on the device.
+
+    The \l{SVG Generator Example} shows how the same painting commands can be used
+    for painting a widget and writing an SVG file.
+
+    \sa QSvgRenderer, QSvgWidget, {About SVG}
 */
 
 /*!
