@@ -1,9 +1,9 @@
 /****************************************************************************
 **
 ** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
-** Contact: Qt Software Information (qt-info@nokia.com)
+** Contact: Nokia Corporation (qt-info@nokia.com)
 **
-** This file is part of the QtXMLPatterns module of the Qt Toolkit.
+** This file is part of the QtXmlPatterns module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial Usage
@@ -34,7 +34,7 @@
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
 ** If you are unsure which license is appropriate for your use, please
-** contact the sales department at qt-sales@nokia.com.
+** contact the sales department at http://www.qtsoftware.com/contact.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -943,10 +943,13 @@ void QXmlQuery::setFocus(const QXmlItem &item)
 
 /**
  * This function should be a private member function of QXmlQuery,
- * but we don't dare that due to our weird compilers. */
+ * but we don't dare that due to our weird compilers.
+ * @internal
+ * @relates QXmlQuery
+ */
 template<typename TInputType>
-static bool setFocusHelper(QXmlQuery *const queryInstance,
-                           const TInputType &focusValue)
+bool setFocusHelper(QXmlQuery *const queryInstance,
+                    const TInputType &focusValue)
 {
     /* We call resourceLoader(), so we have ensured that we have a resourceLoader
      * that we will share in our copy. */
@@ -954,6 +957,9 @@ static bool setFocusHelper(QXmlQuery *const queryInstance,
 
     QXmlQuery focusQuery(*queryInstance);
 
+    /* Now we use the same, so we own the loaded document. */
+    focusQuery.d->m_resourceLoader = queryInstance->d->m_resourceLoader;
+
     /* The copy constructor doesn't allow us to copy an existing QXmlQuery and
      * changing the language at the same time so we need to use private API. */
     focusQuery.d->queryLanguage = QXmlQuery::XQuery10;
@@ -965,6 +971,8 @@ static bool setFocusHelper(QXmlQuery *const queryInstance,
 
     QXmlResultItems focusResult;
 
+    queryInstance->d->m_resourceLoader = focusQuery.d->m_resourceLoader;
+
     focusQuery.evaluateTo(&focusResult);
     const QXmlItem focusItem(focusResult.next());
 
