@@ -1,7 +1,7 @@
 /****************************************************************************
 **
 ** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
-** Contact: Qt Software Information (qt-info@nokia.com)
+** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the Qt Designer of the Qt Toolkit.
 **
@@ -34,7 +34,7 @@
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
 ** If you are unsure which license is appropriate for your use, please
-** contact the sales department at qt-sales@nokia.com.
+** contact the sales department at http://www.qtsoftware.com/contact.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -202,12 +202,15 @@ namespace qdesigner_internal {
             isContainer = widgetItem->isContainer();
         }
 
+        // We might encounter temporary states with no layouts when re-layouting.
+        // Just default to Widget handling for the moment.
         if (isQLayoutWidget(w)) {
-            m_type = LayoutWidget;
-            const QLayout *layout = w->layout();
-            m_managedLayoutType = LayoutInfo::layoutType(ctx.core, layout);
-            m_className = QLatin1String(layout->metaObject()->className());
-            m_objectName = layout->objectName();
+            if (const QLayout *layout = w->layout()) {
+                m_type = LayoutWidget;
+                m_managedLayoutType = LayoutInfo::layoutType(ctx.core, layout);
+                m_className = QLatin1String(layout->metaObject()->className());
+                m_objectName = layout->objectName();
+            }
             return;
         }
 
