@@ -1,43 +1,41 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2008 Trolltech ASA. All rights reserved.
+** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the tools applications of the Qt Toolkit.
 **
-** This file may be used under the terms of the GNU General Public
-** License versions 2.0 or 3.0 as published by the Free Software
-** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file.  Alternatively you may (at
-** your option) use any later version of the GNU General Public
-** License if such license has been publicly approved by Trolltech ASA
-** (or its successors, if any) and the KDE Free Qt Foundation. In
-** addition, as a special exception, Trolltech gives you certain
-** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.2, which can be found at
-** http://www.trolltech.com/products/qt/gplexception/ and in the file
-** GPL_EXCEPTION.txt in this package.
+** $QT_BEGIN_LICENSE:LGPL$
+** Commercial Usage
+** Licensees holding valid Qt Commercial licenses may use this file in
+** accordance with the Qt Commercial License Agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Nokia.
 **
-** Please review the following information to ensure GNU General
-** Public Licensing requirements will be met:
-** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
-** you are unsure which license is appropriate for your use, please
-** review the following information:
-** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
-** or contact the sales department at sales@trolltech.com.
+** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Trolltech, as the sole
-** copyright holder for Qt Designer, grants users of the Qt/Eclipse
-** Integration plug-in the right for the Qt/Eclipse Integration to
-** link to functionality provided by Qt Designer and its related
-** libraries.
+** In addition, as a special exception, Nokia gives you certain
+** additional rights. These rights are described in the Nokia Qt LGPL
+** Exception version 1.0, included in the file LGPL_EXCEPTION.txt in this
+** package.
 **
-** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
-** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
-** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
-** granted herein.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3.0 as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU General Public License version 3.0 requirements will be
+** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
-** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+** If you are unsure which license is appropriate for your use, please
+** contact the sales department at http://www.qtsoftware.com/contact.
+** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
@@ -59,9 +57,9 @@ QT_BEGIN_NAMESPACE
 Node::~Node()
 {
     if (par)
-	par->removeChild(this);
-    if (rel)        
-	rel->removeRelated(this);
+        par->removeChild(this);
+    if (rel)
+        rel->removeRelated(this);
 }
 
 /*!
@@ -69,8 +67,8 @@ Node::~Node()
 void Node::setDoc(const Doc& doc, bool replace)
 {
     if (!d.isEmpty() && !replace) {
-	doc.location().warning(tr("Overrides a previous doc"));
-	d.location().warning(tr("(The previous doc is here)"));
+        doc.location().warning(tr("Overrides a previous doc"));
+        d.location().warning(tr("(The previous doc is here)"));
     }
     d = doc;
 }
@@ -87,7 +85,7 @@ Node::Node(Type type, InnerNode *parent, const QString& name)
       nam(name)
 {
     if (par)
-	par->addChild(this);
+        par->addChild(this);
 }
 
 /*!
@@ -109,7 +107,7 @@ void Node::setUrl(const QString &url)
 void Node::setRelates(InnerNode *pseudoParent)
 {
     if (rel)
-	    rel->removeRelated(this);
+            rel->removeRelated(this);
     rel = pseudoParent;
     pseudoParent->related.append(this);
 }
@@ -130,7 +128,7 @@ Node::Status Node::inheritedStatus() const
 {
     Status parentStatus = Commendable;
     if (par)
-	parentStatus = par->inheritedStatus();
+        parentStatus = par->inheritedStatus();
     return (Status)qMin((int)sta, (int)parentStatus);
 }
 
@@ -139,7 +137,7 @@ Node::Status Node::inheritedStatus() const
 Node::ThreadSafeness Node::threadSafeness() const
 {
     if (par && saf == par->inheritedThreadSafeness())
-	return UnspecifiedSafeness;
+        return UnspecifiedSafeness;
     return saf;
 }
 
@@ -148,7 +146,7 @@ Node::ThreadSafeness Node::threadSafeness() const
 Node::ThreadSafeness Node::inheritedThreadSafeness() const
 {
     if (par && saf == UnspecifiedSafeness)
-	return par->inheritedThreadSafeness();
+        return par->inheritedThreadSafeness();
     return saf;
 }
 
@@ -192,15 +190,15 @@ Node *InnerNode::findNode(const QString& name)
 Node *InnerNode::findNode(const QString& name, Type type)
 {
     if (type == Function) {
-	return primaryFunctionMap.value(name);
+        return primaryFunctionMap.value(name);
     }
     else {
-	Node *node = childMap.value(name);
-	if (node && node->type() == type) {
-	    return node;
+        Node *node = childMap.value(name);
+        if (node && node->type() == type) {
+            return node;
         }
         else {
-	    return 0;
+            return 0;
         }
     }
 }
@@ -270,8 +268,8 @@ void InnerNode::setOverload(const FunctionNode *func, bool overlode)
 void InnerNode::makeUndocumentedChildrenInternal()
 {
     foreach (Node *child, childNodes()) {
-	if (child->doc().isEmpty()) {
-	    child->setAccess(Node::Private);
+        if (child->doc().isEmpty()) {
+            child->setAccess(Node::Private);
             child->setStatus(Node::Internal);
         }
     }
@@ -283,7 +281,7 @@ void InnerNode::normalizeOverloads()
 {
     QMap<QString, Node *>::Iterator p1 = primaryFunctionMap.begin();
     while (p1 != primaryFunctionMap.end()) {
-	FunctionNode *primaryFunc = (FunctionNode *) *p1;
+        FunctionNode *primaryFunc = (FunctionNode *) *p1;
         if (secondaryFunctionMap.contains(primaryFunc->name()) &&
             (primaryFunc->status() != Commendable ||
              primaryFunc->access() == Private)) {
@@ -305,10 +303,10 @@ void InnerNode::normalizeOverloads()
                     secondaryFunctionMap[primaryFunc->name()].replace(index, primaryFunc);
                     break;
                 }
-		++s;
-	    }
+                ++s;
+            }
         }
-	++p1;
+        ++p1;
     }
 
     QMap<QString, Node *>::ConstIterator p = primaryFunctionMap.begin();
@@ -402,7 +400,7 @@ const EnumNode *InnerNode::findEnumNodeForValue(const QString &enumValue) const
 {
     foreach (const Node *node, enumChildren) {
         const EnumNode *enume = static_cast<const EnumNode *>(node);
-	if (enume->hasItem(enumValue))
+        if (enume->hasItem(enumValue))
             return enume;
     }
     return 0;
@@ -414,10 +412,10 @@ int InnerNode::overloadNumber(const FunctionNode *func) const
 {
     Node *node = (Node *) func;
     if (primaryFunctionMap[func->name()] == node) {
-	return 1;
+        return 1;
     }
     else {
-	return secondaryFunctionMap[func->name()].indexOf(node) + 2;
+        return secondaryFunctionMap[func->name()].indexOf(node) + 2;
     }
 }
 
@@ -426,10 +424,10 @@ int InnerNode::overloadNumber(const FunctionNode *func) const
 int InnerNode::numOverloads(const QString& funcName) const
 {
     if (primaryFunctionMap.contains(funcName)) {
-	return secondaryFunctionMap[funcName].count() + 1;
+        return secondaryFunctionMap[funcName].count() + 1;
     }
     else {
-	return 0;
+        return 0;
     }
 }
 
@@ -441,7 +439,7 @@ NodeList InnerNode::overloads(const QString &funcName) const
     Node *primary = primaryFunctionMap.value(funcName);
     if (primary) {
         result << primary;
-	result += secondaryFunctionMap[funcName];
+        result += secondaryFunctionMap[funcName];
     }
     return result;
 }
@@ -473,16 +471,16 @@ void InnerNode::setIncludes(const QStringList& includes)
 bool InnerNode::isSameSignature(const FunctionNode *f1, const FunctionNode *f2)
 {
     if (f1->parameters().count() != f2->parameters().count())
-	return false;
+        return false;
     if (f1->isConst() != f2->isConst())
-	return false;
+        return false;
 
     QList<Parameter>::ConstIterator p1 = f1->parameters().begin();
     QList<Parameter>::ConstIterator p2 = f2->parameters().begin();
     while (p2 != f2->parameters().end()) {
 	if ((*p1).hasType() && (*p2).hasType()) {
             if ((*p1).rightType() != (*p2).rightType())
-		return false;
+                return false;
 
             QString t1 = p1->leftType();
             QString t2 = p2->leftType();
@@ -496,9 +494,9 @@ bool InnerNode::isSameSignature(const FunctionNode *f1, const FunctionNode *f2)
             */
             if (t1 != t2 && t1 != (f2->parent()->name() + "::" + t2))
                 return false;
-	}
-	++p1;
-	++p2;
+        }
+        ++p1;
+        ++p2;
     }
     return true;
 }
@@ -509,8 +507,8 @@ void InnerNode::addChild(Node *child)
 {
     children.append(child);
     if (child->type() == Function) {
-	FunctionNode *func = (FunctionNode *) child;
-	if (!primaryFunctionMap.contains(func->name())) {
+        FunctionNode *func = (FunctionNode *) child;
+        if (!primaryFunctionMap.contains(func->name())) {
             primaryFunctionMap.insert(func->name(), func);
 	}
         else {
@@ -519,7 +517,7 @@ void InnerNode::addChild(Node *child)
 	}
     }
     else {
-	if (child->type() == Enum)
+        if (child->type() == Enum)
             enumChildren.append(child);
 	childMap.insert(child->name(), child);
     }
@@ -546,6 +544,9 @@ void InnerNode::removeChild(Node *child)
         else {
 	    secs.removeAll(child);
 	}
+        QMap<QString, Node *>::Iterator ent = childMap.find( child->name() );
+        if ( *ent == child )
+            childMap.erase( ent );
     }
     else {
 	QMap<QString, Node *>::Iterator ent = childMap.find(child->name());
@@ -613,8 +614,8 @@ QString Node::moduleName() const
 /*!
  */
 void InnerNode::removeRelated(Node *pseudoChild)
-{       
-    related.removeAll(pseudoChild);    
+{
+    related.removeAll(pseudoChild);
 }
 
 /*!
@@ -663,7 +664,7 @@ ClassNode::ClassNode(InnerNode *parent, const QString& name)
  */
 void ClassNode::addBaseClass(Access access,
                              ClassNode *node,
-			     const QString &dataTypeWithTemplateArgs)
+                             const QString &dataTypeWithTemplateArgs)
 {
     bas.append(RelatedClass(access, node, dataTypeWithTemplateArgs));
     node->der.append(RelatedClass(access, this));
@@ -729,13 +730,13 @@ QString FakeNode::fullTitle() const
             return title();
     }
     else if (sub == HeaderFile) {
-	if (title().isEmpty())
-	    return name();
-	else
-	    return name() + " - " + title();
+        if (title().isEmpty())
+            return name();
+        else
+            return name() + " - " + title();
     }
     else {
-	return title();
+        return title();
     }
 }
 
@@ -747,7 +748,7 @@ QString FakeNode::subTitle() const
         return stle;
 
     if (sub == File) {
-	if (title().isEmpty() && name().contains("/"))
+        if (title().isEmpty() && name().contains("/"))
             return name();
     }
     return QString();
@@ -777,10 +778,10 @@ void EnumNode::addItem(const EnumItem& item)
 Node::Access EnumNode::itemAccess(const QString &name) const
 {
     if (doc().omitEnumItemNames().contains(name)) {
-	return Private;
+        return Private;
     }
     else {
-	return Public;
+        return Public;
     }
 }
 
@@ -923,8 +924,8 @@ QStringList FunctionNode::parameterNames() const
     QStringList names;
     QList<Parameter>::ConstIterator p = parameters().begin();
     while (p != parameters().end()) {
-	names << (*p).name();
-	++p;
+        names << (*p).name();
+        ++p;
     }
     return names;
 }
@@ -993,11 +994,11 @@ bool PropertyNode::fromTrool(Trool troolean, bool defaultValue)
 {
     switch (troolean) {
     case Trool_True:
-	return true;
+        return true;
     case Trool_False:
-	return false;
+        return false;
     default:
-	return defaultValue;
+        return defaultValue;
     }
 }
 
