@@ -1,43 +1,41 @@
 /****************************************************************************
 **
-** Copyright (C) 2007-2008 Trolltech ASA. All rights reserved.
+** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the tools applications of the Qt Toolkit.
 **
-** This file may be used under the terms of the GNU General Public
-** License versions 2.0 or 3.0 as published by the Free Software
-** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file.  Alternatively you may (at
-** your option) use any later version of the GNU General Public
-** License if such license has been publicly approved by Trolltech ASA
-** (or its successors, if any) and the KDE Free Qt Foundation. In
-** addition, as a special exception, Trolltech gives you certain
-** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.2, which can be found at
-** http://www.trolltech.com/products/qt/gplexception/ and in the file
-** GPL_EXCEPTION.txt in this package.
+** $QT_BEGIN_LICENSE:LGPL$
+** Commercial Usage
+** Licensees holding valid Qt Commercial licenses may use this file in
+** accordance with the Qt Commercial License Agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Nokia.
 **
-** Please review the following information to ensure GNU General
-** Public Licensing requirements will be met:
-** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
-** you are unsure which license is appropriate for your use, please
-** review the following information:
-** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
-** or contact the sales department at sales@trolltech.com.
+** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Trolltech, as the sole
-** copyright holder for Qt Designer, grants users of the Qt/Eclipse
-** Integration plug-in the right for the Qt/Eclipse Integration to
-** link to functionality provided by Qt Designer and its related
-** libraries.
+** In addition, as a special exception, Nokia gives you certain
+** additional rights. These rights are described in the Nokia Qt LGPL
+** Exception version 1.0, included in the file LGPL_EXCEPTION.txt in this
+** package.
 **
-** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
-** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
-** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
-** granted herein.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3.0 as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU General Public License version 3.0 requirements will be
+** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
-** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+** If you are unsure which license is appropriate for your use, please
+** contact the sales department at http://www.qtsoftware.com/contact.
+** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
@@ -100,7 +98,7 @@ QT_BEGIN_NAMESPACE
  \section1 Changing Parsing Convention
 
  QApplicationArgumentParser by default parses the command line in the style
- of Qt's utilities, where arguments are preceeded by a single dash, and identified
+ of Qt's utilities, where arguments are preceded by a single dash, and identified
  by a single name. However, in some cases it might be of interest to parse
  another style, such as the well-established UNIX \c getopt convention(\c -l
  and \c --long).
@@ -126,6 +124,7 @@ class QApplicationArgumentParserPrivate
     Q_DECLARE_TR_FUNCTIONS(QApplicationArgumentParserPrivate)
 public:
     // TODO Isn't it like ten times better with QHash<QApplicationArgument, QList<QVariant> >?
+    // TODO test QApplicationArgument::nameless()
     typedef QList<QPair<QApplicationArgument, QVariant> > UsedList;
 
     /*!
@@ -139,7 +138,7 @@ public:
         Q_ASSERT(!aInput.isEmpty());
     }
 
-    QApplicationArgument nextNamelessArgument();
+    QApplicationArgument nextNamelessArgument() const;
     static QStringList argumentsFromLocal(const int argc, const char *const *const argv);
 
     bool error(const QString &message);
@@ -152,15 +151,20 @@ public:
     static inline bool isBuiltinVariant(const int type);
     void displayVersion() const;
     void displayHelp() const;
+    void parseNameless();
+    bool parseNamelessArguments(const QString &in);
 
     QApplicationArgumentParser::ExitCode    exitCode;
     const QStringList                       input;
 
     /*!
-      Since the QString is the QApplicationArgument name anyway, why
+      Since the QString is QApplicationArgument::name() anyway, why
       not use a QSet?
      */
     QHash<QString, QApplicationArgument>    declaredArguments;
+
+    QList<QApplicationArgument>             declaredNamelessArguments;
+
     UsedList                                usedArguments;
     QString                                 applicationDescription;
     QString                                 applicationVersion;
@@ -175,9 +179,34 @@ private:
     static QList<QApplicationArgument> builtinArguments();
 };
 
-QApplicationArgument QApplicationArgumentParserPrivate::nextNamelessArgument()
+QApplicationArgument QApplicationArgumentParserPrivate::nextNamelessArgument() const
 {
-    return declaredArguments.value(QString());
+    /* Count how many nameless arguments we have so far. */
+    int count = 0;
+
+    for(int i = 0; i < usedArguments.count(); ++i)
+    {
+        if(usedArguments.at(i).first.isNameless())
+            ++count;
+    }
+
+    /* TODO this doesn't work for arguments that have more than one
+     * mandatory value(e.g nameless ones), since several values should
+     * then only count for one argument. */
+    for(int i = 0; i < declaredNamelessArguments.count(); ++i)
+    {
+        if(count)
+        {
+            /* Skip the ones we already have processed. */
+            --count;
+            continue;
+        }
+
+        if(declaredNamelessArguments.at(i).isNameless())
+            return declaredNamelessArguments.at(i);
+    }
+
+    return QApplicationArgument();
 }
 
 int QApplicationArgumentParserPrivate::count(const QApplicationArgument &arg) const
@@ -270,7 +299,7 @@ void QApplicationArgumentParserPrivate::displayVersion() const
 {
     QTextStream out(stderr);
 
-    out << tr("%1 version %2 using Qt %3").arg(QCoreApplication::applicationName(), applicationVersion, qVersion())
+    out << tr("%1 version %2 using Qt %3").arg(QCoreApplication::applicationName(), applicationVersion, QString::fromAscii(qVersion()))
         << endl;
 }
 
@@ -287,7 +316,7 @@ public:
     inline bool operator()(const QApplicationArgument &o1,
                            const QApplicationArgument &o2) const
     {
-        return o1.name().compare(o2.name()) > 0;
+        return o1.name().compare(o2.name()) < 0;
     }
 };
 
@@ -315,10 +344,9 @@ void QApplicationArgumentParserPrivate::displayHelp() const
     QList<QApplicationArgument> args(declaredArguments.values());
     args += builtinArguments();
 
-    const int argCount = args.count();
     /* Sort them, such that we get the nameless options at the end, and it
      * generally looks tidy. */
-    qSort(args.begin(), args.end());
+    qSort(args);
 
     /* This is the basic approach:
      * Switches:
@@ -326,6 +354,9 @@ void QApplicationArgumentParserPrivate::displayHelp() const
      * Value arguments:
      *  -name <name-of-value-type> description
      *
+     * Nameless arguments
+     *  name <type> description
+     *
      * It all line-wraps at OutputWidth and the description is indented,
      * where the highest indent is the length of the name plus length of the name
      * of the type. */
@@ -333,9 +364,17 @@ void QApplicationArgumentParserPrivate::displayHelp() const
     /* First we find the name with the largest width. */
     int maxWidth = 0;
 
-    for(int i = 0; i < argCount; ++i)
+    QList<QApplicationArgument> nameless(declaredNamelessArguments);
+    qSort(nameless);
+
+    /* Note, here the nameless arguments appear last, but are sorted
+     * with themselves. */
+    QList<QApplicationArgument> allArgs(args + nameless);
+    const int allArgsCount = allArgs.count();
+
+    for(int i = 0; i < allArgsCount; ++i)
     {
-        const QApplicationArgument &at = args.at(i);
+        const QApplicationArgument &at = allArgs.at(i);
         const int nameLength = at.name().length();
         const QString typeName(q_ptr->typeToName(at));
         const int typeNameLength = typeName.length();
@@ -347,9 +386,10 @@ void QApplicationArgumentParserPrivate::displayHelp() const
     out << endl
         << QString(IndentPadding, QLatin1Char(' '))
         << QCoreApplication::applicationName()
-        << " -- "
+        << QLatin1String(" -- ")
         << applicationDescription
         << endl;
+    // TODO synopsis
 
     /* One extra so we get some space between the overview and the options. */
     out << endl;
@@ -357,9 +397,9 @@ void QApplicationArgumentParserPrivate::displayHelp() const
     const int indentWidth = maxWidth + 3;
 
     /* Ok, print them out. */
-    for(int i = 0; i < argCount; ++i)
+    for(int i = 0; i < allArgsCount; ++i)
     {
-        const QApplicationArgument &at = args.at(i);
+        const QApplicationArgument &at = allArgs.at(i);
         /* "  -name ". Indent a bit first, inspired by Qt's moc. */
         const QString &name = at.name();
         QString prolog(QLatin1String("  "));
@@ -367,8 +407,13 @@ void QApplicationArgumentParserPrivate::displayHelp() const
         /* We have a special case for the single dash. */
         if(name == QChar::fromLatin1('-'))
             prolog.append(name);
-        else if(!name.isEmpty())
-           prolog.append(QLatin1Char('-') + name + QLatin1Char(' '));
+        else
+        {
+            if(!at.isNameless())
+                prolog.append(QLatin1Char('-'));
+
+             prolog.append(name + QLatin1Char(' '));
+        }
 
         if(at.type() != QVariant::Invalid)
         {
@@ -551,7 +596,10 @@ QApplicationArgumentParser::~QApplicationArgumentParser()
  */
 void QApplicationArgumentParser::addArgument(const QApplicationArgument &argument)
 {
-    d->declaredArguments.insert(argument.name(), argument);
+    if(argument.isNameless())
+        d->declaredNamelessArguments.append(argument);
+    else
+        d->declaredArguments.insert(argument.name(), argument);
 }
 
 /*!
@@ -580,6 +628,21 @@ QList<QApplicationArgument> QApplicationArgumentParser::declaredArguments() cons
     return d->declaredArguments.values();
 }
 
+bool QApplicationArgumentParserPrivate::parseNamelessArguments(const QString &in)
+{
+    /* It's a nameless options, such as simply "value". */
+    const QApplicationArgument nameless(nextNamelessArgument());
+
+    const QVariant val(q_ptr->convertToValue(nameless, in));
+    if(val.isValid())
+    {
+        usedArguments.append(qMakePair(nameless, val));
+        return true;
+    }
+    else
+        return false; // TODO error msg?
+}
+
 /*!
  Parses input() together with declaredArguments() and returns \c false if the caller 
  should exit immediately, which is the case of which an error was encountered or
@@ -612,19 +675,15 @@ bool QApplicationArgumentParser::parse()
 
             for(; i < inputCount; ++i)
             {
-                const QApplicationArgument nameless(d->nextNamelessArgument());
-
-                QVariant val(convertToValue(nameless, d->input.at(i)));
-                if(val.isValid())
-                    d->usedArguments.append(qMakePair(nameless, val));
-                else
+                if(!d->parseNamelessArguments(d->input.at(i)))
                     return false;
+                /* Process nameless options. Have code for this elsewhere, factor it out. */
             }
 
             break;
         }
 
-        if(in.startsWith(sep)) /* Is it "-name"? */
+        if(in.startsWith(sep)) /* It is "-name". */
         {
             const QString name(in.mid(1));
 
@@ -683,18 +742,16 @@ bool QApplicationArgumentParser::parse()
                     return false; // TODO error msg?
             }
         }
-
-        /* It's a nameless options, such as simply "value". */
-        const QApplicationArgument nameless(d->nextNamelessArgument());
-
-        const QVariant val(convertToValue(nameless, in));
-        if(val.isValid())
-            d->usedArguments.append(qMakePair(nameless, val));
         else
-                return false; // TODO error msg?
+        {
+            if(!d->parseNamelessArguments(in))
+                return false;
+        }
     }
 
-    const QList<QApplicationArgument> declaredArguments(d->declaredArguments.values());
+    /* Check that all arguments that have been declared as mandatory, are actually
+     * specified. */
+    const QList<QApplicationArgument> declaredArguments(d->declaredArguments.values() + d->declaredNamelessArguments);
     const int len = declaredArguments.count();
     for(int i = 0; i < len; ++i)
     {
@@ -742,7 +799,7 @@ bool QApplicationArgumentParser::parse()
 
  The default implementation uses QVariant::canConvert() and QVariant::convert() for doing conversions.
 
- QApplicationArgumentParser can be subclassed and this function subsequently overriden, to handle custom types.
+ QApplicationArgumentParser can be subclassed and this function subsequently overridden, to handle custom types.
 
  If \a input isn't valid input for \a argument, this function returns a default constructed
  QVariant.
@@ -850,7 +907,8 @@ QVariant QApplicationArgumentParser::defaultValue(const QApplicationArgument &ar
  */
 int QApplicationArgumentParser::count(const QApplicationArgument &argument) const
 {
-    Q_ASSERT_X(d->declaredArguments.contains(argument.name()), Q_FUNC_INFO,
+    Q_ASSERT_X(d->declaredArguments.contains(argument.name()) ||
+               d->declaredNamelessArguments.contains(argument), Q_FUNC_INFO,
                "The argument isn't known to the parser. Has addArgument() been called?");
     return d->count(argument);
 }
@@ -863,7 +921,8 @@ int QApplicationArgumentParser::count(const QApplicationArgument &argument) cons
  */
 bool QApplicationArgumentParser::has(const QApplicationArgument &argument) const
 {
-    Q_ASSERT_X(d->declaredArguments.contains(argument.name()), Q_FUNC_INFO,
+    Q_ASSERT_X(d->declaredArguments.contains(argument.name()) ||
+               d->declaredNamelessArguments.contains(argument), Q_FUNC_INFO,
                "The argument isn't known to the parser. Has addArgument() been called?");
     return d->contains(argument);
 }
@@ -875,7 +934,8 @@ bool QApplicationArgumentParser::has(const QApplicationArgument &argument) const
  */
 QVariant QApplicationArgumentParser::value(const QApplicationArgument &argument) const
 {
-    Q_ASSERT_X(d->declaredArguments.contains(argument.name()), Q_FUNC_INFO,
+    Q_ASSERT_X(d->declaredArguments.contains(argument.name()) ||
+               d->declaredNamelessArguments.contains(argument), Q_FUNC_INFO,
                "The argument isn't known to the parser. Has addArgument() been called?");
 
     const int len = d->usedArguments.count();
@@ -895,7 +955,9 @@ QVariant QApplicationArgumentParser::value(const QApplicationArgument &argument)
  */
 QVariantList QApplicationArgumentParser::values(const QApplicationArgument &argument) const
 {
-    Q_ASSERT_X(d->declaredArguments.contains(argument.name()), Q_FUNC_INFO,
+    Q_ASSERT_X(d->declaredArguments.contains(argument.name()) ||
+               d->declaredNamelessArguments.contains(argument),
+               Q_FUNC_INFO,
                "The argument isn't known to the parser. Has addArgument() been called?");
 
     const int len = d->usedArguments.count();
